namespace Ato.CD.Inbound.TRT202002
{

    using System.Linq;
    using System.Collections.Generic;
    using VaTS;
    using DataContracts;
    using Ato.EN.IntegrationServices.CodeGenerationTRT;
    using Ato.EN.IntegrationServices.CodeGenerationPSS;
    using Ato.EN.IntegrationServices.CodeGenerationRS;
    using System.Text.RegularExpressions;
    using Ato.EN.IntegrationServices.CodeGenerationIDS;
    using Ato.EN.IntegrationServices.CodeGenerationIEE;
    using Ato.EN.IntegrationServices.CodeGenerationLS;
    using Ato.EN.IntegrationServices.CodeGenerationCGTS;
    using Ato.EN.IntegrationServices.CodeGenerationFTER;

    internal class CrossFormValidator
    {
        private TRT2020 ParentDocument { get; }
        private BusinessDocumentCollection ChildDocuments { get; }

        private IEnumerable<IDS2020> IDSChildDocuments { get; }
        private IEnumerable<LS2017> LSChildDocuments { get; }
        private IEnumerable<CGTS2018> CGTSChildDocuments { get; }
        private IEnumerable<PSS2018> PSSChildDocuments { get; }
        private IEnumerable<RS2018> RSChildDocuments { get; }
        private IEnumerable<FTER2018> FTERChildDocuments { get; }
        private IEnumerable<IEE2018> IEEChildDocuments { get; }

        public CrossFormValidator(TRT2020 report, BusinessDocumentCollection childDocuments)
        {
            ParentDocument = report;
            ChildDocuments = childDocuments ?? new BusinessDocumentCollection();

            IDSChildDocuments = ChildDocuments
                       .Where(s => s.DocumentName.Equals(Schedule.IDS.ToString()))
                       .Select(p => (IDS2020)p.ConsumedReport);

            LSChildDocuments = ChildDocuments
                       .Where(s => s.DocumentName.Equals(Schedule.LS.ToString()))
                       .Select(p => (LS2017)p.ConsumedReport);

            CGTSChildDocuments = ChildDocuments
                       .Where(s => s.DocumentName.Equals(Schedule.CGTS.ToString()))
                       .Select(p => (CGTS2018)p.ConsumedReport);

            PSSChildDocuments = ChildDocuments
                       .Where(s => s.DocumentName.Equals(Schedule.PSS.ToString()))
                       .Select(p => (PSS2018)p.ConsumedReport);

            RSChildDocuments = ChildDocuments
                       .Where(s => s.DocumentName.Equals(Schedule.RS.ToString()))
                       .Select(p => (RS2018)p.ConsumedReport);

            FTERChildDocuments = ChildDocuments
                       .Where(s => s.DocumentName.Equals(Schedule.FTER.ToString()))
                       .Select(p => (FTER2018)p.ConsumedReport);

            IEEChildDocuments = ChildDocuments
                       .Where(s => s.DocumentName.Equals(Schedule.IEE.ToString()))
                       .Select(p => (IEE2018)p.ConsumedReport);
        }

        public List<ProcessMessageDocument> ValidateCrossFormRules()
        {
            var response = new List<ProcessMessageDocument>();
            //LS 
            VRATOTRT432429(response); 
            VRATOTRT432210(response);

            //CGTS
            VRATOTRT432042(response);

            // PSS
            VRATOTRT432043(response);
            VRATOTRT432044(response);
            VRATOTRT432051(response);
            VRATOTRT432088(response);

            //RS
            VRATOTRT432115(response);
            VRATOTRT432116(response);
            VRATOTRT432117(response);
            VRATOTRT432120(response);
            VRATOTRT432122(response);
            VRATOTRT432124(response);
            VRATOTRT432126(response);

            //TRTAMIS - No Rule 

            //FTER
            VRATOTRT432017(response);

            //IEE
            VRATOTRT432020(response);
            VRATOTRT432011(response);
            VRATOTRT432019(response);
            VRATOGEN430008(response);

            //IDS 
            VRATOGEN438002(response);
            VRATOTRT432646(response);
            VRATOTRT432647(response); 
            VRATOTRT432650(response);
            VRATOTRT432651(response);
            VRATOTRT432652(response); 
            VRATOTRT432653(response); 
            VRATOTRT432655(response); 
            VRATOTRT432648(response);
            VRATOTRT432656(response);
            VRATOTRT432720(response); 
            VRATOGEN438017(response); 
            VRATOTRT432675(response);
            VRATOTRT432676(response);
            VRATOTRT432707(response);
            VRATOTRT432797(response);

            return response;
        }

        #region VR.ATO.TRT.432429
        /*  VR.ATO.TRT.432429
                    Losses schedule must be attached

                Legacy Rule Format: 
                    IF [TRT32] = "179" AND [TRT144] > 100000 AND (COUNT(SCHEDULE = "LS") = 0) 
                    RETURN VALIDATION MESSAGE
                    ENDIF

                    [TRT32] = TRT:RP:pyde.02.26:OrganisationDetails.OrganisationType.Code
                    [TRT144] = TRT:RP:rvctc2.02.00:IncomeTax.Deduction.TaxLossesDeducted.Amount

            Technical Business Rule Format:
                    ^TRT32 = '179' AND ^TRT144 > 100000 AND (CountDocument('LS') = 0)

        Data Elements:
            
                    RP:^TRT144 = tns:IncomeTax.Deduction.TaxLossesDeducted.Amount
            
                    RP:^TRT32 = tns:OrganisationDetails.OrganisationType.Code
                    */
        private void VRATOTRT432429(List<ProcessMessageDocument> response)
        {
            bool assertion = LSChildDocuments.Count() == 0 &&
                             !string.IsNullOrWhiteSpace(ParentDocument.TRT32) && ParentDocument.TRT32  == "179" 
                             && ParentDocument .TRT144.GetValueOrDefault ()  > 100000;
            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.432429",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Losses schedule must be attached",
                    LongDescription = @"Public unit trust (listed other than cash management unit trust) that have Tax losses deducted of greater than $100,000 must complete Part B1 of the Losses schedule for unconsolidated entities",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TaxLossesDeducted.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432429" } },
                }; 

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRT32", Value = ParentDocument.TRT32 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRT144", Value = ParentDocument.TRT144.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432429

        #region VR.ATO.TRT.432042

        /*  VR.ATO.TRT.432042
                    If Net capital gain is greater than $10,000, a Capital Gains Tax schedule must be present

                Legacy Rule Format:
                    IF [TRT133] > 10000 AND [TRT44] <> TRUE AND  (COUNT(SCHEDULE = "CGTS") = 0)
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT44] = TRT:RP:pyde.02.00:OrganisationDetails.TaxConsolidationStatus.Indicator
                    [TRT133] = TRT:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount

            Technical Business Rule Format:
                    ^TRT133 > 10000 AND ^TRT44 <> TRUE AND  (CountDocument('CGTS') = 0)

        Data Elements:
            
                    RP:^TRT44 = tns:OrganisationDetails.TaxConsolidationStatus.Indicator
            
                    RP:^TRT133 = tns:Income.CapitalGainsNet.Amount
                    */
        protected void VRATOTRT432042(List<ProcessMessageDocument> response )
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            assertion = CGTSChildDocuments.Count() == 0 && ParentDocument.TRT133.GetValueOrDefault () > 10000 && !ParentDocument.TRT44.GetValueOrDefault();
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432042",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital Gains Tax schedule must be present",
                    LongDescription = @"If Net capital gain is greater than $10,000, a Capital Gains Tax schedule must be present",
                    Location = "/xbrli:xbrl/tns:OrganisationDetails.TaxConsolidationStatus.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432042" } },
                };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT133", Value = ParentDocument .TRT133.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT44", Value = ParentDocument.TRT44.GetValueOrDefault ().ToString ()  });
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432042

        #region VR.ATO.TRT.432210

        /*  VR.ATO.TRT.432210
                    Where the total of tax losses AND net capital losses exceeds $100000, a Losses schedule must be attached.

                Legacy Rule Format:  
                IF ([TRT145] + [TRT475]) > 100000 AND [TRT44] <> TRUE AND (COUNT(SCHEDULE = "LS") = 0)
                     RETURN VALIDATION MESSAGE
                ENDIF

                    [TRT44] = TRT:RP:pyde.02.00:OrganisationDetails.TaxConsolidationStatus.Indicator
                    [TRT145] = TRT:RP.Closing:rvctc3.02.02:Tax.Losses.CarriedForward.LaterIncomeYearsTotal.Amount
                    [TRT475] = TRT:RP.Closing(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount

            Technical Business Rule Format:
                    (^TRT145 + ^TRT475) > 100000 AND ^TRT44 <> TRUE AND (CountDocument('LS') = 0)
                   
        Data Elements:
            
                    RP.Closing:^TRT145 = tns:Tax.Losses.CarriedForward.LaterIncomeYearsTotal.Amount
            
                    RP:^TRT44 = tns:OrganisationDetails.TaxConsolidationStatus.Indicator
            
                    RP.Closing(Instant):^TRT475 = tns:Capital.Losses.CarriedForward.Net.Amount
                    */
        protected void VRATOTRT432210(List<ProcessMessageDocument> response )
        {
            ProcessMessageDocument processMessage;
            bool assertion;  
            assertion = LSChildDocuments.Count() == 0 &&  (ParentDocument.TRT145.GetValueOrDefault () + ParentDocument.TRT475.GetValueOrDefault ()) > 100000 && ! ParentDocument.TRT44.GetValueOrDefault()  ;

            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432210",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Losses schedule must be attached where total of tax losses and net capital losses exceeds $100000",
                    Location = "/xbrli:xbrl/tns:Tax.Losses.CarriedForward.LaterIncomeYearsTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432210" } },
                }; 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT145", Value = ParentDocument .TRT145.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT475", Value = ParentDocument.TRT475.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT44", Value = ParentDocument.TRT44.GetValueOrDefault ().ToString () }); 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432210 

        #region VR.ATO.TRT.432043

        /*  VR.ATO.TRT.432043
                    Payment Summary schedule must be present if Gross payments where ABN not quoted is present

                Legacy Rule Format:
                    IF ([TRT46] + [TRT47]) > 0 AND (COUNT(SCHEDULE = "PSS") = 0)
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT46] = TRT:RP.JAUS.Prim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
                    [TRT47] = TRT:RP.JAUS.NonPrim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount

            Technical Business Rule Format:
                    (^TRT46 + ^TRT47) > 0 AND (CountDocument('PSS') = 0)

        Data Elements:
            
                    RP.JAUS.Prim:^TRT46 = tns:Remuneration.ABNNotQuotedPaymentGross.Amount
            
                    RP.JAUS.NonPrim:^TRT47 = tns:Remuneration.ABNNotQuotedPaymentGross.Amount
                    */
        protected void VRATOTRT432043(List<ProcessMessageDocument> response )
        {
            ProcessMessageDocument processMessage; 

            bool assertion = (PSSChildDocuments.Count() == 0 && (  ParentDocument.TRT46.GetValueOrDefault() + ParentDocument.TRT47.GetValueOrDefault()) > 0  );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430019",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Payment Summary schedule must be present if Gross payments where ABN not quoted is present",
                    Location = "/xbrli:xbrl/tns:Remuneration.ABNNotQuotedPaymentGross.Amount[@contextRef='" + ParentDocument .RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432043" } },
                };
                 
                  processMessage.Parameters.Add(new ProcessMessageParameter
                  { Name = "TRT46", Value = ParentDocument.TRT46.GetValueOrDefault().ToString() });

                  processMessage.Parameters.Add(new ProcessMessageParameter
                  { Name = "TRT47", Value = ParentDocument.TRT47.GetValueOrDefault().ToString() });



                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432043

        #region VR.ATO.TRT.432044

        /*  VR.ATO.TRT.432044
                    The sum of gross payment with indicator 'N' in the payment summary schedule must equal the sum of total gross payment where ABN not quoted for primary production AND total gross payment where ABN not quoted for non-primary production on the main return form

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "PSS") > 0) AND SUM([PSS19]) <> [TRT46] + [TRT47] 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PSS19] = PSS:RP.Payer.{PSSeqNum}:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
                    [TRT46] = TRT:RP.JAUS.Prim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
                    [TRT47] = TRT:RP.JAUS.NonPrim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount

            Technical Business Rule Format:
                    (CountDocument('PSS') > 0) AND Sum(^PSS19) <> ^TRT46 + ^TRT47

        Data Elements:
            
                    RP.JAUS.Prim:^TRT46 = tns:Remuneration.ABNNotQuotedPaymentGross.Amount
            
                    RP.JAUS.Prim:^PSS19 = PSS:RP.Payer.{PSSeqNum}:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            
                    RP.JAUS.NonPrim:^TRT47 = tns:Remuneration.ABNNotQuotedPaymentGross.Amount
                    */
        protected void VRATOTRT432044(List<ProcessMessageDocument> response)
        {
            ProcessMessageDocument processMessage;
            decimal sumPSS19 = 0; 
            foreach (var childDocument in PSSChildDocuments)
            {
                sumPSS19 += childDocument.RPPayerPSSeqNumContextCollection.Sum(c => c.PSS19.GetValueOrDefault());
            }
            
            bool assertion = (PSSChildDocuments.Count() > 0 && (ParentDocument.TRT46.GetValueOrDefault() + ParentDocument.TRT47.GetValueOrDefault()) != sumPSS19);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430020",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross payment where ABN not quoted must equal sum of payments on schedule",
                    LongDescription = @"The sum of gross payment with indicator 'N' in the Payment Summary schedule must equal the sum of Primary production and Non-primary production Gross payments where ABN not quoted on main return form",
                    Location = "/xbrli:xbrl/tns:Remuneration.ABNNotQuotedPaymentGross.Amount[@contextRef='" + ParentDocument.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432044" } },
                };

                 processMessage.Parameters.Add(new ProcessMessageParameter
                  { Name = "TRT46", Value = ParentDocument .TRT46.GetValueOrDefault().ToString() });

                  processMessage.Parameters.Add(new ProcessMessageParameter
                  { Name = "TRT47", Value = ParentDocument.TRT47.GetValueOrDefault().ToString() }); 

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432044

        #region VR.ATO.TRT.432051

        /*  VR.ATO.TRT.432051
                    The sum of gross payment with indicator 'F' in the Payment Summary schedule must equal total gross payment subject to foreign resident withholding (excluding capital gains) non-primary production on the main return form.

                Legacy Rule Format:
                    IF  (COUNT(SCHEDULE = "PSS") > 0) AND ([TRT49] <> SUM([PSS18]))
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PSS18] = PSS:RP.Payer.{PSSeqNum}:lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
                    [TRT49] = TRT:RP.JAUS.NonPrim:lrla.02.21:Remuneration.PaymentToForeignResidentGross.Amount

            Technical Business Rule Format:
                    (CountDocument('PSS') > 0) AND (^TRT49 <> Sum(^PSS18))

        Data Elements:
            
                    RP.JAUS.NonPrim:^TRT49 = tns:Remuneration.PaymentToForeignResidentGross.Amount
            
                    RP.JAUS.NonPrim:^PSS18 = PSS:RP.Payer.{PSSeqNum}:lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
                    */
        protected void VRATOTRT432051(List<ProcessMessageDocument> response )
        {
            ProcessMessageDocument processMessage;
            decimal sumPSS18 = 0;
            foreach (var childDocument in PSSChildDocuments)
            {
                sumPSS18 += childDocument.RPPayerPSSeqNumContextCollection.Sum(c => c.PSS18.GetValueOrDefault ());
            }

            bool assertion  = (PSSChildDocuments.Count() > 0 && !ParentDocument.TRT49.GetValueOrDefault ().Equals(sumPSS18));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432749",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross payment - Foreign Resident Withholding (excluding capital gains) is incorrect",
                    LongDescription = @"The sum of gross payments with indicator 'F' in the Payment Summary schedule must be equal to total gross payment subject to foreign resident withholding (excluding capital gains) non-primary production on the main return form.",
                    Location = "/xbrli:xbrl/tns:Remuneration.PaymentToForeignResidentGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432051" } },
                };

                   processMessage.Parameters.Add(new ProcessMessageParameter
                  { Name = "TRT49", Value = ParentDocument.TRT49.GetValueOrDefault().ToString() });



              response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432051

        #region VR.ATO.TRT.432088

        /*  VR.ATO.TRT.432088
                    If an amount greater than zero has been entered at Tax withheld from payments where ABN not quoted, then a Payment Summary schedule must be present AND it must include the same total amount of Tax withheld from payments where ABN not quoted

                Legacy Rule Format:
                    IF ([TRT73] > 0) AND ((COUNT(SCHEDULE = "PSS") = 0) OR (SUM([PSS23]) <> [TRT73]))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PSS23] = PSS:RP.Payer.{PSSeqNum}:rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessABNNotQuoted.Amount
                    [TRT73] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount

            Technical Business Rule Format:
                    (^TRT73 > 0) AND ((CountDocument('PSS') = 0) OR (Sum(^PSS23) <> ^TRT73))

        Data Elements:
            
                    RP.JAUS:^TRT73 = tns:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
            
                    RP.JAUS:^PSS23 = PSS:RP.Payer.{PSSeqNum}:rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessABNNotQuoted.Amount
                    */
        protected void VRATOTRT432088(List<ProcessMessageDocument> response)
        {
            ProcessMessageDocument processMessage;
            decimal sumPSS23 = 0;
            foreach (var childDocument in PSSChildDocuments)
            {
                sumPSS23 += childDocument.RPPayerPSSeqNumContextCollection.Sum(c => c.PSS23.GetValueOrDefault());
            }

            bool assertion = (ParentDocument.TRT73.GetValueOrDefault() > 0) && ((PSSChildDocuments.Count() == 0 || !ParentDocument.TRT73.GetValueOrDefault ().Equals(sumPSS23)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430183",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Payment Summary schedule must include Tax withheld where ABN not quoted amount",
                    LongDescription = @"If an amount greater than zero has been entered at Tax withheld from payments where ABN no quoted, then a Payment Summary schedule must be present and it must include the same total amount of Tax withheld from payments where ABN no quoted",
                    Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount[@contextRef='" + ParentDocument .RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432088" } },
                };

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT73", Value = ParentDocument.TRT73.GetValueOrDefault().ToString() }); 
               
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432088

        #region VR.ATO.TRT.432115

        /*  VR.ATO.TRT.432115
                    A rental schedule is attached AND rental income AND deduction amounts are not present in the return form.

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "RS") > 0) AND [TRT89] = NULL AND ([TRT90] = 0 OR [TRT90] = NULL) AND ([TRT91] = 0 OR [TRT91] = NULL) AND ([TRT92] = 0 OR [TRT92] = NULL) 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT89] = TRT:RP.JAUS:bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount
                    [TRT90] = TRT:RP.JAUS:bafpr1.02.05:Expense.Interest.Amount
                    [TRT91] = TRT:RP.JAUS:bafpr1.02.02:Expense.CapitalWorksDeduction.Amount
                    [TRT92] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount

            Technical Business Rule Format:
                    (CountDocument('RS') > 0) AND ^TRT89 = NULL AND (^TRT90 = 0 OR ^TRT90 = NULL) AND (^TRT91 = 0 OR ^TRT91 = NULL) AND (^TRT92 = 0 OR ^TRT92 = NULL)

        Data Elements:
            
                    RP.JAUS:^TRT89 = tns:Income.Operating.RentalIncomeGross.Amount
            
                    RP.JAUS:^TRT90 = tns:Expense.Interest.Amount
            
                    RP.JAUS:^TRT91 = tns:Expense.CapitalWorksDeduction.Amount
            
                    RP.JAUS:^TRT92 = tns:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
                    */
        protected void VRATOTRT432115(List<ProcessMessageDocument> response )
        {
            ProcessMessageDocument processMessage;

            bool assertion = RSChildDocuments.Count() > 0 &&  !ParentDocument.TRT89.HasValue && (!ParentDocument.TRT90.HasValue || ParentDocument.TRT90.GetValueOrDefault().Equals (0) ) && (!ParentDocument.TRT91.HasValue || ParentDocument.TRT91.GetValueOrDefault().Equals(0)) &&  (!ParentDocument.TRT92.HasValue || ParentDocument.TRT92.GetValueOrDefault().Equals(0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430082",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross rent and Rental deduction amounts must be present",
                    LongDescription = @"If a Rental property schedule is present, Gross rent or rental deductions must be present",
                    Location = "/xbrli:xbrl/tns:Income.Operating.RentalIncomeGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432115" } },
                };

                     processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT89", Value = ParentDocument.TRT89.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT90", Value = ParentDocument.TRT90.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT91", Value = ParentDocument.TRT91.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT92", Value = ParentDocument.TRT92.GetValueOrDefault().ToString() });


                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432115

        #region VR.ATO.TRT.432116

        /*  VR.ATO.TRT.432116
                    Total gross rent of schedules AND gross rent value in return must be equal

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "RS") > 0) AND (SUM(ALL OCCURRENCES OF([RS38] + [RS39]))) <> [TRT89]
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [RS38] = RS:RP:bafpr2.02.04:Income.RealEstateProperty.Rental.Amount
                    [RS39] = RS:RP:bafpr2.02.04:Income.RealEstateProperty.RentalRelatedOther.Amount
                    [TRT89] = TRT:RP.JAUS:bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount

            Technical Business Rule Format:
                    (CountDocument('RS') > 0) AND (Sum((^RS38 + ^RS39))) <> ^TRT89

        Data Elements:
            
                    RP.JAUS:^TRT89 = tns:Income.Operating.RentalIncomeGross.Amount
            
                    RP.JAUS:^RS38 = RS:RP:bafpr2.02.04:Income.RealEstateProperty.Rental.Amount
            
                    RP.JAUS:^RS39 = RS:RP:bafpr2.02.04:Income.RealEstateProperty.RentalRelatedOther.Amount
                    */
        protected void VRATOTRT432116(List<ProcessMessageDocument> response)
        {
            ProcessMessageDocument processMessage;
            decimal sumRS3839 = 0;
            foreach (var childDocument in RSChildDocuments)
            {
                sumRS3839 += childDocument.RS38.GetValueOrDefault() + childDocument.RS39.GetValueOrDefault();
            }

            bool assertion  = RSChildDocuments.Count() > 0 && !ParentDocument.TRT89.GetValueOrDefault ().Equals(sumRS3839);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430083",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross rent must equal the total Gross rent in the Rental Schedules",
                    LongDescription = @"The Gross rent amount in the main form must equal the total Gross rent present in all Rental property schedules",
                    Location = "/xbrli:xbrl/tns:Income.Operating.RentalIncomeGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432116" } },
                };

                
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432116

        #region VR.ATO.TRT.432117

        /*  VR.ATO.TRT.432117
                    Total net rent of schedule AND net rent value in return must be equal

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "RS") > 0) AND SUM([RS59]) <> ([TRT89] - [TRT90] - [TRT91] - [TRT92]) 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [RS59] = RS:RP:bafpr2.02.04:Income.RealEstateProperty.RentalNet.Amount
                    [TRT89] = TRT:RP.JAUS:bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount
                    [TRT90] = TRT:RP.JAUS:bafpr1.02.05:Expense.Interest.Amount
                    [TRT91] = TRT:RP.JAUS:bafpr1.02.02:Expense.CapitalWorksDeduction.Amount
                    [TRT92] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount

            Technical Business Rule Format:
                    (CountDocument('RS') > 0) AND Sum(^RS59) <> (^TRT89 - ^TRT90 - ^TRT91 - ^TRT92)

        Data Elements:
            
                    RP.JAUS:^TRT89 = tns:Income.Operating.RentalIncomeGross.Amount
            
                    RP.JAUS:^RS59 = RS:RP:bafpr2.02.04:Income.RealEstateProperty.RentalNet.Amount
            
                    RP.JAUS:^TRT90 = tns:Expense.Interest.Amount
            
                    RP.JAUS:^TRT91 = tns:Expense.CapitalWorksDeduction.Amount
            
                    RP.JAUS:^TRT92 = tns:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
                    */
        protected void VRATOTRT432117(List<ProcessMessageDocument> response )
        {
            ProcessMessageDocument processMessage;
            decimal sumRS59 = 0;
            foreach (var childDocument in RSChildDocuments)
            {
                sumRS59 += childDocument.RS59.GetValueOrDefault();
            }

            bool assertion = RSChildDocuments.Count() > 0 && (ParentDocument.TRT89.GetValueOrDefault() - ParentDocument.TRT90.GetValueOrDefault() - ParentDocument.TRT91.GetValueOrDefault() - ParentDocument.TRT92.GetValueOrDefault()) != sumRS59;
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430084",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net rent must equal the total net rent in the Rental Schedules",
                    LongDescription = @"The net rent amount in the main form (Gross rent minus rental deduction amounts) must equal the total Net rent present in all Rental property schedules",
                    Location = "/xbrli:xbrl/tns:Income.Operating.RentalIncomeGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432117" } },
                };

                      processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT89", Value = ParentDocument .TRT89.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT90", Value = ParentDocument.TRT90.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT91", Value = ParentDocument.TRT91.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT92", Value = ParentDocument.TRT92.GetValueOrDefault().ToString() });


                
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432117

        #region VR.ATO.TRT.432120

        /*  VR.ATO.TRT.432120
                    If there is a positive amount at Gross rent or at any Rental deductions item, then a Rental property schedule must be submitted for each rental property

                Legacy Rule Format:
                    IF ([TRT89] <> NULL OR [TRT90] > 0 OR [TRT91] > 0 OR [TRT92] > 0) AND (COUNT(SCHEDULE = "RS") = 0)
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT89] = TRT:RP.JAUS:bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount
                    [TRT90] = TRT:RP.JAUS:bafpr1.02.05:Expense.Interest.Amount
                    [TRT91] = TRT:RP.JAUS:bafpr1.02.02:Expense.CapitalWorksDeduction.Amount
                    [TRT92] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount

            Technical Business Rule Format:
                    (^TRT89 <> NULL OR ^TRT90 > 0 OR ^TRT91 > 0 OR ^TRT92 > 0) AND (CountDocument('RS') = 0)

        Data Elements:
            
                    RP.JAUS:^TRT89 = tns:Income.Operating.RentalIncomeGross.Amount
            
                    RP.JAUS:^TRT90 = tns:Expense.Interest.Amount
            
                    RP.JAUS:^TRT91 = tns:Expense.CapitalWorksDeduction.Amount
            
                    RP.JAUS:^TRT92 = tns:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
                    */
        protected void VRATOTRT432120(List<ProcessMessageDocument> response )
        {
            ProcessMessageDocument processMessage;
            bool assertion = (RSChildDocuments.Count() == 0 && (ParentDocument.TRT89.HasValue  || ParentDocument.TRT90.GetValueOrDefault() > 0 || ParentDocument.TRT91.GetValueOrDefault() > 0 || ParentDocument.TRT92.GetValueOrDefault() >0 )) ;
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430081",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Rental schedule must be present",
                    LongDescription = @"If there is a positive amount at Gross rent or at any Rental deductions item, then a Rental property schedule must be submitted for each rental property",
                    Location = "/xbrli:xbrl/tns:Income.Operating.RentalIncomeGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432120" } },
                };

                //  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT89", Value = ParentDocument.TRT89.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT90", Value = ParentDocument.TRT90.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT91", Value = ParentDocument.TRT91.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT92", Value = ParentDocument.TRT92.GetValueOrDefault().ToString() });


                
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432120

        #region VR.ATO.TRT.432122

        /*  VR.ATO.TRT.432122
                    Total interest of schedules AND interest value in return must be equal

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "RS") > 0) AND SUM([RS48]) <> [TRT90] 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [RS48] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.LoanInterest.Amount
                    [TRT90] = TRT:RP.JAUS:bafpr1.02.05:Expense.Interest.Amount

            Technical Business Rule Format:
                    (CountDocument('RS') > 0) AND Sum(^RS48) <> ^TRT90

        Data Elements:
            
                    RP.JAUS:^TRT90 = tns:Expense.Interest.Amount
            
                    RP.JAUS:^RS48 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.LoanInterest.Amount
                    */
        protected void VRATOTRT432122(List<ProcessMessageDocument> response )
        {
            ProcessMessageDocument processMessage;
            decimal sumRS48 = 0;
            foreach (var childDocument in RSChildDocuments)
            {
                sumRS48 += childDocument.RS48.GetValueOrDefault();
            }

            bool assertion  = RSChildDocuments.Count() > 0 &&  !ParentDocument.TRT90.GetValueOrDefault ().Equals(sumRS48);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430086",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Rental Interest deductions must equal the total Interest deductions in the Rental Schedules",
                    LongDescription = @"The Interest deductions amount in the main form must equal the total of all Interest deductions in the Rental property schedules",
                    Location = "/xbrli:xbrl/tns:Expense.Interest.Amount[@contextRef='" + ParentDocument.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432122" } },
                };

                      processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT90", Value = ParentDocument.TRT90.GetValueOrDefault().ToString() });


                
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432122

        #region VR.ATO.TRT.432124

        /*  VR.ATO.TRT.432124
                    Total of capital works deductions claimed in attached schedules are not equal to the total of capital works deductions claimed in the return.

                Legacy Rule Format:
                    IF  (COUNT(SCHEDULE = "RS") > 0) AND SUM([RS54]) <> [TRT91]
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [RS54] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.CapitalWorksDeduction.Amount
                    [TRT91] = TRT:RP.JAUS:bafpr1.02.02:Expense.CapitalWorksDeduction.Amount

            Technical Business Rule Format:
                    (CountDocument('RS') > 0) AND Sum(^RS54) <> ^TRT91

        Data Elements:
            
                    RP.JAUS:^TRT91 = tns:Expense.CapitalWorksDeduction.Amount
            
                    RP.JAUS:^RS54 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.CapitalWorksDeduction.Amount
                    */
        protected void VRATOTRT432124(List<ProcessMessageDocument> response)
        {
            ProcessMessageDocument processMessage;
            decimal sumRS54 = 0;
            foreach (var childDocument in RSChildDocuments)
            {
                sumRS54 += childDocument.RS54.GetValueOrDefault();
            }

            bool assertion  = RSChildDocuments.Count() > 0 &&  !ParentDocument.TRT91.GetValueOrDefault ().Equals(sumRS54);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430088",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital works deductions must equal the total Capital works deductions in the Rental Schedules",
                    LongDescription = @"The Capital works deductions amount in the main form must equal the total of all Capital works deductions in the Rental property schedules",
                    Location = "/xbrli:xbrl/tns:Expense.CapitalWorksDeduction.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432124" } },
                };

                     processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT91", Value = ParentDocument.TRT91.GetValueOrDefault().ToString() });


               
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432124

        #region VR.ATO.TRT.432126

        /*  VR.ATO.TRT.432126
                    Total of rental expenses less Interest AND Capital works (Special building write-off) of schedules AND value of Other rental deductions in return must be equal

                Legacy Rule Format:
                    IF  (COUNT(SCHEDULE = "RS") > 0) AND (SUM(ALL OCCURRENCES OF([RS40] + [RS41] + [RS42] + [RS43] + [RS44] + [RS45] + [RS46] + [RS47] + [RS49] + [RS50] + [RS51] + [RS52] + [RS53] + [RS55] + [RS56] + [RS57] + [RS58])) <> [TRT92])
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [RS40] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Marketing.Amount
                    [RS41] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.BodyCorporate.Amount
                    [RS42] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Borrowing.Amount
                    [RS43] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Cleaning.Amount
                    [RS44] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.CouncilRates.Amount
                    [RS45] = RS:RP:bafpr1.02.04:Expense.DepreciationandAmortisation.Amount
                    [RS46] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Gardening.Amount
                    [RS47] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Insurance.Amount
                    [RS49] = RS:RP:bafpr1.02.00:Expense.LandTax.Amount
                    [RS50] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.LegalFees.Amount
                    [RS51] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.PestControl.Amount
                    [RS52] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.AgentFeesCommission.Amount
                    [RS53] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.RepairsAndMaintenance.Amount
                    [RS55] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.OfficeSupplies.Amount
                    [RS56] = RS:RP:bafpr1.02.00:Expense.Travel.Amount
                    [RS57] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Water.Amount
                    [RS58] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Sundry.Amount
                    [TRT92] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount

            Technical Business Rule Format:
                    (CountDocument('RS') > 0) AND (Sum((^RS40 + ^RS41 + ^RS42 + ^RS43 + ^RS44 + ^RS45 + ^RS46 + ^RS47 + ^RS49 + ^RS50 + ^RS51 + ^RS52 + ^RS53 + ^RS55 + ^RS56 + ^RS57 + ^RS58)) <> ^TRT92)

        Data Elements:
            
                    RP.JAUS:^TRT92 = tns:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
            
                    RP.JAUS:^RS40 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Marketing.Amount
            
                    RP.JAUS:^RS41 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.BodyCorporate.Amount
            
                    RP.JAUS:^RS42 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Borrowing.Amount
            
                    RP.JAUS:^RS43 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Cleaning.Amount
            
                    RP.JAUS:^RS44 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.CouncilRates.Amount
            
                    RP.JAUS:^RS45 = RS:RP:bafpr1.02.04:Expense.DepreciationandAmortisation.Amount
            
                    RP.JAUS:^RS46 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Gardening.Amount
            
                    RP.JAUS:^RS47 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Insurance.Amount
            
                    RP.JAUS:^RS49 = RS:RP:bafpr1.02.00:Expense.LandTax.Amount
            
                    RP.JAUS:^RS50 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.LegalFees.Amount
            
                    RP.JAUS:^RS51 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.PestControl.Amount
            
                    RP.JAUS:^RS52 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.AgentFeesCommission.Amount
            
                    RP.JAUS:^RS53 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.RepairsAndMaintenance.Amount
            
                    RP.JAUS:^RS55 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.OfficeSupplies.Amount
            
                    RP.JAUS:^RS56 = RS:RP:bafpr1.02.00:Expense.Travel.Amount
            
                    RP.JAUS:^RS57 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Water.Amount
            
                    RP.JAUS:^RS58 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Sundry.Amount
                    */
        protected void VRATOTRT432126(List<ProcessMessageDocument> response )
        {
            ProcessMessageDocument processMessage;
            decimal sumRS = 0;
            foreach (var childDocument in RSChildDocuments)
            {
                //RS40 + ^RS41 + ^RS42 + ^RS43 + ^RS44 + ^RS45 + ^RS46 + ^RS47 + ^RS49 + ^RS50 + ^RS51 + ^RS52 + ^RS53 + ^RS55 + ^RS56 + ^RS57 + ^RS58
                sumRS += childDocument.RS40.GetValueOrDefault() + childDocument.RS41.GetValueOrDefault() +
                        childDocument.RS42.GetValueOrDefault() + childDocument.RS43.GetValueOrDefault() +
                        childDocument.RS44.GetValueOrDefault() + childDocument.RS45.GetValueOrDefault() +
                        childDocument.RS46.GetValueOrDefault() + childDocument.RS47.GetValueOrDefault() +
                        childDocument.RS49.GetValueOrDefault() + childDocument.RS50.GetValueOrDefault() +
                        childDocument.RS51.GetValueOrDefault() + childDocument.RS52.GetValueOrDefault() +
                        childDocument.RS53.GetValueOrDefault() + childDocument.RS55.GetValueOrDefault() +
                        childDocument.RS56.GetValueOrDefault() + childDocument.RS57.GetValueOrDefault() +
                        childDocument.RS58.GetValueOrDefault()
                    ;
            }

            bool assertion  = RSChildDocuments.Count() > 0 &&  !ParentDocument.TRT92.GetValueOrDefault ().Equals(sumRS);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430090",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Other rental deductions must equal the total other deductions in the Rental Schedules",
                    LongDescription = @"The Other rental deductions amount in the main form must equal the total of all rental expenses in the Rental property, excluding Interest deductions and Capital works deductions",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432126" } },
                };

                      processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT92", Value = ParentDocument.TRT92.GetValueOrDefault().ToString() });


                
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432126
         
        #region VR.ATO.GEN.438002

        /*  VR.ATO.GEN.438002
                    If the response to the question to whether the entity had either a direct or indirect interest in a foreign trust, controlled foreign company or transferor trust (Attributed foreign income) is 'Yes, an International Dealings Schedule must be attached

                Legacy Rule Format:
                    IF (bafpr3.xx.xx:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator = TRUE) AND (COUNT(SCHEDULE = "IDS") = 0) 
                        RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    (^TRT138 = TRUE) AND (CountDocument('IDS') = 0)

        Data Elements:
            
                    RP.JFOREIGN:^TRT138 = tns:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator
                    */
        protected void VRATOGEN438002(List<ProcessMessageDocument> response)
        {
            ProcessMessageDocument processMessage;
            bool assertion;
            assertion =  ParentDocument.TRT138.GetValueOrDefault () && IDSChildDocuments.Count() == 0;
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438002",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"International Dealings Schedule is not attached",
                    LongDescription = @"The answer to the question 'Did the entity have either a direct or indirect interest in a foreign trust, controlled foreign company or transferor trust?' is 'Yes', and no International Dealings Schedule is attached.",
                    Location = "/xbrli:xbrl/tns:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438002" } },
                };
                 
                      processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT138", Value =  ParentDocument.TRT138.GetValueOrDefault ().ToString() });

                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438002

        #region VR.ATO.TRT.432646

        /*  VR.ATO.TRT.432646
                    The sum of the amounts showing at attributable income of controlled foreign companies and controlled foreign trusts on IDS should be equal to or less than the sum of the amounts at Listed country and Unlisted country on the main return form.

                Legacy Rule Format:
                    IF (([IDS191] + [IDS192] + [IDS193]) > ([TRT134] + [TRT136]))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS191] = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesAttributableIncomeTotal.Amount
                    [IDS192] = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesChangeOfResidenceIncome.Amount
                    [IDS193] = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesAndControlledForeignTrustsInterposedAustralianEntitiesAttributableIncomeTotal.Amount
                    [TRT134] = TRT:RP.JFOREIGN.CFCLC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
                    [TRT136] = TRT:RP.JFOREIGN.CFCUC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount

            Technical Business Rule Format:
                    ((^IDS191 + ^IDS192 + ^IDS193) > (^TRT134 + ^TRT136))

        Data Elements:
            
                    RP.JFOREIGN.CFCUC:^TRT136 = tns:Income.InternationalDealings.Attributed.Amount
            
                    RP.JFOREIGN.CFCUC:^IDS191 = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesAttributableIncomeTotal.Amount
            
                    RP.JFOREIGN.CFCUC:^IDS192 = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesChangeOfResidenceIncome.Amount
            
                    RP.JFOREIGN.CFCUC:^IDS193 = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesAndControlledForeignTrustsInterposedAustralianEntitiesAttributableIncomeTotal.Amount
            
                    RP.JFOREIGN.CFCLC:^TRT134 = tns:Income.InternationalDealings.Attributed.Amount
                    */
        protected void VRATOTRT432646(List<ProcessMessageDocument> response )
        {
            ProcessMessageDocument processMessage; 
            decimal sum = 0; 
            foreach (var childDocument in IDSChildDocuments)
            {
                sum += childDocument.IDS191.GetValueOrDefault()+ childDocument.IDS192.GetValueOrDefault() + childDocument.IDS193.GetValueOrDefault();
            }
            bool assertion =   (sum > (ParentDocument.TRT134.GetValueOrDefault() + ParentDocument.TRT136.GetValueOrDefault()));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432714",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Attributed foreign income incorrect",
                    LongDescription = @"The sum of amounts at attributable income of controlled foreign companies and controlled foreign trusts on the IDS should be equal to or less than the sum of amounts at Listed country and Unlisted country on the trust return form.",
                    Location = "/xbrli:xbrl/tns:Income.InternationalDealings.Attributed.Amount[@contextRef='" + ParentDocument.RPJFOREIGNCFCUCId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432646" } },
                };

                     processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT134", Value = ParentDocument.TRT134.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT136", Value = ParentDocument.TRT136.GetValueOrDefault().ToString() });
                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432646

        #region VR.ATO.TRT.432017

        /*  VR.ATO.TRT.432017
                    Family Trust Election status code selected requires a Family trust election, revocation or variation schedule to be completed and attached to return and there is no schedule present.

                Legacy Rule Format:
                    IF [TRT29] = SET("R","V") AND  (COUNT(SCHEDULE = "FTER") = 0)
                         RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT29] = TRT:RP:rvctc3.02.03:Elections.FamilyTrustElectionRevocation.Code

            Technical Business Rule Format:
                    InSet(^TRT29, '"R","V"') AND  (CountDocument('FTER') = 0)

        Data Elements:
            
                    RP:^TRT29 = tns:Elections.FamilyTrustElectionRevocation.Code
                    */
        protected void VRATOTRT432017(List<ProcessMessageDocument> response )
        {
            ProcessMessageDocument processMessage;
            bool assertion; 

            assertion = (!string.IsNullOrWhiteSpace (ParentDocument.TRT29) && Regex.IsMatch(ParentDocument.TRT29.ToString () ,@"^(R|V)$") && FTERChildDocuments.Count() == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.432017",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Family trust election or revocation schedule must be present",
                    Location = "/xbrli:xbrl/tns:Elections.FamilyTrustElectionRevocation.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432017" } },
                };

                     processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT29", Value = ParentDocument.TRT29 });

                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432017

        #region VR.ATO.TRT.432020

        /*  VR.ATO.TRT.432020
                    The Interposed Entity Election code indicates a revocation is being made, however no schedule is attached.

                Legacy Rule Format:
                    IF ([TRT31] = "R") AND (COUNT(SCHEDULE = "IEE") = 0)
                         RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT31] = TRT:RP:rvctc3.02.00:Elections.InterposedEntityElectionRevocation.Code

            Technical Business Rule Format:
                    (^TRT31 = 'R') AND (CountDocument('IEE') = 0)

        Data Elements:
            
                    RP:^TRT31 = tns:Elections.InterposedEntityElectionRevocation.Code
                    */
        protected void VRATOTRT432020(List<ProcessMessageDocument> response )
        {
            ProcessMessageDocument processMessage;
            bool assertion;                             

            assertion =  (!string.IsNullOrWhiteSpace(ParentDocument.TRT31) && ParentDocument.TRT31.ToUpper().Equals("R") && IEEChildDocuments.Count() == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430180",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"If the Interposed Entity Election revocation code is present, an IEE schedule must be present",
                    Location = "/xbrli:xbrl/tns:Elections.InterposedEntityElectionRevocation.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432020" } },
                };

                     processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT31", Value = ParentDocument.TRT31 });
                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432020

        #region VR.ATO.TRT.432011

        /*  VR.ATO.TRT.432011
                    The income year for which the details of the Interposed Entity Election are being revoked must equal the income year of the main return being lodged.

                Legacy Rule Format:
                    IF [TRT1] <> NULL AND (COUNT(SCHEDULE = "IEE") > 0) AND (ANY OCCURRENCE OF ([IEE116] = "R") AND ([IEE82] <> [TRT1]))
                         RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE82] = IEE:RP:rvctc3.xx.xx:Elections.Revocation.Year
                    [IEE116] = IEE:RP:rvctc3.xx.xx:Elections.InterposedEntityElectionOrRevocation.Code
                    [TRT1] = TRT:RP:pyin.02.37:Report.TargetFinancial.Year

            Technical Business Rule Format:
                    ^TRT1 <> NULL AND (CountDocument('IEE') > 0) AND ((AnyOccurrence(^IEE116, ^IEE116 = 'R')) AND (^IEE82 <> ^TRT1))

        Data Elements:
            
                    RP:^TRT1 = tns:Report.TargetFinancial.Year
            
                    RP:^IEE82 = IEE:RP:rvctc3.66.66:Elections.Revocation.Year
            
                    RP:^IEE116 = IEE:RP:rvctc3.66.66:Elections.InterposedEntityElectionOrRevocation.Code
                    */
        protected void VRATOTRT432011(List<ProcessMessageDocument> response )
        {
            ProcessMessageDocument processMessage;
            bool isIEE116= false ; 
            int IEE82 = 0;
            foreach (var childDocument in IEEChildDocuments)
            {
                if (!string.IsNullOrWhiteSpace (childDocument.IEE116) && childDocument.IEE116.Equals("R"))
                    isIEE116 = true;
                IEE82 = childDocument.IEE82.GetValueOrDefault();
            }
             
            bool assertion =  (ParentDocument.TRT1.HasValue  && IEEChildDocuments.Count() > 0)  && ((isIEE116 &&  !ParentDocument.TRT1.GetValueOrDefault().Equals(IEE82)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430176",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Year of revocation must be the same as the year of the return",
                    LongDescription = @"The income year for which the details of the Interposed Entity Election are being revoked must equal the income year of the main return being lodged.",
                    Location = "/xbrli:xbrl/tns:Report.TargetFinancial.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432011" } },
                };

                     processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT1", Value = ParentDocument.TRT1.GetValueOrDefault().ToString() });

                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432011

        #region VR.ATO.TRT.432019

        /*  VR.ATO.TRT.432019
                    An IEE schedule is attached; however the IEE specified income year has not been completed on the main form.

                Legacy Rule Format:
                    IF [TRT30] = NULL AND (COUNT(SCHEDULE = "IEE") > 0)
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT30] = TRT:RP:rvctc3.02.00:Elections.InterposedEntityElectionStatus.Year

            Technical Business Rule Format:
                    ^TRT30 = NULL AND (CountDocument('IEE') > 0)

        Data Elements:
            
                    RP:^TRT30 = tns:Elections.InterposedEntityElectionStatus.Year
                    */
        protected void VRATOTRT432019(List<ProcessMessageDocument> response )
        {
            ProcessMessageDocument processMessage;
            bool assertion  =   (!ParentDocument.TRT30.HasValue && IEEChildDocuments.Count() > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430178",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"IEE specified income year must be present",
                    LongDescription = @"If an IEE schedule is present, the Interposed entity election status - Specified income year must be present",
                    Location = "/xbrli:xbrl/tns:Elections.InterposedEntityElectionStatus.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432019" } },
                };

                     processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT30", Value = ParentDocument.TRT30.GetValueOrDefault().ToString() });
                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432019

        #region VR.ATO.GEN.430008

        /*  VR.ATO.GEN.430008
                    An Entity cannot interpose itself; therefore the family trust tax file number quoted in Interposed Entity Election cannot be the same as the tax file number of the entity making the election

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "IEE") > 0) AND RP:entity.identifier.TFN = (ANY OCCURRENCE OF([IEE50])) 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE50] = IEE:RP:pyid.xx.xx:Identifiers.TaxFileNumber.Identifier IN TUPLE(iee.0003.lodge.req.xx.xx:FamilyTrust)

            Technical Business Rule Format:
                    (CountDocument('IEE') > 0) AND AnyOccurrence(^IEE50, ^IEE50 = ^TFN)

        Data Elements:
            
                    ^IEE50 = IEE:RP:pyid.66.66:Identifiers.TaxFileNumber.Identifier IN TUPLE(iee.0003.lodge.req.66.66:FamilyTrust)
            
                    ^TFN = RP
                    */
        protected void VRATOGEN430008(List<ProcessMessageDocument> response )
        {
            ProcessMessageDocument processMessage;
            bool matchTFN = false ;
            foreach (var childDocument in IEEChildDocuments)
            {
                if (!string.IsNullOrWhiteSpace (childDocument.IEE50) && childDocument.IEE50.Equals(ParentDocument.RPIdentifierTFN))
                    matchTFN = true; 
            }

            bool assertion = IEEChildDocuments.Count() > 0 && (matchTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430008",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The Family Trust TFN on the IEE must not be the same as the main entity TFN",
                    LongDescription = @"An Entity cannot interpose itself; therefore the family trust tax file number quoted in Interposed Entity Election cannot be the same as the tax file number of the entity making the election",
                    Location = "/xbrli:xbrl/xbrli:context", 
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430008" } },
                }; 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.430008

        #region VR.ATO.TRT.432647

        /*  VR.ATO.TRT.432647
                    The amount showing at section 456 –attributable income controlled foreign companies (question 22 Label D) on IDS should be equal to or less than the amount at Listed country on the main return form (company item 8 label B; partnership and trust item 22 label M).

                Legacy Rule Format:
                    IF ([TRT134] > 0) AND ([IDS188] > [TRT134])
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS188] = IDS:RP.CFCLC:bafpr1.02.12:Income.ControlledForeignCompaniesAttributableIncome.Amount
                    [TRT134] = TRT:RP.JFOREIGN.CFCLC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount

            Technical Business Rule Format:
                    (^TRT134 > 0) AND (^IDS188 > ^TRT134)

        Data Elements:
            
                    RP.JFOREIGN.CFCLC:^TRT134 = tns:Income.InternationalDealings.Attributed.Amount
            
                    RP.JFOREIGN.CFCLC:^IDS188 = IDS:RP.CFCLC:bafpr1.02.12:Income.ControlledForeignCompaniesAttributableIncome.Amount
                    */
        protected void VRATOTRT432647(List<ProcessMessageDocument> response )
        {
            ProcessMessageDocument processMessage;

            bool IDS188GreaterTRT134 = false;
            foreach (var childDocument in IDSChildDocuments)
            {
                if (ParentDocument.TRT134.GetValueOrDefault() > 0 && childDocument.IDS188.GetValueOrDefault() > ParentDocument.TRT134.GetValueOrDefault())
                    IDS188GreaterTRT134 = true; 
            }

            bool assertion = (IDSChildDocuments.Count() > 0 && IDS188GreaterTRT134);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432647",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Attributed foreign income incorrect",
                    LongDescription = @"Assessable income of controlled foreign companies and controlled foreign trusts under section 456 for Listed countries (Question 22 Label A) on the IDS should be equal to or less than Attributed foreign income - Listed country on the trust return.",
                    Location = "/xbrli:xbrl/tns:Income.InternationalDealings.Attributed.Amount[@contextRef='" + ParentDocument.RPJFOREIGNCFCLCId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432647" } },
                };

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT134", Value = ParentDocument.TRT134.GetValueOrDefault().ToString() });
                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432647

        #region VR.ATO.TRT.432650

        /*  VR.ATO.TRT.432650
                    An amount is showing at debt deductions in earning non-assessable foreign income claimed under s25-90 (question 25 Label B on IDS) and there is no amount at Interest expenses, Other deductions or Interest expenses overseas on the partnership or trust return form (item 5 label I, item 18 label Q, item 54 label Q)

                Legacy Rule Format:
                    IF ([TRT470] = NULL OR [TRT470] = 0 ) AND ([TRT182] = NULL OR [TRT182] = 0 ) AND ([TRT129] = NULL OR [TRT129] = 0) AND ([IDS207] > 0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS207] = IDS:RP:rvctc2.02.09:IncomeTax.Deduction.DebtEarningNonAssessableNonExemptForeignIncomeTotal.Amount
                    [TRT129] = TRT:RP.JAUS:bafpr1.02.04:Expense.DeductibleOther.Amount IN TUPLE(trt.0007.lodge.req.02.00:OtherDeductions)
                    [TRT182] = TRT:RP.JFOREIGN:bafpr1.02.05:Expense.Interest.Amount
                    [TRT470] = TRT:RP.JAUS:bafpr2.02.17:Expense.Operating.Interest.Amount

            Technical Business Rule Format:
                    (^TRT470 = NULL OR ^TRT470 = 0) AND (^TRT182 = NULL OR ^TRT182 = 0) AND (^TRT129 = NULL OR ^TRT129 = 0) AND (^IDS207 > 0)

        Data Elements:
            
                    RP.JAUS:^TRT470 = tns:Expense.Operating.Interest.Amount
            
                    RP.JAUS:^IDS207 = IDS:RP:rvctc2.02.09:IncomeTax.Deduction.DebtEarningNonAssessableNonExemptForeignIncomeTotal.Amount
            
                    RP.JAUS:^TRT129 = tns:Expense.DeductibleOther.Amount
            
                    RP.JFOREIGN:^TRT182 = tns:Expense.Interest.Amount
                    */
        protected void VRATOTRT432650(List<ProcessMessageDocument> response )
        {
            ProcessMessageDocument processMessage;

            decimal  IDS207 = 0;
            foreach (var childDocument in IDSChildDocuments)
            {
                IDS207 = childDocument.IDS207.GetValueOrDefault();
            }
             
            bool assertion  =  ((ParentDocument.TRT470.GetValueOrDefault ().Equals(0)) && (ParentDocument.TRT182.GetValueOrDefault ().Equals(0) ) &&
               (ParentDocument.OtherDeductionsCollection == null || 
                (ParentDocument.OtherDeductionsCollection != null &&  ParentDocument.OtherDeductionsCollection.Any(c=>c.TRT129.GetValueOrDefault().Equals (0) ))) && IDS207 > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432650",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Interest expenses required",
                    LongDescription = @"If Debt deductions in earning non-assessable foreign income claimed under s25-90 has been completed on the IDS then Total interest expenses or Other deductions or Interest expenses overseas must be greater than zero on trust return form.",
                    Location = "/xbrli:xbrl/tns:Expense.Operating.Interest.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432650" } },
                };

                     processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT470", Value = ParentDocument.TRT470.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT182", Value = ParentDocument.TRT182.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "TRT129", Value = ParentDocument.OtherDeductionsCollection != null ? ParentDocument.OtherDeductionsCollection.Select (c => c.TRT129.GetValueOrDefault()).ToString (): "null" });


                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432650

        #region VR.ATO.TRT.432651

        /*  VR.ATO.TRT.432651
                    An amount is showing at Total amount of your share of net income/distribution of profit (question 29 Label C on IDS) and there is no amount at Item 23 other assessable foreign source income, Label B Gross on the partnership or trust return form.

                Legacy Rule Format:
                    IF ([TRT140] = NULL) AND ([IDS219] > 0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS219] = IDS:RP:bafpo7.02.11:Assets.Investment.ForeignHybridLimitedPartnershipOrForeignHybridCompanyNetIncomeProfitTotal.Amount
                    [TRT140] = TRT:RP.JFOREIGN:bafpr1.02.04:Income.Other.Amount

            Technical Business Rule Format:
                    (^TRT140 = NULL) AND (^IDS219 > 0)

        Data Elements:
            
                    RP.JFOREIGN:^TRT140 = tns:Income.Other.Amount
            
                    RP.JFOREIGN:^IDS219 = IDS:RP:bafpo7.02.11:Assets.Investment.ForeignHybridLimitedPartnershipOrForeignHybridCompanyNetIncomeProfitTotal.Amount
                    */
        protected void VRATOTRT432651(List<ProcessMessageDocument> response )
        {
            ProcessMessageDocument processMessage;
            decimal  IDS219 = 0;
            foreach (var childDocument in IDSChildDocuments)
            {
                IDS219 = childDocument.IDS219.GetValueOrDefault();
            }

            bool  assertion = (IDSChildDocuments.Count() > 0 && !ParentDocument.TRT140.HasValue  && IDS219 > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432651",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross other assessable foreign source income required",
                    LongDescription = @"If Total amount of your share of net income/distribution of profit (Question 29 Label C) has been completed on the IDS then the Gross Other assessable foreign source income (Item 23 Label B) must be completed on the trust return form.",
                    Location = "/xbrli:xbrl/tns:Income.Other.Amount[@contextRef='" + ParentDocument.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432651" } },
                };

                       processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT140", Value = ParentDocument.TRT140.GetValueOrDefault().ToString() }); 
                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432651
        
        #region VR.ATO.TRT.432652

        /*  VR.ATO.TRT.432652
                    Amounts are showing at question 26 on IDS but there is no 'yes' response to the question concerning whether you have a CGT event during the year (Item 7 label G on the company return form; item 21 label G on the partnership and trust return forms).

                Legacy Rule Format:
                    IF ([TRT131] = FALSE) AND ([IDS208] = TRUE)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS208] = IDS:RP:rvctc3.02.00:CapitalGainsTax.Event.Indicator
                    [TRT131] = TRT:RP:rvctc3.02.00:CapitalGainsTax.Event.Indicator

            Technical Business Rule Format:
                    (^TRT131 = FALSE) AND (^IDS208 = TRUE)

        Data Elements:
            
                    RP:^TRT131 = tns:CapitalGainsTax.Event.Indicator
            
                    RP:^IDS208 = IDS:RP:rvctc3.02.00:CapitalGainsTax.Event.Indicator
                    */
        protected void VRATOTRT432652(List<ProcessMessageDocument> response )
        {
            ProcessMessageDocument processMessage;
            bool IDS208 = false;
            foreach (var childDocument in IDSChildDocuments)
            {
                IDS208 = childDocument.IDS208.GetValueOrDefault();
            } 
            bool assertion = (IDSChildDocuments.Count() > 0 && !ParentDocument.TRT131.GetValueOrDefault() && IDS208); 
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432652",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Check response to CGT event question",
                    LongDescription = @"If there is a 'True' response to 'Did you have any capital gains tax (CGT) events in relation to your interest in a foreign company?' On the IDS then a 'True' response is required at 'Did you have a CGT event during the year?' on the trust return.",
                    Location = "/xbrli:xbrl/tns:CapitalGainsTax.Event.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432652" } },
                };

                     processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT131", Value =  ParentDocument.TRT131.GetValueOrDefault().ToString () });
 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432652

        #region VR.ATO.TRT.432653

        /*  VR.ATO.TRT.432653
                    An amount is showing at question 26 on IDS and there is no amount at net capital gain (Item 7 label A on the company return form, Item 21 label A on the partnership and trust return forms).

                Legacy Rule Format:
                    IF ([TRT133] = NULL AND ([IDS209] > 0))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS209] = IDS:RP:rvctc3.02.10:CapitalGainsTax.ActiveForeignBusinessCapitalGains.Amount
                    [TRT133] = TRT:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount

            Technical Business Rule Format:
                    (^TRT133 = NULL AND (^IDS209 > 0))

        Data Elements:
            
                    RP:^TRT133 = tns:Income.CapitalGainsNet.Amount
            
                    RP:^IDS209 = IDS:RP:rvctc3.02.10:CapitalGainsTax.ActiveForeignBusinessCapitalGains.Amount
                    */
        protected void VRATOTRT432653(List<ProcessMessageDocument> response )
        {
            ProcessMessageDocument processMessage;

            decimal  IDS209 =0;
            foreach (var childDocument in IDSChildDocuments)
            {
                IDS209 = childDocument.IDS209.GetValueOrDefault();
            }

            bool assertion  = (IDSChildDocuments.Count() > 0 && !ParentDocument.TRT133.HasValue && IDS209 > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432653",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net capital gain required",
                    LongDescription = @"If Capital gain amounts has been completed on the IDS then Net capital gain must be completed on the trust return.",
                    Location = "/xbrli:xbrl/tns:Income.CapitalGainsNet.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432653" } },
                };

                processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT133", Value = ParentDocument.TRT133.GetValueOrDefault().ToString() });

                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432653

        #region VR.ATO.TRT.432655

        /*  VR.ATO.TRT.432655
                    If there is an amount at question 6a, label C on the IDS it must be less than or equal to the amount at item 55 label R on the partnership and trust return form.

                Legacy Rule Format:
                    IF ([TRT183] = 0 OR [TRT183] = NULL) AND ([IDS22] > 0)
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS22] = IDS:RP.Royalties:bafpr1.02.12:Expense.DeductionsTotal.Amount
                    [TRT183] = TRT:RP.JFOREIGN:bafpr1.02.27:Expense.Royalties.Amount

            Technical Business Rule Format:
                    (^TRT183 = 0 OR ^TRT183 = NULL) AND (^IDS22 > 0)

        Data Elements:
            
                    RP.JFOREIGN:^TRT183 = tns:Expense.Royalties.Amount
            
                    RP.JFOREIGN:^IDS22 = IDS:RP.Royalties:bafpr1.02.12:Expense.DeductionsTotal.Amount
                    */
        protected void VRATOTRT432655(List<ProcessMessageDocument> response )
        {
            ProcessMessageDocument processMessage;
            decimal IDS22 = 0;
            foreach (var childDocument in IDSChildDocuments)
            {
                IDS22 = childDocument.IDS22.GetValueOrDefault();
            }

            bool assertion = (IDSChildDocuments.Count() > 0  && (ParentDocument.TRT183.GetValueOrDefault().Equals (0) || !ParentDocument.TRT183.HasValue ) && IDS22  > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432655",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Royalty expenses overseas required",
                    LongDescription = @"If a Royalties deduction amount (Question 6a Label C) has been completed on the IDS then Royalty expenses overseas (BPI Label R) must be greater than zero on the partnership or trust return form.",
                    Location = "/xbrli:xbrl/tns:Expense.Royalties.Amount[@contextRef='" + ParentDocument.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432655" } },
                };

                processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT183", Value = ParentDocument.TRT183.GetValueOrDefault().ToString() });

                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432655

        #region VR.ATO.TRT.432648

        /*  VR.ATO.TRT.432648
                    Listed country, Unlisted country or Transferor trust (Item 8 Labels B, U or V) has been completed on the trust return form and the number of controlled foreign companies or trusts (Question 21) on the IDS has not been shown. Show zero at Question 21 Labels B, C or D on the IDS if at the end of your income year you no longer held an interest in CFCs or CFTs.

                Legacy Rule Format:
                    IF ([TRT134] > 0 OR [TRT136] > 0) AND ([IDS185] = NULL AND [IDS186] = NULL AND [IDS187] = NULL)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS185] = IDS:RP.CFCLC:bafot.02.29:Miscellaneous.ControlledForeignCompaniesTrustsInterest.Count
                    [IDS186] = IDS:RP.CFCSP:bafot.02.29:Miscellaneous.ControlledForeignCompaniesTrustsInterest.Count
                    [IDS187] = IDS:RP.CFCUC:bafot.02.29:Miscellaneous.ControlledForeignCompaniesTrustsInterest.Count
                    [TRT134] = TRT:RP.JFOREIGN.CFCLC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
                    [TRT136] = TRT:RP.JFOREIGN.CFCUC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount

            Technical Business Rule Format:
                    (^TRT134 > 0 OR ^TRT136 > 0) AND (^IDS185 = NULL AND ^IDS186 = NULL AND ^IDS187 = NULL)

        Data Elements:
            
                    RP.JFOREIGN.CFCLC:^TRT134 = tns:Income.InternationalDealings.Attributed.Amount
            
                    RP.JFOREIGN.CFCLC:^IDS185 = IDS:RP.CFCLC:bafot.02.29:Miscellaneous.ControlledForeignCompaniesTrustsInterest.Count
            
                    RP.JFOREIGN.CFCLC:^IDS186 = IDS:RP.CFCSP:bafot.02.29:Miscellaneous.ControlledForeignCompaniesTrustsInterest.Count
            
                    RP.JFOREIGN.CFCLC:^IDS187 = IDS:RP.CFCUC:bafot.02.29:Miscellaneous.ControlledForeignCompaniesTrustsInterest.Count
            
                    RP.JFOREIGN.CFCUC:^TRT136 = tns:Income.InternationalDealings.Attributed.Amount
                    */
        protected void VRATOTRT432648(List<ProcessMessageDocument> response )
        {
            ProcessMessageDocument processMessage;
            bool hasNillValue = false ;
            foreach (var childDocument in IDSChildDocuments)
            {
                hasNillValue = !childDocument.IDS185.HasValue && !childDocument.IDS186.HasValue && !childDocument.IDS187.HasValue;
            }

            bool assertion = (IDSChildDocuments.Count() > 0 && (ParentDocument.TRT134.GetValueOrDefault() > 0 || ParentDocument.TRT136.GetValueOrDefault() > 0) && (hasNillValue));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432756",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Number of controlled foreign companies or trusts required",
                    LongDescription = @"Listed country, Unlisted country or Transferor trust (Item 8 Labels B, U or V) has been completed on the trust return form and the number of controlled foreign companies or trusts (Question 21) on the IDS has not been shown. Show zero at Question 21 Labels B, C or D on the IDS if at the end of your income year you no longer held an interest in CFCs or CFTs.",
                    Location = "/xbrli:xbrl/tns:Income.InternationalDealings.Attributed.Amount[@contextRef='" + ParentDocument.RPJFOREIGNCFCLCId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432648" } },
                };


                processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT134", Value = ParentDocument.TRT134.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT136", Value = ParentDocument.TRT136.GetValueOrDefault().ToString() }); 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432648

        #region VR.ATO.TRT.432656

        /*  VR.ATO.TRT.432656
                    Where there is a Royalties deduction amount present on the IDS, there must also be an amount at Royalty expenses overseas on the partnership or trust return form.

                Legacy Rule Format:
                    IF ([TRT183] > 0) AND ([TRT183] < [IDS22])
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS22] = IDS:RP.Royalties:bafpr1.02.12:Expense.DeductionsTotal.Amount
                    [TRT183] = TRT:RP.JFOREIGN:bafpr1.02.27:Expense.Royalties.Amount

            Technical Business Rule Format:
                    (^TRT183 > 0) AND (^TRT183 < ^IDS22)

        Data Elements:
            
                    RP.JFOREIGN:^TRT183 = tns:Expense.Royalties.Amount
            
                    RP.JFOREIGN:^IDS22 = IDS:RP.Royalties:bafpr1.02.12:Expense.DeductionsTotal.Amount
                    */
        protected void VRATOTRT432656(List<ProcessMessageDocument> response )
        {
            ProcessMessageDocument processMessage;
            decimal IDS22 = 0;
            foreach (var childDocument in IDSChildDocuments)
            {
                IDS22 = childDocument.IDS22.GetValueOrDefault();
            }  
            bool assertion  = (IDSChildDocuments.Count() > 0 && ParentDocument .TRT183.GetValueOrDefault() > 0 && ParentDocument .TRT183.GetValueOrDefault() < IDS22);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432656",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Royalties deductions incorrect",
                    LongDescription = @"If Royalties deductions has been completed on the IDS then it must be less than or equal to Royalty expenses overseas on the trust return form",
                    Location = "/xbrli:xbrl/tns:Expense.Royalties.Amount[@contextRef='" + ParentDocument.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432656" } },
                };

                //  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT183", Value = ParentDocument.TRT183.GetValueOrDefault().ToString() });

                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432656

        #region VR.ATO.TRT.432720

        /*  VR.ATO.TRT.432720
                    If there is an amount at either 'Total TOFA gains' or 'Total TOFA Losses' that indicates you made a gain or loss from a financial arrangement subject to the TOFA rules, then there must be a 'True' response to 'Are you subject to the Taxation of Financial Arrangements rules contained in Division 230 of the ITAA 1997?' on the IDS

                Legacy Rule Format:
                    IF ([TRT356] > 0 OR [TRT357] > 0) 
                    AND (COUNT(SCHEDULE = "IDS") = 1 AND [IDS177] <> TRUE)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS177] = IDS:RP:rvctc4.02.05:InternationalDealings.TaxationOfFinancialArrangementsRulesApplied.Indicator
                    [TRT356] = TRT:RP.TOFA:bafpr1.02.04:Income.GainsTotal.Amount
                    [TRT357] = TRT:RP.TOFA:bafpr1.02.04:Expense.LossesTotal.Amount

            Technical Business Rule Format:
                    (^TRT356 > 0 OR ^TRT357 > 0) AND (CountDocument('IDS') = 1 AND ^IDS177 <> TRUE)

        Data Elements:
            
                    RP.TOFA:^TRT356 = tns:Income.GainsTotal.Amount
            
                    RP.TOFA:^IDS177 = IDS:RP:rvctc4.02.05:InternationalDealings.TaxationOfFinancialArrangementsRulesApplied.Indicator
            
                    RP.TOFA:^TRT357 = tns:Expense.LossesTotal.Amount
                    */
        protected void VRATOTRT432720(List<ProcessMessageDocument> response )
        {
            ProcessMessageDocument processMessage;
            bool IDS177 = false;
            foreach (var childDocument in IDSChildDocuments)
            {
                IDS177 = childDocument.IDS177.GetValueOrDefault();
            }
              
            bool assertion  = ((ParentDocument.TRT356.GetValueOrDefault() > 0 || ParentDocument.TRT357.GetValueOrDefault() > 0) && (IDSChildDocuments.Count() > 0 &&  IDS177 != true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432762",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Response to TOFA gains or losses is required",
                    LongDescription = @"If there is an amount at either 'Total TOFA gains' or 'Total TOFA Losses' that indicates you made a gain or loss from a financial arrangement subject to the TOFA rules, then there must be a 'True' response to 'Are you subject to the Taxation of Financial Arrangements rules contained in Division 230 of the ITAA 1997?' on the IDS.",
                    Location = "/xbrli:xbrl/tns:Income.GainsTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432720" } },
                };

                processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT356", Value = ParentDocument.TRT356.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT357", Value = ParentDocument.TRT357.GetValueOrDefault().ToString() });
                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432720

        #region VR.ATO.GEN.438017

        /*  VR.ATO.GEN.438017
                    When you were affected by the Thin capitalisation provisions, the International Dealings Schedule (IDS) must be attached

                Legacy Rule Format:
                    IF (RP:bafpo6.xx.xx:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator = TRUE) AND (COUNT(SCHEDULE = "IDS") = 0)
                      RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    (^TRT449 = TRUE) AND (CountDocument('IDS') = 0)

        Data Elements:
            
                    RP:^TRT449 = tns:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator
                    */
        protected void VRATOGEN438017(List<ProcessMessageDocument> response )
        {
            ProcessMessageDocument processMessage;
            bool assertion = (ParentDocument.TRT449.GetValueOrDefault() && IDSChildDocuments.Count() == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438017",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"International Dealings Schedule not attached",
                    LongDescription = @"If the thin capitalisation rules apply  then International Dealings Schedule must be attached",
                    Location = "/xbrli:xbrl/tns:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438017" } },
                };

                      processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT449", Value = ParentDocument.TRT449.GetValueOrDefault().ToString ()  });

                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438017

        #region VR.ATO.TRT.432675

        /*  VR.ATO.TRT.432675
                    Ensure an ‘International dealings schedule’ (IDS) is completed when a value greater than zero is present at label Interest expenses overseas

                Legacy Rule Format:
                    IF ([TRT182] > 0) AND (COUNT(SCHEDULE = "IDS") = 0)
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT182] = TRT:RP.JFOREIGN:bafpr1.02.05:Expense.Interest.Amount

            Technical Business Rule Format:
                    (^TRT182 > 0) AND (CountDocument('IDS') = 0)

        Data Elements:
            
                    RP.JFOREIGN:^TRT182 = tns:Expense.Interest.Amount
                    */
        protected void VRATOTRT432675(List<ProcessMessageDocument> response )
        {
            ProcessMessageDocument processMessage;
            bool assertion  = (ParentDocument.TRT182.GetValueOrDefault() > 0 && IDSChildDocuments.Count() == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432675",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"International Dealings Schedule not attached",
                    LongDescription = @"An 'International dealings schedule' (IDS) is not attached when a value greater than zero is present at label Interest expenses overseas",
                    Location = "/xbrli:xbrl/tns:Expense.Interest.Amount[@contextRef='" + ParentDocument.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432675" } },
                };

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT182", Value = ParentDocument .TRT182.GetValueOrDefault().ToString() });
                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432675

        #region VR.ATO.TRT.432676

        /*  VR.ATO.TRT.432676
                    Ensure an ‘International dealings schedule’ (IDS) is completed when a value greater than zero is present at label Royalty expenses overseas

                Legacy Rule Format:
                    IF ([TRT183] > 0) AND (COUNT(SCHEDULE = "IDS") = 0)
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT183] = TRT:RP.JFOREIGN:bafpr1.02.27:Expense.Royalties.Amount

            Technical Business Rule Format:
                    (^TRT183 > 0) AND (CountDocument('IDS') = 0)

        Data Elements:
            
                    RP.JFOREIGN:^TRT183 = tns:Expense.Royalties.Amount
                    */
        protected void VRATOTRT432676(List<ProcessMessageDocument> response )
        {
            ProcessMessageDocument processMessage;
            bool assertion  = (ParentDocument.TRT183.GetValueOrDefault() > 0 && IDSChildDocuments.Count() == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432676",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"International Dealings Schedule not attached",
                    LongDescription = @"An 'International dealings schedule' (IDS) is not attached when a value greater than zero is present at label Royalty expenses overseas",
                    Location = "/xbrli:xbrl/tns:Expense.Royalties.Amount[@contextRef='" + ParentDocument.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432676" } },
                };

                     processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT183", Value = ParentDocument.TRT183.GetValueOrDefault().ToString() });

                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432676

        #region VR.ATO.TRT.432707

        /*  VR.ATO.TRT.432707
                    If there is a 'NO' response to the questions concerning whether the entity had either a direct or indirect interest in a foreign trust, controlled foreign company or transferor trust and whether the value of dealings with international related parties is greater than $2 million and whether the thin capitalisation provisions apply, and no amounts have been completed at Interest expenses overseas and Royalty expenses overseas, an International Dealings schedule must not be attached.

                Legacy Rule Format:
                    IF ([TRT138] = FALSE AND [TRT148] = FALSE AND [TRT449] = FALSE) AND (([TRT182] = NULL 
                    OR [TRT182] = 0) AND ([TRT183] = NULL OR [TRT183] = 0)) AND (COUNT (Schedule = “IDS”) = 1)
                    RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT138] = TRT:RP.JFOREIGN:bafpr3.02.03:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator
                    [TRT148] = TRT:RP:rvctc4.02.10:InternationalDealings.RelatedPartiesTransactionsExcessAggregateValue.Indicator
                    [TRT182] = TRT:RP.JFOREIGN:bafpr1.02.05:Expense.Interest.Amount
                    [TRT183] = TRT:RP.JFOREIGN:bafpr1.02.27:Expense.Royalties.Amount
                    [TRT449] = TRT:RP:bafpo6.02.15:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator

            Technical Business Rule Format:
                    (^TRT138 = FALSE AND ^TRT148 = FALSE AND ^TRT449 = FALSE) AND ((^TRT182 = NULL OR ^TRT182 = 0) AND (^TRT183 = NULL OR ^TRT183 = 0)) AND (CountDocument('IDS') = 1)

        Data Elements:
            
                    RP.JFOREIGN:^TRT138 = tns:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator
            
                    RP:^TRT148 = tns:InternationalDealings.RelatedPartiesTransactionsExcessAggregateValue.Indicator
            
                    RP.JFOREIGN:^TRT182 = tns:Expense.Interest.Amount
            
                    RP.JFOREIGN:^TRT183 = tns:Expense.Royalties.Amount
            
                    RP:^TRT449 = tns:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator
                    */
        protected void VRATOTRT432707(List<ProcessMessageDocument> response )
        {
            ProcessMessageDocument processMessage;
            bool assertion  = (!ParentDocument.TRT138.GetValueOrDefault()   && !ParentDocument.TRT148.GetValueOrDefault ()  && !ParentDocument.TRT449.GetValueOrDefault () && ((!ParentDocument.TRT182.HasValue || ParentDocument.TRT182.GetValueOrDefault ().Equals (0)) && (!ParentDocument.TRT183.HasValue || ParentDocument.TRT183.GetValueOrDefault().Equals (0) ) && (IDSChildDocuments.Count() == 1)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438031",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"International Dealings Schedule is not required",
                    LongDescription = @"If there is a 'NO' response to the questions concerning whether the entity had either a direct or indirect interest in a foreign trust, controlled foreign company or transferor trust and whether the value of dealings with international related parties is greater than $2 million and whether the thin capitalisation provisions apply, and no amounts have been completed at Interest expenses overseas and Royalty expenses overseas, an International Dealings schedule must not be attached.",
                    Location = "/xbrli:xbrl/tns:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432707" } },
                };

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT138", Value =  ParentDocument.TRT138 .GetValueOrDefault ().ToString () });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT148", Value = ParentDocument.TRT148.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT449", Value = ParentDocument.TRT449.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT182", Value = ParentDocument.TRT182.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "TRT183", Value = ParentDocument.TRT183.GetValueOrDefault().ToString() });

                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432707

        #region VR.ATO.TRT.432797

        /*  VR.ATO.TRT.432797
                    If the entity has self-assessed as a Significant Global Entity (SGE), then 'Are you a small business entity, not a significant global entity and your international related party dealings do not exceed 50% of your current year aggregated turnover?' must not be 'TRUE' on the International Dealings Schedule (IDS) form.
     
            Technical Business Rule Format:
                    IF [TRT487] = TRUE AND (COUNT(SCHEDULE = "IDS") = 1 AND [IDS425] = TRUE)
                        RETURN VALIDATION MESSAGE
                    ENDIF

                    [IDS425] = IDS:RP:rvctc4.02.16:InternationalDealings.SmallBusinessConcessionsEligibility.Indicator
                    [TRT487] = TRT:RP:pyde.02.31:OrganisationDetails.SignificantGlobalEntityStatus.Indicator
                    */
        private void VRATOTRT432797(List<ProcessMessageDocument> response)
        {
            ProcessMessageDocument processMessage;
            bool assertion = ParentDocument.TRT487.GetValueOrDefault() == true
                && (IDSChildDocuments.Count() == 1
                && IDSChildDocuments.First().IDS425.GetValueOrDefault() == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.432797",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Significant Global Entity question on the International Dealings Schedule (IDS) form must not be 'TRUE'.",
                    LongDescription = @"If the entity has self-assessed as a Significant Global Entity (SGE), then 'Are you a small business entity, not a significant global entity and your international related party dealings do not exceed 50% of your current year aggregated turnover?' must not be 'TRUE' on the International Dealings Schedule (IDS) form.",
                    Location = "/xbrli:xbrl/tns:OrganisationDetails.SignificantGlobalEntityStatus.Indicator",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432797" },
                        new ProcessMessageParameter { Name = "TRT487", Value = ParentDocument.TRT487.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter { Name = "IDS425", Value = IDSChildDocuments.FirstOrDefault().IDS425.GetValueOrDefault().ToString() }
                    }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432797

    }
}
