using System.Collections.Generic;
using Ato.CD.Inbound.Shared;
using VaTS;
using DataContracts;
using Ato.EN.IntegrationServices.CodeGenerationTRT;
using Ato.EN.IntegrationServices.CodeGenerationIEE;

namespace Ato.CD.Inbound.TRT202002
{ 
    internal class CrossFormValidatorIEE : ICrossFormValidator
    {
        private TRT2020 ParentDocument { get; }

        private IEE2018 ChildDocument { get; }

        internal CrossFormValidatorIEE(TRT2020 report, BusinessDocument childDocument)
        {
            ParentDocument = report;
            ChildDocument = (IEE2018)childDocument.ConsumedReport;
        }

        public IEnumerable<ProcessMessageDocument> ValidateCrossFormRules()
        {
            var response = new List<ProcessMessageDocument>();

            VRATOGEN402010(response);
            VRATOGEN410115(response);
            VRATOGEN410119(response);
            VRATOIEE410051(response);
            VRATOIEE410070(response);
            VRATOIEE410223(response);
            VRATOIEE410224(response);
            VRATOIEE410225(response);
            VRATOIEE410226(response);

            return response;
        }

        #region VR.ATO.GEN.402010

        /*  VR.ATO.GEN.402010

                Legacy Rule Format:
                    IF (RP:pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier <> NULLORBLANK) AND (PARENT RETURN:RP:pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier <> NULLORBLANK) AND (RP:pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier <> PARENT RETURN:RP:pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier)
                    RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    (^IEE162 <> BLANK) AND (^ParentABN <> BLANK) AND (^IEE162 <> ^ParentABN)

        Data Elements:
            
                    RP:^IEE162 = tns:Identifiers.AustralianBusinessNumber.Identifier
            
                    RP:^ParentABN = ParentABN
                    */
        private void VRATOGEN402010(List<ProcessMessageDocument> response )
        {
            bool assertion =  !string.IsNullOrWhiteSpace(ChildDocument.IEE162)  && !string.IsNullOrWhiteSpace(ParentDocument.TRT7 )  &&  !ParentDocument.TRT7.Equals (ChildDocument.IEE162);

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.402010",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Your supplied ABN does not match the ABN supplied on the form it was submitted with",
                    LongDescription = @"Your supplied ABN does not match the ABN supplied on the parent form it was submitted with",
                    Location = "/xbrli:xbrl/tns:Identifiers.AustralianBusinessNumber.Identifier",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.402010" },
                        new ProcessMessageParameter { Name = "IEE162", Value = ChildDocument.IEE162 }
                    }
                };
                
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.402010

        #region VR.ATO.GEN.410115

        /*  VR.ATO.GEN.410115

                Legacy Rule Format:
                    WHERE PARENT RETURN EXISTS 
                    IF (IEE:RP:entity.identifier.TFN <> NULLORBLANK) AND (IEE:RP:entity.identifier.TFN <> PARENT RETURN:RP:entity.identifier.TFN)
                           RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    (^TFN <> BLANK) AND (^TFN <> ^ParentTFN)

        Data Elements:
            
                    ^ParentTFN = ParentTFN
            
                    ^TFN = RP
                    */
        private void VRATOGEN410115(List<ProcessMessageDocument> response )
        {
            bool assertion = !string.IsNullOrWhiteSpace(ChildDocument.RPIdentifierTFN) && !string.IsNullOrWhiteSpace(ParentDocument.RPIdentifierTFN) && ! ParentDocument.RPIdentifierTFN.Equals(ChildDocument.RPIdentifierTFN);

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.410115",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"TFN on Schedule must match TFN on main form",
                    Location = "/xbrli:xbrl/xbrli:context/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410115" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.410115

        #region VR.ATO.GEN.410119

        /*  VR.ATO.GEN.410119

                Legacy Rule Format:
                    WHERE PARENT RETURN EXISTS
                    IF (IEE:INT:entity.identifier.ABN <> NULLORBLANK) AND (IEE:INT:entity.identifier.ABN <> PARENT RETURN:INT:entity.identifier.ABN)
                          RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    (^ABN <> BLANK) AND (^ABN <> ^ParentABN)

        Data Elements:
            
                    ^ParentABN = ParentABN
            
                    ^ABN = INT
                    */
        private void VRATOGEN410119(List<ProcessMessageDocument> response )
        {
            bool assertion = !string.IsNullOrWhiteSpace(ChildDocument.INTIdentifierABN) && !string.IsNullOrWhiteSpace(ParentDocument.INTIdentifierABN) &&  ! ParentDocument.INTIdentifierABN.Equals(ChildDocument.INTIdentifierABN);

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.410119",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"ABN of the Intermediary on the schedule must match the ABN on the main form",
                    Location = "/xbrli:xbrl/xbrli:context/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410119" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.410119

        #region VR.ATO.IEE.410051

        /*  VR.ATO.IEE.410051
                    Election Commencement Time must not be entered if Income Year Specified is different to the parent return.

                Legacy Rule Format:
                    WHERE PARENT RETURN EXISTS
                    IF ([IEE49] <> NULL) AND ([IEE45] <> PARENT RETURN:Report.TargetFinancial.Year) 
                        RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    (^IEE49 <> NULL) AND (^IEE45 <> ^ParentTargetFinancialYear)

        Data Elements:
            
                    RP:^IEE49 = tns:Elections.Commencement.Date
            
                    RP:^ParentTargetFinancialYear = ParentTargetFinancialYear
            
                    RP:^IEE45 = tns:Elections.InterposedEntityElectionStatus.Year
                    */
        private void VRATOIEE410051(List<ProcessMessageDocument> response )
        {
            bool assertion =   ChildDocument.IEE49.HasValue  && !ParentDocument.TRT1.GetValueOrDefault ().Equals(ChildDocument.IEE45.GetValueOrDefault());

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410235",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Election Commencement Time must not be present if Income Year Specified is different to the parent return target financial year",
                    Location = "/xbrli:xbrl/tns:Elections.Commencement.Date",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410051" },
                        new ProcessMessageParameter { Name = "IEE49", Value = ChildDocument.IEE49.GetValueOrDefault().ToString ()},
                        new ProcessMessageParameter { Name = "IEE45", Value = ChildDocument.IEE45.GetValueOrDefault().ToString() }
                    }
                };
                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IEE.410051

        #region VR.ATO.IEE.410070

        /*  VR.ATO.IEE.410070
                    The income year for which the details of the Interposed Entity Election are being revoked must equal the income year of the main return being lodged.

                Legacy Rule Format:
                    WHERE PARENT RETURN EXISTS
                    IF ([IEE116] = "R") AND ([IEE82] <> PARENT RETURN:Report.TargetFinancial.Year) 
                        RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    (^IEE116 = 'R') AND (^IEE82 <> ^ParentTargetFinancialYear)

        Data Elements:
            
                    RP:^IEE82 = tns:Elections.Revocation.Year
            
                    RP:^ParentTargetFinancialYear = ParentTargetFinancialYear
            
                    RP:^IEE116 = tns:Elections.InterposedEntityElectionOrRevocation.Code
                    */
        private void VRATOIEE410070(List<ProcessMessageDocument> response )
        {
            bool assertion = ChildDocument.IEE116 == @"R" && ! ParentDocument.TRT1.GetValueOrDefault ().Equals(ChildDocument.IEE82.GetValueOrDefault());

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410070",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The income year of the Interposed Entity Revocation must equal the income year of the main return",
                    Location = "/xbrli:xbrl/tns:Elections.Revocation.Year",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410070" },
                        new ProcessMessageParameter { Name = "IEE116", Value = ChildDocument.IEE116 },
                        new ProcessMessageParameter { Name = "IEE82", Value = ChildDocument.IEE82.GetValueOrDefault().ToString() }
                    }
                };
                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IEE.410070

        #region VR.ATO.IEE.410223

        /*  VR.ATO.IEE.410223
                    When lodged as a schedule, Address Line 1 must be either blank or must equal Line 1 on the main form

                Legacy Rule Format:
                    WHERE PARENT RETURN EXISTS
                    IF ([IEE5] <> NULLORBLANK) AND ([IEE5] <> (PARENT RETURN:RP:pyde.xx.xx:AddressDetails.Line1.Text WHERE ((TUPLE ELEMENT EXPLICIT pyde.xx.xx:AddressDetails.Usage.Code = "POS") AND (TUPLE ELEMENT EXPLICIT pyde.xx.xx:AddressDetails.Currency.Code = "C")) IN TUPLE(xbrli\address2.xx.xx:AddressDetails)
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    (^IEE5 <> BLANK) AND (^IEE5 <> ^ParentADD1.T)

        Data Elements:
            
                    RP:^IEE5 = tns:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            
                    RP:^ParentADD1.T = ParentADD1.T
                    */
        private void VRATOIEE410223(List<ProcessMessageDocument> response )
        {
              
            bool assertion =  !string.IsNullOrWhiteSpace(ChildDocument.IEE5) && !string.IsNullOrWhiteSpace(ParentDocument.TRT9) && ! ParentDocument.TRT9.Equals (ChildDocument.IEE5);

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410223",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Interposed Entity Address must be blank or be the same as on main form",
                    LongDescription = @"When the IEE is lodged as a schedule, the Address of the interposed entity must be either blank or must equal the address on the main form",
                    Location = "/xbrli:xbrl/tns:AddressDetails",// + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410223" },
                        new ProcessMessageParameter { Name = "IEE5", Value = ChildDocument.IEE5 }
                    }
                };
                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IEE.410223

        #region VR.ATO.IEE.410224

        /*  VR.ATO.IEE.410224
                    When lodged as a schedule, Address Line 1 must be either blank or must equal Line 1 on the main form

                Legacy Rule Format:
                    WHERE PARENT RETURN EXISTS
                    IF ([IEE11] <> NULLORBLANK) AND ([IEE11] <> (PARENT RETURN:RP:pyde.xx.xx:AddressDetails.Line1.Text WHERE ((TUPLE ELEMENT EXPLICIT pyde.xx.xx:AddressDetails.Usage.Code = "POS") AND (TUPLE ELEMENT EXPLICIT pyde.xx.xx:AddressDetails.Currency.Code = "P")) IN TUPLE(xbrli\address2.xx.xx:AddressDetails)
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    (^IEE11 <> BLANK) AND (^IEE11 <> ^ParentADD1.T)

        Data Elements:
            
                    RP:^IEE11 = tns:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            
                    RP:^ParentADD1.T = ParentADD1.T
                    */
        private void VRATOIEE410224(List<ProcessMessageDocument> response )
        {   
            bool assertion = !string.IsNullOrWhiteSpace(ChildDocument.IEE11) && !string.IsNullOrWhiteSpace(ParentDocument.TRT14) && ! ParentDocument.TRT14.Equals( ChildDocument.IEE11);

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410223",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Interposed Entity Address must be blank or be the same as on main form",
                    LongDescription = @"When the IEE is lodged as a schedule, the Address of the interposed entity must be either blank or must equal the address on the main form",
                    Location = "/xbrli:xbrl/tns:AddressDetails",// + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410224" },
                        new ProcessMessageParameter { Name = "IEE11", Value = ChildDocument.IEE11 }
                    }
                };
 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IEE.410224

        #region VR.ATO.IEE.410225

        /*  VR.ATO.IEE.410225
                    If lodged as a schedule, the Organisation Name must be either blank or the same as on main form.

                Legacy Rule Format:
                    WHERE PARENT RETURN EXISTS
                    IF ([IEE10] <> NULLORBLANK) AND ([IEE10] <> (PARENT RETURN:RP:pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text IN TUPLE (xbrli\orgname2.xx.xx:OrganisationNameDetails) WHERE ((TUPLE EXPLICIT pyde.xx.xx:OrganisationNameDetails.OrganisationalNameType.Code = "MN") AND (TUPLE EXPLICIT pyde.xx.xx:OrganisationNameDetails.Currency.Code = "P"))) 
                      RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    (^IEE10 <> BLANK) AND (^IEE10 <> ^ParentORGN.T)

        Data Elements:
            
                    RP:^IEE10 = tns:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='P']
            
                    RP:^ParentORGN.T = ParentORGN.T
                    */
        private void VRATOIEE410225(List<ProcessMessageDocument> response )
        {
            bool assertion =  !string.IsNullOrWhiteSpace(ChildDocument.IEE10) && !string.IsNullOrWhiteSpace(ParentDocument.TRT8) && !ChildDocument.IEE10.Equals (ParentDocument.TRT8 );

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.402063",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Name on schedule does not match parent form",
                    LongDescription = @"Name on schedule does not match name on parent form.",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails",// + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_P_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410225" },
                        new ProcessMessageParameter { Name = "IEE10", Value = ChildDocument.IEE10 }
                    }
                };
                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IEE.410225

        #region VR.ATO.IEE.410226

        /*  VR.ATO.IEE.410226
                    If lodged as a schedule, the Organisation Name must be either blank or the same as on main form.

                Legacy Rule Format:
                    WHERE PARENT RETURN EXISTS
                    IF ([IEE4] <> NULLORBLANK) AND ([IEE4] <> (PARENT RETURN:RP:pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text IN TUPLE (xbrli\orgname2.xx.xx:OrganisationNameDetails) WHERE ((TUPLE EXPLICIT pyde.xx.xx:OrganisationNameDetails.OrganisationalNameType.Code = "MN") AND (TUPLE EXPLICIT pyde.xx.xx:OrganisationNameDetails.Currency.Code = "C"))) 
                             RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    (^IEE4 <> BLANK) AND (^IEE4 <> ^ParentORGN.T)

        Data Elements:
            
                    RP:^IEE4 = tns:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            
                    RP:^ParentORGN.T = ParentORGN.T
                    */
        private void VRATOIEE410226(List<ProcessMessageDocument> response )
        {
            bool assertion = !string.IsNullOrWhiteSpace(ChildDocument.IEE4)  && !string.IsNullOrWhiteSpace(ParentDocument.TRT6) && ! ChildDocument .IEE4.Equals (ParentDocument .TRT6);

            if (assertion) 
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.402063",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Name on schedule does not match parent form",
                    LongDescription = @"Name on schedule does not match name on parent form.",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails",// + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410226" },
                        new ProcessMessageParameter { Name = "IEE4", Value = ChildDocument.IEE4 }
                    }
                };
 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IEE.410226
    }
}