using System.Linq;
using System.Collections.Generic;
using Ato.CD.Inbound.Shared;
using VaTS;
using Ato.EN.IntegrationServices.CodeGenerationLS;
using DataContracts;
using Ato.EN.IntegrationServices.CodeGenerationTRT;

namespace Ato.CD.Inbound.TRT202002
{
    internal class CrossFormValidatorLS : ICrossFormValidator
    {
        private TRT2020 ParentDocument { get; }

        private LS2017 ChildDocument { get; }

        private List<Context> Contexts { get; }

        internal CrossFormValidatorLS(TRT2020 report, BusinessDocument childDocument)
        {
            ParentDocument = report;
            ChildDocument = (LS2017)childDocument.ConsumedReport;
            Contexts = GetContexts();
        }

        public IEnumerable<ProcessMessageDocument> ValidateCrossFormRules()
        {
            var response = new List<ProcessMessageDocument>();

            VRATOTRT432215(response);
            VRATOTRT432208(response);
            VRATOTRT432212(response);
            VRATOLS414022(response);
            VRATOGEN402009(response);
            VRATOGEN402010(response);
            VRATOGEN438000(response);
            VRATOGEN438001(response);
            VRATOGEN428045(response);

            return response;
        }

        #region VR.ATO.TRT.432215

        /*  VR.ATO.TRT.432215
                    Net capital losses carried forward to later income years - Total in the Losses schedule for unconsolidated entities is equal to the Net capital losses carried forward to later income years on the main income tax return.

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "LS") = 1) AND [LS52] <> [TRT475]  
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [LS52] = LS:RP.ALL(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
                    [TRT475] = TRT:RP.Closing(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount

            Technical Business Rule Format:
                    (CountDocument('LS') = 1) AND ^LS52 <> ^TRT475

        Data Elements:
            
                    RP.Closing(Instant):^TRT475 = tns:Capital.Losses.CarriedForward.Net.Amount
            
                    RP.Closing(Instant):^LS52 = LS:RP.ALL(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
                    */
        private void VRATOTRT432215(List<ProcessMessageDocument> response )
        {
            bool assertion = !ParentDocument.TRT475.GetValueOrDefault().Equals(ChildDocument.LS52.GetValueOrDefault());

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.434087",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net capital losses carried forward to later income years must equal amount in Losses schedule",
                    LongDescription = @"The total of the 'Net capital losses carried forward to later income years'  for unconsolidated entities in the Losses schedule must equal 'Net capital losses carried forward to later income years' on the main form",
                    Location = "/xbrli:xbrl/tns:Capital.Losses.CarriedForward.Net.Amount",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432215" },
                        new ProcessMessageParameter
                            { Name = "TRT475", Value = ParentDocument.TRT475.GetValueOrDefault().ToString() }
                    }
                };
                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432215

        #region VR.ATO.TRT.432208

        /*  VR.ATO.TRT.432208
                    Public unit trust (listed other than cash management unit trust) that have Tax losses deducted of greater than $100,000 must complete Part B1 of the Losses schedule for unconsolidated entities. Trusts are required to indicate whether continuity of majority ownership test passed for each year of loss.

                Legacy Rule Format:
                    IF [TRT32] = "179" AND [TRT144] > 100000 AND (COUNT(SCHEDULE = "LS") = 1) AND [LS15] = NULL AND [LS16] = NULL AND [LS17] = NULL AND [LS18] = NULL AND [LS19] = NULL AND [LS53] = NULL 
                    RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [LS15] = LS:RP.Y0:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
                    [LS16] = LS:RP.Y0-1:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
                    [LS17] = LS:RP.Y0-2:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
                    [LS18] = LS:RP.Y0-3:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
                    [LS19] = LS:RP.Y0-4:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
                    [LS53] = LS:RP.Y0-5P:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
                    [TRT32] = TRT:RP:pyde.02.26:OrganisationDetails.OrganisationType.Code
                    [TRT144] = TRT:RP:rvctc2.02.00:IncomeTax.Deduction.TaxLossesDeducted.Amount

            Technical Business Rule Format:
                    ^TRT32 = '179' AND ^TRT144 > 100000 AND (CountDocument('LS') = 1) AND ^LS15 = NULL AND ^LS16 = NULL AND ^LS17 = NULL AND ^LS18 = NULL AND ^LS19 = NULL AND ^LS53 = NULL

        Data Elements:
            
                    RP:^TRT144 = tns:IncomeTax.Deduction.TaxLossesDeducted.Amount
            
                    RP:^LS15 = LS:RP.Y0:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
            
                    RP:^LS16 = LS:RP.Y0-1:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
            
                    RP:^LS17 = LS:RP.Y0-2:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
            
                    RP:^LS18 = LS:RP.Y0-3:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
            
                    RP:^LS19 = LS:RP.Y0-4:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
            
                    RP:^LS53 = LS:RP.Y0-5P:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
            
                    RP:^TRT32 = tns:OrganisationDetails.OrganisationType.Code
                    */
        private void VRATOTRT432208(List<ProcessMessageDocument> response )
        {
            bool assertion = !string.IsNullOrWhiteSpace(ParentDocument.TRT32) && ParentDocument.TRT32.ToString().Equals("179") 
                && ParentDocument.TRT144.GetValueOrDefault() > 100000 &&
                !(ChildDocument.LS15.HasValue && ChildDocument.LS16.HasValue && ChildDocument.LS17.HasValue 
                && ChildDocument.LS18.HasValue && ChildDocument.LS19.HasValue && ChildDocument .LS53 .HasValue );

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.TRT.432208",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Widely held trusts must indicate whether continuity of majority ownership test passed for each year of loss",
                    LongDescription = @"Public unit trust (listed other than cash management unit trust) that have Tax losses deducted of greater than $100,000 must complete Part B1 of the Losses schedule for unconsolidated entities",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TaxLossesDeducted.Amount",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432208" },
                        new ProcessMessageParameter { Name = "TRT32", Value = ParentDocument.TRT32 },
                        new ProcessMessageParameter { Name = "TRT144", Value = ParentDocument.TRT144.GetValueOrDefault().ToString() }
                    }
                };
                
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432208

        #region VR.ATO.TRT.432212

        /*  VR.ATO.TRT.432212
                    Total of the tax losses carried forward to later income years in the losses schedule for unconsolidated entities must equal tax losses carried forward to later income years on the main income tax return.

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "LS") = 1) AND [LS50] <> [TRT145] 
                    RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [LS50] = LS:RP.ALL:rvctc3.02.00:Tax.Losses.CarriedForward.Total.Amount
                    [TRT145] = TRT:RP.Closing:rvctc3.02.02:Tax.Losses.CarriedForward.LaterIncomeYearsTotal.Amount

            Technical Business Rule Format:
                    (CountDocument('LS') = 1) AND ^LS50 <> ^TRT145

        Data Elements:
            
                    RP.Closing:^TRT145 = tns:Tax.Losses.CarriedForward.LaterIncomeYearsTotal.Amount
            
                    RP.Closing:^LS50 = LS:RP.ALL:rvctc3.02.00:Tax.Losses.CarriedForward.Total.Amount
                    */
        private void VRATOTRT432212(List<ProcessMessageDocument> response)
        {
            bool assertion = !ParentDocument.TRT145.GetValueOrDefault ().Equals(ChildDocument.LS50.GetValueOrDefault());

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.434086",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Tax losses carried forward to later income years in schedule must equal amount in main form",
                    LongDescription = @"The total of the 'Tax losses carried forward to later income years'  for unconsolidated entities in the Losses schedule must equal 'Tax losses carried forward to later income years' on the main form",
                    Location = "/xbrli:xbrl/tns:Tax.Losses.CarriedForward.LaterIncomeYearsTotal.Amount",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.TRT.432212" },
                        new ProcessMessageParameter { Name = "TRT145", Value = ParentDocument.TRT145.GetValueOrDefault().ToString() }
                    }
                };
                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432212

        #region VR.ATO.LS.414022
        /*  VR.ATO.LS.414022
                   Questions relating to unrealised losses are to be completed by companies only. Funds and trusts are not required to complete.

               Legacy Rule Format:
                   IF (PARENT RETURN<>"CTR") AND ([LS25] <> NULL OR [LS26] <> NULL OR [LS27] <> NULL OR [LS28] <> NULL)
                     RETURN VALIDATION MESSAGE
                   ENDIF

                   [LS25] = LS:RP:rvctc3.02.00:Tax.Losses.ChangeOfOwnershipOrControlAfterReferenceTime.Indicator
                   [LS26] = LS:RP:rvctc3.02.00:Tax.Losses.MaximumNetAssetValueTestPassedAtChangeoverTime.Indicator
                   [LS27] = LS:RP:rvctc3.02.00:Tax.Losses.UnrealisedNetLossAtChangeoverTime.Indicator
                   [LS28] = LS:RP:rvctc3.02.00:Tax.Losses.UnrealisedNetLossAtChangeoverTime.Amount

           Technical Business Rule Format:
                   (Not(ParentDocument('CTR'))) AND (^LS25 <> NULL OR ^LS26 <> NULL OR ^LS27 <> NULL OR ^LS28 <> NULL)

       Data Elements:

                   RP:^LS25 = tns:Tax.Losses.ChangeOfOwnershipOrControlAfterReferenceTime.Indicator

                   RP:^LS26 = tns:Tax.Losses.MaximumNetAssetValueTestPassedAtChangeoverTime.Indicator

                   RP:^LS27 = tns:Tax.Losses.UnrealisedNetLossAtChangeoverTime.Indicator

                   RP:^LS28 = tns:Tax.Losses.UnrealisedNetLossAtChangeoverTime.Amount
                   */
        private void VRATOLS414022(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.LS25.HasValue || ChildDocument.LS26.HasValue || ChildDocument.LS27.HasValue || ChildDocument.LS28.HasValue;

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.LS.414022",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Unrealised losses are not required",
                    Location = "/xbrli:xbrl/tns:Tax.Losses.ChangeOfOwnershipOrControlAfterReferenceTime.Indicator",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.LS.414022" },
                        new ProcessMessageParameter { Name = "LS25", Value = ChildDocument.LS25.GetValueOrDefault().ToString () },
                        new ProcessMessageParameter { Name = "LS26", Value = ChildDocument.LS26.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter { Name = "LS27", Value =  ChildDocument.LS27.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter { Name = "LS28", Value = ChildDocument.LS28.GetValueOrDefault().ToString() }
                    }
                };
            
                response.Add(processMessage);
            }
        }

        #endregion // VR.ATO.LS.414022

        #region VR.ATO.GEN.402009

        /*  VR.ATO.GEN.402009
                    Your supplied TFN does not match the TFN supplied on the form it was submitted with

                Legacy Rule Format:
                    IF (RP:entity.identifier.TFN <> PARENT RETURN:RP:entity.identifier.TFN)
                    RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^TFN <> ^ParentTFN

        Data Elements:
            
                    ^ParentTFN = ParentTFN
            
                    ^TFN = RP
                    */
        private void VRATOGEN402009(List<ProcessMessageDocument> response )
        {
            bool assertion = !ParentDocument.RPIdentifierTFN.Equals(ChildDocument.RPIdentifierTFN);

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.402009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Your supplied TFN does not match the TFN supplied on the form it was submitted with",
                    LongDescription = @"Your supplied TFN does not match the TFN supplied on the parent form it was submitted with",
                    Location = "/xbrli:xbrl/xbrli:context/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.402009" } }
                };

                
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.402009

        #region VR.ATO.GEN.402010

        /*  VR.ATO.GEN.402010
                    Your supplied ABN does not match the ABN supplied on the form it was submitted with

                Legacy Rule Format:
                    IF (RP:pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier <> NULLORBLANK) AND (PARENT RETURN:RP:pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier <> NULLORBLANK) AND (RP:pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier <> PARENT RETURN:RP:pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier)
                    RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    (^LS3 <> BLANK) AND (^ParentABN <> BLANK) AND (^LS3 <> ^ParentABN)

        Data Elements:
            
                    RP:^LS3 = tns:Identifiers.AustralianBusinessNumber.Identifier
            
                    RP:^ParentABN = ParentABN
                    */
        private void VRATOGEN402010(List<ProcessMessageDocument> response )
        {
            bool assertion = !string.IsNullOrWhiteSpace(ParentDocument.TRT7) &&
                             !string.IsNullOrWhiteSpace(ChildDocument.LS3) &&
                             !ParentDocument.TRT7.Equals(ChildDocument.LS3);
            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.402010",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Your supplied ABN does not match the ABN supplied on the form it was submitted with",
                    LongDescription = @"Your supplied ABN does not match the ABN supplied on the parent form it was submitted with",
                    Location = "/xbrli:xbrl/tns:Identifiers.AustralianBusinessNumber.Identifier",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.402010" },
                        new ProcessMessageParameter { Name = "LS3", Value = ChildDocument.LS3 }
                    }
                };
                
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.402010

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on the schedule does not match period start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        private void VRATOGEN438000(List<ProcessMessageDocument> response )
        {
            response.AddRange(from context in Contexts
                              where context.StartDate.GetValueOrDefault() != ParentDocument.RPStartDate.GetValueOrDefault()
                              select new ProcessMessageDocument
                              {
                                  Code = "CMN.ATO.GEN.438000",
                                  Severity = ProcessMessageSeverity.Error,
                                  Description = @"The context period start date is incorrect.",
                                  LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                                  Location = $"/xbrli:xbrl/xbrli:context[{context.Index}]/xbrli:period/xbrli:startDate",
                                  Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } }
                              });
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on the schedule does not match period end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.Y0-3
                    */
        protected void VRATOGEN438001(List<ProcessMessageDocument> response )
        {
            response.AddRange(from context in Contexts
                              where context.EndDate.GetValueOrDefault() != ParentDocument.RPEndDate.GetValueOrDefault()
                              select new ProcessMessageDocument
                              {
                                  Code = "CMN.ATO.GEN.438001",
                                  Severity = ProcessMessageSeverity.Error,
                                  Description = @"The context period end date is incorrect.",
                                  LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                                  Location = $"/xbrli:xbrl/xbrli:context[{context.Index}]/xbrli:period/xbrli:endDate",
                                  Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } }
                              });
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.428045

        /*  VR.ATO.GEN.428045
                    Organisation Name in schedule should match Organisation Name in parent return

                Legacy Rule Format:
                    IF RP:pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text
                    IN TUPLE (xbrli\organisationname1.xx.xx:OrganisationNameDetails) <> NULL
                    AND RP:pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text
                    IN TUPLE (xbrli\organisationname1.xx.xx:OrganisationNameDetails) <> 
                    (PARENT RETURN:RP:pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text
                    IN TUPLE (xbrli\organisationname2.xx.xx:OrganisationNameDetails)
                    WHERE (TUPLE EXPLICIT pyde.xx.xx:OrganisationNameDetails.OrganisationalNameType.Code = "MN")
                    AND  (TUPLE EXPLICIT pyde.xx.xx:OrganisationNameDetails.Currency.Code = "C")) 
                             RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^LS2 <> NULL AND ^LS2 <> ^ParentORGN.T

        Data Elements:
            
                    RP:^LS2 = tns:OrganisationNameDetails.OrganisationalName.Text
            
                    RP:^ParentORGN.T = ParentORGN.T
                    */
        private void VRATOGEN428045(List<ProcessMessageDocument> response)
        {
            bool assertion = !string.IsNullOrWhiteSpace(ParentDocument.TRT6) &&
                             !string.IsNullOrWhiteSpace(ChildDocument.LS2) &&
                             !ParentDocument.TRT6.ToUpper().Equals(ChildDocument.LS2.ToUpper()) &&
                             !string.IsNullOrWhiteSpace(ParentDocument.TRT365) && ParentDocument.TRT365.Equals("MN") &&
                             !string.IsNullOrWhiteSpace(ParentDocument.TRT366) && ParentDocument.TRT366.Equals("C");
            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.428045",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Organisation Name in schedule should match Organisation Name in parent return",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails/tns:OrganisationNameDetails.OrganisationalName.Text", 
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.428045" },
                        new ProcessMessageParameter { Name = "LS2", Value = ChildDocument.LS2 }
                    }
                };
                
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.428045

        private List<Context> GetContexts() => new List<Context>()
            .AddContext(ChildDocument.RPStartDate, ChildDocument.RPEndDate, ChildDocument.RPOccurrenceIndex, ChildDocument.RPId)
            .AddContext(ChildDocument.RPY0StartDate, ChildDocument.RPY0EndDate, ChildDocument.RPY0OccurrenceIndex, ChildDocument.RPY0Id)
            .AddContext(ChildDocument.RPY0Minus1StartDate, ChildDocument.RPY0Minus1EndDate, ChildDocument.RPY0Minus1OccurrenceIndex, ChildDocument.RPY0Minus1Id)
            .AddContext(ChildDocument.RPY0Minus2StartDate, ChildDocument.RPY0Minus2EndDate, ChildDocument.RPY0Minus2OccurrenceIndex, ChildDocument.RPY0Minus2Id)
            .AddContext(ChildDocument.RPY0Minus3StartDate, ChildDocument.RPY0Minus3EndDate, ChildDocument.RPY0Minus3OccurrenceIndex, ChildDocument.RPY0Minus3Id)
            .AddContext(ChildDocument.RPY0Minus4StartDate, ChildDocument.RPY0Minus4EndDate, ChildDocument.RPY0Minus4OccurrenceIndex, ChildDocument.RPY0Minus4Id)
            .AddContext(ChildDocument.RPY0Minus5PStartDate, ChildDocument.RPY0Minus5PEndDate, ChildDocument.RPY0Minus5POccurrenceIndex, ChildDocument.RPY0Minus5PId)
            .AddContext(ChildDocument.RPALLStartDate, ChildDocument.RPALLEndDate, ChildDocument.RPALLOccurrenceIndex, ChildDocument.RPALLId);
    }
}
