using System.Linq;
using System.Collections.Generic;
using Ato.CD.Inbound.Shared;
using VaTS;
using DataContracts;
using Ato.EN.IntegrationServices.CodeGenerationTRT;
using Ato.EN.IntegrationServices.CodeGenerationPSS;

namespace Ato.CD.Inbound.TRT202002
{
    internal class CrossFormValidatorPSS : ICrossFormValidator
    {
        private TRT2020 ParentDocument { get; }

        private PSS2018 ChildDocument { get; }

        internal CrossFormValidatorPSS(TRT2020 report, BusinessDocument childDocument)
        {
            ParentDocument = report;
            ChildDocument = (PSS2018)childDocument.ConsumedReport;
        }

        public IEnumerable<ProcessMessageDocument> ValidateCrossFormRules()
        {
            var response = new List<ProcessMessageDocument>();

            VRATOPSS000036(response);
            VRATOPSS000035(response);

            return response;
        }

        #region VR.ATO.PSS.000036

        /*  VR.ATO.PSS.000036
                    Your supplied TFN does not match the TFN supplied on the form it was submitted with

                Legacy Rule Format:
                    WHERE IN CONTEXT (RP.Payer.{PSSeqNum})
                    IF (RP.Payer.{PSSeqNum}.entity.identifier.TFN <> PARENT RETURN:RP.entity.identifier.TFN)
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    (^TFN <> PARENT:RP.^TFN2)

        Data Elements:
            
                    ^TFN = RP.Payer.{PSSeqNum}
            
                    ^TFN2 = TFN2
                    */
        private void VRATOPSS000036(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.RPPayerPSSeqNumContextCollection != null && ChildDocument.RPPayerPSSeqNumContextCollection.Any(c => c.IdentifierTFN != ParentDocument.RPIdentifierTFN);

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.PSS.000036",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Your supplied TFN does not match the TFN supplied on the form it was submitted with",
                    Location = "/xbrli:xbrl/xbrli:context/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.PSS.000036" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.PSS.000036

        #region VR.ATO.PSS.000035 

        /*  VR.ATO.PSS.000035
                     Payment Type not applicable to Non-individual income tax return

                 Legacy Rule Format:
                     IF PARENT RETURN <> “IITR”
                      AND ([PSS20] <> NULLORBLANK OR [PSS21] <> NULLORBLANK OR [PSS31] <> NULLORBLANK OR [PSS24] <> NULLORBLANK OR [PSS25] <> NULLORBLANK OR [PSS32] <> NULLORBLANK)
                        RETURN VALIDATION MESSAGE
                     ENDIF


                     [PSS20] = PSS:RP.Payer.{PSSeqNum}:lrla.02.00:Remuneration.VoluntaryAgreementGross.Amount
                     [PSS21] = PSS:RP.Payer.{PSSeqNum}:lrla.02.00:Remuneration.LabourHireArrangementPaymentGross.Amount
                     [PSS24] = PSS:RP.Payer.{PSSeqNum}:rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessVoluntaryAgreement.Amount
                     [PSS25] = PSS:RP.Payer.{PSSeqNum}:rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessLabourHireOrOtherPayments.Amount
                     [PSS31] = PSS:RP.Payer.{PSSeqNum}:lrla.02.17:Remuneration.PersonalServicesIncome.AttributedGross.Amount
                     [PSS32] = PSS:RP.Payer.{PSSeqNum}:rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldPersonalServicesIncome.Amount

             Technical Business Rule Format:
                     PARENT <> 'IITR' AND (^PSS20 <> BLANK OR ^PSS21 <> BLANK OR ^PSS31 <> BLANK OR ^PSS24 <> BLANK OR ^PSS25 <> BLANK OR ^PSS32 <> BLANK)

         Data Elements:

                     RP.Payer.{PSSeqNum}:^PSS20 = lrla.02.00:Remuneration.VoluntaryAgreementGross.Amount

                     RP.Payer.{PSSeqNum}:^PSS21 = lrla.02.00:Remuneration.LabourHireArrangementPaymentGross.Amount

                     RP.Payer.{PSSeqNum}:^PSS24 = rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessVoluntaryAgreement.Amount

                     RP.Payer.{PSSeqNum}:^PSS25 = rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessLabourHireOrOtherPayments.Amount

                     RP.Payer.{PSSeqNum}:^PSS31 = lrla.02.17:Remuneration.PersonalServicesIncome.AttributedGross.Amount

                     RP.Payer.{PSSeqNum}:^PSS32 = rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldPersonalServicesIncome.Amount
                     */
        private void VRATOPSS000035(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.RPPayerPSSeqNumContextCollection != null &&  ChildDocument.RPPayerPSSeqNumContextCollection.Any(c => c.PSS20.HasValue || c.PSS21.HasValue || c.PSS31.HasValue || c.PSS24.HasValue || c.PSS25.HasValue || c.PSS32.HasValue);

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.PSS.000035",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Payment Type not applicable to Non-individual income tax return",
                    Location = "/xbrli:xbrl/tns:Remuneration.PaymentToForeignResidentGross.Amount",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.PSS.000035" } }
                };
                
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.PSS.000035 
    }
}