using System.Linq;
using System.Collections.Generic;
using Ato.CD.Inbound.Shared;
using Ato.EN.IntegrationServices.CodeGenerationPSS;
using Ato.EN.IntegrationServices.CodeGenerationTRTAMI;
using VaTS;
using DataContracts;

namespace Ato.CD.Inbound.TRTAMI202002
{
    internal class CrossFormValidatorPSS : ICrossFormValidator
    {       
        private TRTAMI2018 ParentDocument { get; }
        private PSS2018 ChildDocument { get; }

        internal CrossFormValidatorPSS(TRTAMI2018 report, BusinessDocument childDocument)
        {
            ParentDocument = report;
            ChildDocument = (PSS2018)childDocument.ConsumedReport;
        }

        public IEnumerable<ProcessMessageDocument> ValidateCrossFormRules()
        {
            var response = new List<ProcessMessageDocument>();

            VRATOPSS000035(response);
            VRATOPSS000036(response);

            return response;
        }

        #region VR.ATO.PSS.000035
        /*   
            Rule ID: VR.ATO.PSS.000035
            
            IF PARENT RETURN <> “IITR”
             AND (
              [PSS20] <> NULLORBLANK OR 
              [PSS21] <> NULLORBLANK OR 
              [PSS31] <> NULLORBLANK OR 
              [PSS24] <> NULLORBLANK OR 
              [PSS25] <> NULLORBLANK OR 
              [PSS32] <> NULLORBLANK)
              RETURN VALIDATION MESSAGE
            ENDIF
        */
        private void VRATOPSS000035(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.RPPayerPSSeqNumContextCollection != null && ChildDocument
                                 .RPPayerPSSeqNumContextCollection.ToList().Any(p =>
                                     p.PSS20.HasValue || p.PSS21.HasValue || p.PSS24.HasValue ||
                                     p.PSS25.HasValue || p.PSS31.HasValue || p.PSS32.HasValue);
            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.PSS.000035",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Payment Type not applicable to Non-individual income tax return",
                    Location = "/xbrli:xbrl/xbrli:context",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.PSS.000035" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion 
        #region VR.ATO.PSS.000036
        /*   
            Rule ID: VR.ATO.PSS.000036
            WHERE IN CONTEXT (RP.Payer.{PSSeqNum})
            IF (RP.Payer.{PSSeqNum}.entity.identifier.TFN <> PARENT RETURN:RP.entity.identifier.TFN)
                RETURN VALIDATION MESSAGE
            ENDIF
        */
        private void VRATOPSS000036(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.RPPayerPSSeqNumContextCollection != null && ChildDocument
                                 .RPPayerPSSeqNumContextCollection.ToList().Any(p =>
                                     p.IdentifierTFN != ParentDocument.RPIdentifierTFN);
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.PSS.000036",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Your supplied TFN does not match the TFN supplied on the form it was submitted with",
                    Location = "/xbrli:xbrl/xbrli:context/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.PSS.000036" } }
                });
            }
        }
        #endregion 
    }
}