using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml;
using System.Text.RegularExpressions;
using Au.Gov.Sbr.Xbrl.Document;
using DataContracts;
using Au.Gov.Sbr.Xbrl.Metadata;

namespace Ato.EN.IntegrationServices.CodeGenerationIEE
{
    public class IEE2018Consumer
    {

        private static Regex StripWhitespace = new Regex(@"\s*");

        #region Helper/Validation Methods
        private static bool? ToBoolean(string str)
        {
            bool returnValue;
            string value = (str ?? "").Trim();
            if (value == "0" || value == "1")
                return Convert.ToBoolean(Convert.ToInt32(value));
            
            if (Boolean.TryParse(value, out returnValue))
                return returnValue;
            return null;
        }

        private static string GetValueOrEmpty(bool? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        private static string GetValueOrEmpty(DateTime? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        /// <summary>
        /// Derives the SBR element location.
        /// </summary>
        /// <param name="sbrElement">The SBR element.</param>
        /// <param name="addOccurrenceIndex">if set to <c>true</c> [add occurrence the occurrence index to the path].</param>
        /// <returns></returns>
        private static string DeriveSbrElementLocation(SBRElement sbrElement, bool addOccurrenceIndex = false)
        {
            if (sbrElement == null) return "/xbrli:xbrl";
            int xpathOccurrence = sbrElement.OccurrenceIndex + 1;
            string location;

            if (addOccurrenceIndex)
            {
                location = "/" + sbrElement.NamespacePrefix + ":" + sbrElement.Name + "[" +
                           xpathOccurrence + "]";
            }
            else
            {
                location = "/" + sbrElement.NamespacePrefix + ":" + sbrElement.Name;
            }

            if (sbrElement.IsInTuple)
            {
                location = DeriveSbrElementLocation(sbrElement.ParentTuple, true) + location;
            }
            else
            {
                location = "/xbrli:xbrl" + location;
            }

            return location;
        }

        private static bool IsMatch(int? field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(Convert.ToString(field.Value), expression, options);
        }

        private static bool IsMatch(string field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(field, expression, options);
        }

        public DateTime AsDate(string dateAsString)
        {
            DateTime response = DateTime.MinValue;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date;
            }

            return response;
        }

         #endregion

        public IEE2018 Consume(SBRDocument sbrDocument)
        {
            List<ProcessMessageDocument> errors;
            return Consume(sbrDocument, false, out errors);
        }

        /// <summary>
        /// Allows for validation of rules that can not be generated and must run at a lower level.
        /// A sub-class will need to be created with an override for this method - which does nothing in its base implementation.
        /// </summary>
        /// <param name="sbrDocument">The SBR document.</param>
        /// <param name="populatedReportElementsMap">The populated report elements map can be used to find data elements and tuples.</param>
        /// <param name="contextArray">The context array can be used to find context values.</param>
        /// <param name="errors">The errors collection is to be populated with validation results.</param>
        protected virtual void ValidateUnsupportedRules(SBRDocument sbrDocument, IDictionary<string, SBRElement> populatedReportElementsMap, SBRContext[] contextArray, List<ProcessMessageDocument> errors)
        {

        }

        public IEE2018 Consume(SBRDocument sbrDocument, bool validationMode, out List<ProcessMessageDocument> errors)
        {
            IEE2018 report;
            report = new IEE2018();
            errors = new List<ProcessMessageDocument>();

            SBRContext[] contextArray;
            IDictionary<string, SBRElement> populatedReportElementsMap;

            contextArray = sbrDocument.ContextsMap.Values.ToArray();
            populatedReportElementsMap = sbrDocument.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();

            if (validationMode) ValidateUnsupportedRules(sbrDocument, populatedReportElementsMap, contextArray, errors);

            /// <summary>
            /// Current element being extracted
            /// </summary>
            SBRElement currentValue = null;
            /// <summary>
            /// Current elements being extracted
            /// </summary>
            IList<SBRElement> currentValues = null;
            /// <summary>
            /// Current occurrances being extracted
            /// </summary>
            IList<SBRElement> currentOccurrences = null;
            /// <summary>
            /// Working Variable for date time variables
            /// </summary>
            DateTime currentDateTimeValue;
            /// <summary>
            /// Working Variable for boolean variables
            /// </summary>
            bool currentBooleanValue;
            /// <summary>
            /// Working Variable for decimal variables
            /// </summary>
            decimal currentDecimalValue;
            /// <summary>
            /// Working Variable for integer variables
            /// </summary>
            int currentIntValue;
            /// <summary>
            /// Working Variable for long variables
            /// </summary>
            long currentLongValue;
            XmlDocumentFragment currentSegment;
            XmlNamespaceManager currentNamespaceManager;
            XmlNode currentNode;
    
            #region sbrINTContext
            IEnumerable<SBRContext> sbrINTContexts = null;
            SBRContext sbrINTContext = null;
    
            sbrINTContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 1 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Intermediary")) ));
            Dictionary<string, IList<SBRElement>> sbrPopulatedINTElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrINTContexts != null && sbrINTContexts.Count() > 0)
            {
                sbrINTContext = sbrINTContexts.First();
        
                report.INTId = sbrINTContext.Id;
                report.INTIdentifier = sbrINTContext.EntityIdentifier;
                report.INTIdentifierScheme = sbrINTContext.EntityScheme;
        
                report.INTCount = sbrINTContexts.Count();
                report.INTExists = true;
                report.INTOccurrenceIndex = Array.IndexOf(contextArray, sbrINTContext) + 1;
                report.INTLastOccurrenceIndex = Array.IndexOf(contextArray, sbrINTContexts.Last()) + 1;
        

                if (report.INTIdentifierScheme == "http://www.ato.gov.au/abn") report.INTIdentifierABN = report.INTIdentifier;
    
                report.INTStartDate = ((SBRDurationPeriod)sbrINTContext.Period).DurationStartAsDateObject;
                report.INTEndDate = ((SBRDurationPeriod)sbrINTContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedINTElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrINTContexts.Count(); i++)
                {
                    sbrINTContext = sbrINTContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrINTContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedINTElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedINTElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedINTElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
        
                    IEnumerable<SBRElement> sbrDeclarationCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Declaration", out currentValue))
                        sbrDeclarationCollection = currentValue.Occurrences;
                    else
                        sbrDeclarationCollection = null;
            
                    #region sbrDeclarationCollection
            
                    if (sbrDeclarationCollection != null)
                    {
                        report.DeclarationCollectionExists = true;
                        report.DeclarationCollectionCount = sbrDeclarationCollection.Count();
                
                        IEnumerable<SBRElement> sbrDeclarationCollection_TrueAndCorrect_INTCollection;
                        sbrDeclarationCollection_TrueAndCorrect_INTCollection = sbrDeclarationCollection.Where(
                            ce => ce.ChildElements != null && 
                            ce.ChildElements.Any(te => te.Name == "Declaration.StatementType.Code" && te.Value == "TrueAndCorrect" &&
                            sbrINTContexts.Contains(te.Context)));
                        if (sbrDeclarationCollection_TrueAndCorrect_INTCollection != null && sbrDeclarationCollection_TrueAndCorrect_INTCollection.Count() > 0)
                        {
                            SBRElement sbrDeclarationCollection_TrueAndCorrect_INT;
                            sbrDeclarationCollection_TrueAndCorrect_INT = sbrDeclarationCollection_TrueAndCorrect_INTCollection.ElementAt(0);
                
                            IDictionary<string, SBRElement> sbrDeclarationCollection_TrueAndCorrect_INTTupleElementMap;
                            sbrDeclarationCollection_TrueAndCorrect_INTTupleElementMap = sbrDeclarationCollection_TrueAndCorrect_INT.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                            report.DeclarationCollection_TrueAndCorrect_INTExists = true;
                            report.DeclarationCollection_TrueAndCorrect_INTCount = sbrDeclarationCollection_TrueAndCorrect_INTCollection.Count();
                            report.DeclarationCollection_TrueAndCorrect_INTOccurrenceIndex = sbrDeclarationCollection_TrueAndCorrect_INT.OccurrenceIndex + 1;
                            report.DeclarationCollection_TrueAndCorrect_INTLastOccurrenceIndex = sbrDeclarationCollection_TrueAndCorrect_INTCollection.Last().OccurrenceIndex + 1;
                    
                            if (sbrDeclarationCollection_TrueAndCorrect_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Declaration.StatementType.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE159 = currentValue.Value;
                            }
                    
                            if (sbrDeclarationCollection_TrueAndCorrect_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Declaration.StatementAccepted.Indicator", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE179 = ToBoolean(currentValue.Value);
                            }
                    
                            if (sbrDeclarationCollection_TrueAndCorrect_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Declaration.Statement.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE180 = currentValue.Value;
                            }
                    
                            if (sbrDeclarationCollection_TrueAndCorrect_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Declaration.Signature.Date", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE181 = DateTime.Parse(currentValue.Value);
                            }
                    
                            if (sbrDeclarationCollection_TrueAndCorrect_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Declaration.SignatoryIdentifier.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE182 = currentValue.Value;
                            }
                
                            IEnumerable<SBRElement> sbrDeclaration_PersonUnstructuredNameCollection;
                            if (sbrDeclarationCollection_TrueAndCorrect_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonUnstructuredName", out currentValue))
                                sbrDeclaration_PersonUnstructuredNameCollection = currentValue.Occurrences;
                            else
                                sbrDeclaration_PersonUnstructuredNameCollection = null;
                    
                            #region sbrDeclaration_PersonUnstructuredNameCollection
                    
                            if (sbrDeclaration_PersonUnstructuredNameCollection != null)
                            {
                                report.Declaration_PersonUnstructuredNameCollectionExists = true;
                                report.Declaration_PersonUnstructuredNameCollectionCount = sbrDeclaration_PersonUnstructuredNameCollection.Count();
                        
                                IEnumerable<SBRElement> sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTCollection;
                                sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTCollection = sbrDeclaration_PersonUnstructuredNameCollection.Where(
                                    ce => ce.ChildElements != null && 
                                    ce.ChildElements.Any(te => te.Name == "PersonUnstructuredName.Usage.Code" && te.Value == "DeclarationSignatory" &&
                                    sbrINTContexts.Contains(te.Context)));
                                if (sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTCollection != null && sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTCollection.Count() > 0)
                                {
                                    SBRElement sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INT;
                                    sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INT = sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTCollection.ElementAt(0);
                        
                                    IDictionary<string, SBRElement> sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTTupleElementMap;
                                    sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTTupleElementMap = sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INT.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                        
                                    report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTExists = true;
                                    report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTCount = sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTCollection.Count();
                                    report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTOccurrenceIndex = sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INT.OccurrenceIndex + 1;
                                    report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTLastOccurrenceIndex = sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTCollection.Last().OccurrenceIndex + 1;
                            
                                    if (sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonUnstructuredName.Usage.Code", out currentValue))
                                    {
                                        if (!currentValue.IsNil) report.IEE183 = currentValue.Value;
                                    }
                            
                                    if (sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonUnstructuredName.FullName.Text", out currentValue))
                                    {
                                        if (!currentValue.IsNil) report.IEE184 = currentValue.Value;
                                    }
                                }  
                            } 
                            #endregion End of Tuple sbrDeclaration_PersonUnstructuredNameCollection
                            if (validationMode) ValidateINTDeclaration_PersonUnstructuredNameCollection(errors, sbrDeclaration_PersonUnstructuredNameCollection, sbrINTContexts, report);
                    
                        }  
                    } 
                    #endregion End of Tuple sbrDeclarationCollection
                    if (validationMode) ValidateINTDeclarationCollection(errors, sbrDeclarationCollection, sbrINTContexts, report);
            
            
                    if (sbrPopulatedINTElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Identifiers.TaxAgentNumber.Identifier", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IEE2 = currentValues[0].Value;
                    }
        
                    IEnumerable<SBRElement> sbrPersonUnstructuredNameCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonUnstructuredName", out currentValue))
                        sbrPersonUnstructuredNameCollection = currentValue.Occurrences;
                    else
                        sbrPersonUnstructuredNameCollection = null;
            
                    #region sbrPersonUnstructuredNameCollection
            
                    if (sbrPersonUnstructuredNameCollection != null)
                    {
                        report.PersonUnstructuredNameCollectionExists = true;
                        report.PersonUnstructuredNameCollectionCount = sbrPersonUnstructuredNameCollection.Count();
                
                        IEnumerable<SBRElement> sbrPersonUnstructuredNameCollection_Contact_INTCollection;
                        sbrPersonUnstructuredNameCollection_Contact_INTCollection = sbrPersonUnstructuredNameCollection.Where(
                            ce => ce.ChildElements != null && 
                            ce.ChildElements.Any(te => te.Name == "PersonUnstructuredName.Usage.Code" && te.Value == "Contact" &&
                            sbrINTContexts.Contains(te.Context)));
                        if (sbrPersonUnstructuredNameCollection_Contact_INTCollection != null && sbrPersonUnstructuredNameCollection_Contact_INTCollection.Count() > 0)
                        {
                            SBRElement sbrPersonUnstructuredNameCollection_Contact_INT;
                            sbrPersonUnstructuredNameCollection_Contact_INT = sbrPersonUnstructuredNameCollection_Contact_INTCollection.ElementAt(0);
                
                            IDictionary<string, SBRElement> sbrPersonUnstructuredNameCollection_Contact_INTTupleElementMap;
                            sbrPersonUnstructuredNameCollection_Contact_INTTupleElementMap = sbrPersonUnstructuredNameCollection_Contact_INT.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                            report.PersonUnstructuredNameCollection_Contact_INTExists = true;
                            report.PersonUnstructuredNameCollection_Contact_INTCount = sbrPersonUnstructuredNameCollection_Contact_INTCollection.Count();
                            report.PersonUnstructuredNameCollection_Contact_INTOccurrenceIndex = sbrPersonUnstructuredNameCollection_Contact_INT.OccurrenceIndex + 1;
                            report.PersonUnstructuredNameCollection_Contact_INTLastOccurrenceIndex = sbrPersonUnstructuredNameCollection_Contact_INTCollection.Last().OccurrenceIndex + 1;
                    
                            if (sbrPersonUnstructuredNameCollection_Contact_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonUnstructuredName.Usage.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE185 = currentValue.Value;
                            }
                    
                            if (sbrPersonUnstructuredNameCollection_Contact_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonUnstructuredName.FullName.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE186 = currentValue.Value;
                            }
                        }  
                    } 
                    #endregion End of Tuple sbrPersonUnstructuredNameCollection
                    if (validationMode) ValidateINTPersonUnstructuredNameCollection(errors, sbrPersonUnstructuredNameCollection, sbrINTContexts, report);
            
        
                    IEnumerable<SBRElement> sbrElectronicContactTelephoneCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}ElectronicContactTelephone", out currentValue))
                        sbrElectronicContactTelephoneCollection = currentValue.Occurrences;
                    else
                        sbrElectronicContactTelephoneCollection = null;
            
                    #region sbrElectronicContactTelephoneCollection
            
                    if (sbrElectronicContactTelephoneCollection != null)
                    {
                        report.ElectronicContactTelephoneCollectionExists = true;
                        report.ElectronicContactTelephoneCollectionCount = sbrElectronicContactTelephoneCollection.Count();
                
                        IEnumerable<SBRElement> sbrElectronicContactTelephoneCollection_03_INTCollection;
                        sbrElectronicContactTelephoneCollection_03_INTCollection = sbrElectronicContactTelephoneCollection.Where(
                            ce => ce.ChildElements != null && 
                            ce.ChildElements.Any(te => te.Name == "ElectronicContact.Telephone.Usage.Code" && te.Value == "03" &&
                            sbrINTContexts.Contains(te.Context)));
                        if (sbrElectronicContactTelephoneCollection_03_INTCollection != null && sbrElectronicContactTelephoneCollection_03_INTCollection.Count() > 0)
                        {
                            SBRElement sbrElectronicContactTelephoneCollection_03_INT;
                            sbrElectronicContactTelephoneCollection_03_INT = sbrElectronicContactTelephoneCollection_03_INTCollection.ElementAt(0);
                
                            IDictionary<string, SBRElement> sbrElectronicContactTelephoneCollection_03_INTTupleElementMap;
                            sbrElectronicContactTelephoneCollection_03_INTTupleElementMap = sbrElectronicContactTelephoneCollection_03_INT.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                            report.ElectronicContactTelephoneCollection_03_INTExists = true;
                            report.ElectronicContactTelephoneCollection_03_INTCount = sbrElectronicContactTelephoneCollection_03_INTCollection.Count();
                            report.ElectronicContactTelephoneCollection_03_INTOccurrenceIndex = sbrElectronicContactTelephoneCollection_03_INT.OccurrenceIndex + 1;
                            report.ElectronicContactTelephoneCollection_03_INTLastOccurrenceIndex = sbrElectronicContactTelephoneCollection_03_INTCollection.Last().OccurrenceIndex + 1;
                    
                            if (sbrElectronicContactTelephoneCollection_03_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}ElectronicContact.Telephone.Usage.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE187 = currentValue.Value;
                            }
                    
                            if (sbrElectronicContactTelephoneCollection_03_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}ElectronicContact.Telephone.ServiceLine.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE188 = currentValue.Value;
                            }
                    
                            if (sbrElectronicContactTelephoneCollection_03_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}ElectronicContact.Telephone.Area.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE189 = currentValue.Value;
                            }
                    
                            if (sbrElectronicContactTelephoneCollection_03_INTTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}ElectronicContact.Telephone.Minimal.Number", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE190 = currentValue.Value;
                            }
                        }  
                    } 
                    #endregion End of Tuple sbrElectronicContactTelephoneCollection
                    if (validationMode) ValidateINTElectronicContactTelephoneCollection(errors, sbrElectronicContactTelephoneCollection, sbrINTContexts, report);
            
                } 
            } 
            #endregion End of Context sbrINTContext
    
            #region sbrRPContext
            IEnumerable<SBRContext> sbrRPContexts = null;
            SBRContext sbrRPContext = null;
    
            sbrRPContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 1 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty")) ));
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPContexts != null && sbrRPContexts.Count() > 0)
            {
                sbrRPContext = sbrRPContexts.First();
        
                report.RPId = sbrRPContext.Id;
                report.RPIdentifier = sbrRPContext.EntityIdentifier;
                report.RPIdentifierScheme = sbrRPContext.EntityScheme;
        
                report.RPCount = sbrRPContexts.Count();
                report.RPExists = true;
                report.RPOccurrenceIndex = Array.IndexOf(contextArray, sbrRPContext) + 1;
                report.RPLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPContexts.Last()) + 1;
        

                if (report.RPIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPIdentifierTFN = report.RPIdentifier;
    
                report.RPStartDate = ((SBRDurationPeriod)sbrRPContext.Period).DurationStartAsDateObject;
                report.RPEndDate = ((SBRDurationPeriod)sbrRPContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPContexts.Count(); i++)
                {
                    sbrRPContext = sbrRPContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
                    report.IEE3 = report.RPIdentifier;
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Identifiers.AustralianBusinessNumber.Identifier", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IEE162 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Lodgment.PeriodYear.Year", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IEE1 = int.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}OrganisationDetails.OrganisationBranch.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IEE163 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Elections.InterposedEntityElectionOrRevocation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IEE116 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Report.CompletionHours.Number", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IEE94 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Party.Type.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IEE16 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Elections.Commencement.Date", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IEE49 = DateTime.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Elections.Revocation.Year", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IEE82 = int.Parse(currentValues[0].Value);
                    }
        
                    IEnumerable<SBRElement> sbrOrganisationNameDetailsCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}OrganisationNameDetails", out currentValue))
                        sbrOrganisationNameDetailsCollection = currentValue.Occurrences;
                    else
                        sbrOrganisationNameDetailsCollection = null;
            
                    #region sbrOrganisationNameDetailsCollection
            
                    if (sbrOrganisationNameDetailsCollection != null)
                    {
                        report.OrganisationNameDetailsCollectionExists = true;
                        report.OrganisationNameDetailsCollectionCount = sbrOrganisationNameDetailsCollection.Count();
                
                        IEnumerable<SBRElement> sbrOrganisationNameDetailsCollection_MN_C_RPCollection;
                        sbrOrganisationNameDetailsCollection_MN_C_RPCollection = sbrOrganisationNameDetailsCollection.Where(
                            ce => ce.ChildElements != null && 
                            ce.ChildElements.Any(te => te.Name == "OrganisationNameDetails.OrganisationalNameType.Code" && te.Value == "MN") &&
                            ce.ChildElements.Any(te => te.Name == "OrganisationNameDetails.Currency.Code" && te.Value == "C" &&
                            sbrRPContexts.Contains(te.Context)));
                        if (sbrOrganisationNameDetailsCollection_MN_C_RPCollection != null && sbrOrganisationNameDetailsCollection_MN_C_RPCollection.Count() > 0)
                        {
                            SBRElement sbrOrganisationNameDetailsCollection_MN_C_RP;
                            sbrOrganisationNameDetailsCollection_MN_C_RP = sbrOrganisationNameDetailsCollection_MN_C_RPCollection.ElementAt(0);
                
                            IDictionary<string, SBRElement> sbrOrganisationNameDetailsCollection_MN_C_RPTupleElementMap;
                            sbrOrganisationNameDetailsCollection_MN_C_RPTupleElementMap = sbrOrganisationNameDetailsCollection_MN_C_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                            report.OrganisationNameDetailsCollection_MN_C_RPExists = true;
                            report.OrganisationNameDetailsCollection_MN_C_RPCount = sbrOrganisationNameDetailsCollection_MN_C_RPCollection.Count();
                            report.OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex = sbrOrganisationNameDetailsCollection_MN_C_RP.OccurrenceIndex + 1;
                            report.OrganisationNameDetailsCollection_MN_C_RPLastOccurrenceIndex = sbrOrganisationNameDetailsCollection_MN_C_RPCollection.Last().OccurrenceIndex + 1;
                    
                            if (sbrOrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}OrganisationNameDetails.OrganisationalNameType.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE117 = currentValue.Value;
                            }
                    
                            if (sbrOrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}OrganisationNameDetails.Currency.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE164 = currentValue.Value;
                            }
                    
                            if (sbrOrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}OrganisationNameDetails.OrganisationalName.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE4 = currentValue.Value;
                            }
                        }  
                    } 
                    #endregion End of Tuple sbrOrganisationNameDetailsCollection
                    if (validationMode) ValidateRPOrganisationNameDetailsCollection(errors, sbrOrganisationNameDetailsCollection, sbrRPContexts, report);
            
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Identifiers.AustralianCompanyNumber.Identifier", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IEE100 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Identifiers.AustralianRegisteredBodyNumber.Identifier", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IEE101 = currentValues[0].Value;
                    }
        
                    IEnumerable<SBRElement> sbrAddressDetailsCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails", out currentValue))
                        sbrAddressDetailsCollection = currentValue.Occurrences;
                    else
                        sbrAddressDetailsCollection = null;
            
                    #region sbrAddressDetailsCollection
            
                    if (sbrAddressDetailsCollection != null)
                    {
                        report.AddressDetailsCollectionExists = true;
                        report.AddressDetailsCollectionCount = sbrAddressDetailsCollection.Count();
                
                        IEnumerable<SBRElement> sbrAddressDetailsCollection_POS_C_RPCollection;
                        sbrAddressDetailsCollection_POS_C_RPCollection = sbrAddressDetailsCollection.Where(
                            ce => ce.ChildElements != null && 
                            ce.ChildElements.Any(te => te.Name == "AddressDetails.Usage.Code" && te.Value == "POS") &&
                            ce.ChildElements.Any(te => te.Name == "AddressDetails.Currency.Code" && te.Value == "C" &&
                            sbrRPContexts.Contains(te.Context)));
                        if (sbrAddressDetailsCollection_POS_C_RPCollection != null && sbrAddressDetailsCollection_POS_C_RPCollection.Count() > 0)
                        {
                            SBRElement sbrAddressDetailsCollection_POS_C_RP;
                            sbrAddressDetailsCollection_POS_C_RP = sbrAddressDetailsCollection_POS_C_RPCollection.ElementAt(0);
                
                            IDictionary<string, SBRElement> sbrAddressDetailsCollection_POS_C_RPTupleElementMap;
                            sbrAddressDetailsCollection_POS_C_RPTupleElementMap = sbrAddressDetailsCollection_POS_C_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                            report.AddressDetailsCollection_POS_C_RPExists = true;
                            report.AddressDetailsCollection_POS_C_RPCount = sbrAddressDetailsCollection_POS_C_RPCollection.Count();
                            report.AddressDetailsCollection_POS_C_RPOccurrenceIndex = sbrAddressDetailsCollection_POS_C_RP.OccurrenceIndex + 1;
                            report.AddressDetailsCollection_POS_C_RPLastOccurrenceIndex = sbrAddressDetailsCollection_POS_C_RPCollection.Last().OccurrenceIndex + 1;
                    
                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.OverseasAddress.Indicator", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE121 = ToBoolean(currentValue.Value);
                            }
                    
                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Usage.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE104 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Currency.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE150 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line1.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE5 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line2.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE122 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line3.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE123 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line4.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE124 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.LocalityName.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE6 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Postcode.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE8 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.StateOrTerritory.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE7 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.CountryName.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE9 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Country.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE111 = currentValue.Value;
                            }
                        }  
                    } 
                    #endregion End of Tuple sbrAddressDetailsCollection
                    if (validationMode) ValidateRPAddressDetailsCollection(errors, sbrAddressDetailsCollection, sbrRPContexts, report);
            
            
                    #region sbrOrganisationNameDetailsCollection
            
                    if (sbrOrganisationNameDetailsCollection != null)
                    {
                
                        IEnumerable<SBRElement> sbrOrganisationNameDetailsCollection_MN_P_RPCollection;
                        sbrOrganisationNameDetailsCollection_MN_P_RPCollection = sbrOrganisationNameDetailsCollection.Where(
                            ce => ce.ChildElements != null && 
                            ce.ChildElements.Any(te => te.Name == "OrganisationNameDetails.OrganisationalNameType.Code" && te.Value == "MN") &&
                            ce.ChildElements.Any(te => te.Name == "OrganisationNameDetails.Currency.Code" && te.Value == "P" &&
                            sbrRPContexts.Contains(te.Context)));
                        if (sbrOrganisationNameDetailsCollection_MN_P_RPCollection != null && sbrOrganisationNameDetailsCollection_MN_P_RPCollection.Count() > 0)
                        {
                            SBRElement sbrOrganisationNameDetailsCollection_MN_P_RP;
                            sbrOrganisationNameDetailsCollection_MN_P_RP = sbrOrganisationNameDetailsCollection_MN_P_RPCollection.ElementAt(0);
                
                            IDictionary<string, SBRElement> sbrOrganisationNameDetailsCollection_MN_P_RPTupleElementMap;
                            sbrOrganisationNameDetailsCollection_MN_P_RPTupleElementMap = sbrOrganisationNameDetailsCollection_MN_P_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                            report.OrganisationNameDetailsCollection_MN_P_RPExists = true;
                            report.OrganisationNameDetailsCollection_MN_P_RPCount = sbrOrganisationNameDetailsCollection_MN_P_RPCollection.Count();
                            report.OrganisationNameDetailsCollection_MN_P_RPOccurrenceIndex = sbrOrganisationNameDetailsCollection_MN_P_RP.OccurrenceIndex + 1;
                            report.OrganisationNameDetailsCollection_MN_P_RPLastOccurrenceIndex = sbrOrganisationNameDetailsCollection_MN_P_RPCollection.Last().OccurrenceIndex + 1;
                    
                            if (sbrOrganisationNameDetailsCollection_MN_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}OrganisationNameDetails.OrganisationalNameType.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE118 = currentValue.Value;
                            }
                    
                            if (sbrOrganisationNameDetailsCollection_MN_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}OrganisationNameDetails.Currency.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE165 = currentValue.Value;
                            }
                    
                            if (sbrOrganisationNameDetailsCollection_MN_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}OrganisationNameDetails.OrganisationalName.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE10 = currentValue.Value;
                            }
                        }  
                    } 
                    #endregion End of Tuple sbrOrganisationNameDetailsCollection
                    if (validationMode) ValidateRPOrganisationNameDetailsCollection1(errors, sbrOrganisationNameDetailsCollection, sbrRPContexts, report);
            
            
                    #region sbrAddressDetailsCollection
            
                    if (sbrAddressDetailsCollection != null)
                    {
                
                        IEnumerable<SBRElement> sbrAddressDetailsCollection_POS_P_RPCollection;
                        sbrAddressDetailsCollection_POS_P_RPCollection = sbrAddressDetailsCollection.Where(
                            ce => ce.ChildElements != null && 
                            ce.ChildElements.Any(te => te.Name == "AddressDetails.Usage.Code" && te.Value == "POS") &&
                            ce.ChildElements.Any(te => te.Name == "AddressDetails.Currency.Code" && te.Value == "P" &&
                            sbrRPContexts.Contains(te.Context)));
                        if (sbrAddressDetailsCollection_POS_P_RPCollection != null && sbrAddressDetailsCollection_POS_P_RPCollection.Count() > 0)
                        {
                            SBRElement sbrAddressDetailsCollection_POS_P_RP;
                            sbrAddressDetailsCollection_POS_P_RP = sbrAddressDetailsCollection_POS_P_RPCollection.ElementAt(0);
                
                            IDictionary<string, SBRElement> sbrAddressDetailsCollection_POS_P_RPTupleElementMap;
                            sbrAddressDetailsCollection_POS_P_RPTupleElementMap = sbrAddressDetailsCollection_POS_P_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                            report.AddressDetailsCollection_POS_P_RPExists = true;
                            report.AddressDetailsCollection_POS_P_RPCount = sbrAddressDetailsCollection_POS_P_RPCollection.Count();
                            report.AddressDetailsCollection_POS_P_RPOccurrenceIndex = sbrAddressDetailsCollection_POS_P_RP.OccurrenceIndex + 1;
                            report.AddressDetailsCollection_POS_P_RPLastOccurrenceIndex = sbrAddressDetailsCollection_POS_P_RPCollection.Last().OccurrenceIndex + 1;
                    
                            if (sbrAddressDetailsCollection_POS_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.OverseasAddress.Indicator", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE125 = ToBoolean(currentValue.Value);
                            }
                    
                            if (sbrAddressDetailsCollection_POS_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Usage.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE105 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Currency.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE151 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line1.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE11 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line2.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE126 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line3.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE127 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line4.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE128 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.LocalityName.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE12 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Postcode.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE14 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.StateOrTerritory.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE13 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.CountryName.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE15 = currentValue.Value;
                            }
                    
                            if (sbrAddressDetailsCollection_POS_P_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Country.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE112 = currentValue.Value;
                            }
                        }  
                    } 
                    #endregion End of Tuple sbrAddressDetailsCollection
                    if (validationMode) ValidateRPAddressDetailsCollection1(errors, sbrAddressDetailsCollection, sbrRPContexts, report);
            
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Residency.CentralManagementAndControlOutsideAustralia.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IEE17 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Residency.CentralManagementAndControlOutsideAustraliaFullPeriod.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IEE18 = ToBoolean(currentValues[0].Value);
                    }
        
                    IEnumerable<SBRElement> sbrPeriodDetailsCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PeriodDetails", out currentValue))
                        sbrPeriodDetailsCollection = currentValue.Occurrences;
                    else
                        sbrPeriodDetailsCollection = null;
            
                    #region sbrPeriodDetailsCollection
            
                    if (sbrPeriodDetailsCollection != null)
                    {
                        report.PeriodDetailsCollectionExists = true;
                        report.PeriodDetailsCollectionCount = sbrPeriodDetailsCollection.Count();
                        if (sbrPeriodDetailsCollection.Count() > 0)
                        {
                    
                            IEnumerable<SBRElement> sbrPeriodDetailsCollection_ControlOutsideAustraliaPartPeriod_RPCollection;
                            sbrPeriodDetailsCollection_ControlOutsideAustraliaPartPeriod_RPCollection = sbrPeriodDetailsCollection.Where(
                                ce => ce.ChildElements != null && 
                                ce.ChildElements.Any(te => te.Name == "Period.Type.Code" && te.Value == "ControlOutsideAustraliaPartPeriod" &&
                                sbrRPContexts.Contains(te.Context)));
                
                            report.PeriodDetailsControlOutsideAustraliaPartPeriodRPCollection = new List<IEE2018.PeriodDetailsControlOutsideAustraliaPartPeriodRP>();
                
                            for (int explicitTupleIndex = 0; explicitTupleIndex < sbrPeriodDetailsCollection_ControlOutsideAustraliaPartPeriod_RPCollection.Count(); explicitTupleIndex++)
                            {
                                SBRElement sbrPeriodDetails;
                                sbrPeriodDetails = sbrPeriodDetailsCollection_ControlOutsideAustraliaPartPeriod_RPCollection.ElementAt(explicitTupleIndex);
                
                                IDictionary<string, SBRElement> sbrPeriodDetailsCollection_ControlOutsideAustraliaPartPeriod_RPTupleElementMap;
                                sbrPeriodDetailsCollection_ControlOutsideAustraliaPartPeriod_RPTupleElementMap = sbrPeriodDetails.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                                IEE2018.PeriodDetailsControlOutsideAustraliaPartPeriodRP periodDetailsControlOutsideAustraliaPartPeriod = new IEE2018.PeriodDetailsControlOutsideAustraliaPartPeriodRP();
                                report.PeriodDetailsControlOutsideAustraliaPartPeriodRPCollection.Add(periodDetailsControlOutsideAustraliaPartPeriod);
                        
                                if (sbrPeriodDetailsCollection_ControlOutsideAustraliaPartPeriod_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Period.Type.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) periodDetailsControlOutsideAustraliaPartPeriod.IEE97 = currentValue.Value;
                                }
                        
                                if (sbrPeriodDetailsCollection_ControlOutsideAustraliaPartPeriod_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Period.Start.Date", out currentValue))
                                {
                                    if (!currentValue.IsNil) periodDetailsControlOutsideAustraliaPartPeriod.IEE19 = DateTime.Parse(currentValue.Value);
                                }
                        
                                if (sbrPeriodDetailsCollection_ControlOutsideAustraliaPartPeriod_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Period.End.Date", out currentValue))
                                {
                                    if (!currentValue.IsNil) periodDetailsControlOutsideAustraliaPartPeriod.IEE20 = DateTime.Parse(currentValue.Value);
                                }
                            }
                        }
                    } 
                    #endregion End of Tuple sbrPeriodDetailsCollection
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Residency.TaxPurposesOrganisationStatus.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IEE21 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Residency.NonResidentFullPeriod.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IEE22 = ToBoolean(currentValues[0].Value);
                    }
            
                    #region sbrPeriodDetailsCollection
            
                    if (sbrPeriodDetailsCollection != null)
                    {
                        if (sbrPeriodDetailsCollection.Count() > 0)
                        {
                    
                            IEnumerable<SBRElement> sbrPeriodDetailsCollection_NonResidentPartPeriod_RPCollection;
                            sbrPeriodDetailsCollection_NonResidentPartPeriod_RPCollection = sbrPeriodDetailsCollection.Where(
                                ce => ce.ChildElements != null && 
                                ce.ChildElements.Any(te => te.Name == "Period.Type.Code" && te.Value == "NonResidentPartPeriod" &&
                                sbrRPContexts.Contains(te.Context)));
                
                            report.PeriodDetailsNonResidentPartPeriodRPCollection = new List<IEE2018.PeriodDetailsNonResidentPartPeriodRP>();
                
                            for (int explicitTupleIndex = 0; explicitTupleIndex < sbrPeriodDetailsCollection_NonResidentPartPeriod_RPCollection.Count(); explicitTupleIndex++)
                            {
                                SBRElement sbrPeriodDetails;
                                sbrPeriodDetails = sbrPeriodDetailsCollection_NonResidentPartPeriod_RPCollection.ElementAt(explicitTupleIndex);
                
                                IDictionary<string, SBRElement> sbrPeriodDetailsCollection_NonResidentPartPeriod_RPTupleElementMap;
                                sbrPeriodDetailsCollection_NonResidentPartPeriod_RPTupleElementMap = sbrPeriodDetails.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                                IEE2018.PeriodDetailsNonResidentPartPeriodRP periodDetailsNonResidentPartPeriod = new IEE2018.PeriodDetailsNonResidentPartPeriodRP();
                                report.PeriodDetailsNonResidentPartPeriodRPCollection.Add(periodDetailsNonResidentPartPeriod);
                        
                                if (sbrPeriodDetailsCollection_NonResidentPartPeriod_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Period.Type.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) periodDetailsNonResidentPartPeriod.IEE98 = currentValue.Value;
                                }
                        
                                if (sbrPeriodDetailsCollection_NonResidentPartPeriod_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Period.Start.Date", out currentValue))
                                {
                                    if (!currentValue.IsNil) periodDetailsNonResidentPartPeriod.IEE23 = DateTime.Parse(currentValue.Value);
                                }
                        
                                if (sbrPeriodDetailsCollection_NonResidentPartPeriod_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Period.End.Date", out currentValue))
                                {
                                    if (!currentValue.IsNil) periodDetailsNonResidentPartPeriod.IEE24 = DateTime.Parse(currentValue.Value);
                                }
                            }
                        }
                    } 
                    #endregion End of Tuple sbrPeriodDetailsCollection
        
                    IEnumerable<SBRElement> sbrTrusteeOrPartnerCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}TrusteeOrPartner", out currentValue))
                        sbrTrusteeOrPartnerCollection = currentValue.Occurrences;
                    else
                        sbrTrusteeOrPartnerCollection = null;
            
                    #region sbrTrusteeOrPartnerCollection
            
                    if (sbrTrusteeOrPartnerCollection != null)
                    {
                        report.TrusteeOrPartnerCollectionExists = true;
                        report.TrusteeOrPartnerCollectionCount = sbrTrusteeOrPartnerCollection.Count();
                        if (sbrTrusteeOrPartnerCollection != null && sbrTrusteeOrPartnerCollection.Count() > 0)
                        {
                            report.TrusteeOrPartnerCollection = new List<IEE2018.TrusteeOrPartner>();
                            for (int tupleIndex = 0; tupleIndex < sbrTrusteeOrPartnerCollection.Count(); tupleIndex++)
                            {
                                SBRElement sbrTrusteeOrPartner;
                                sbrTrusteeOrPartner = sbrTrusteeOrPartnerCollection.ElementAt(tupleIndex);
                
                                IDictionary<string, SBRElement> sbrTrusteeOrPartnerTupleElementMap;
                                sbrTrusteeOrPartnerTupleElementMap = sbrTrusteeOrPartner.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                                IEE2018.TrusteeOrPartner trusteeOrPartner = new IEE2018.TrusteeOrPartner();
                                report.TrusteeOrPartnerCollection.Add(trusteeOrPartner);
                                trusteeOrPartner.OccurrenceIndex = tupleIndex + 1;
                        
                                if (sbrTrusteeOrPartnerTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Identifiers.AustralianCompanyNumber.Identifier", out currentValue))
                                {
                                    if (!currentValue.IsNil) trusteeOrPartner.IEE34 = currentValue.Value;
                                }
                        
                                if (sbrTrusteeOrPartnerTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Identifiers.AustralianRegisteredBodyNumber.Identifier", out currentValue))
                                {
                                    if (!currentValue.IsNil) trusteeOrPartner.IEE95 = currentValue.Value;
                                }
                        
                                if (sbrTrusteeOrPartnerTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Identifiers.TaxFileNumber.Identifier", out currentValue))
                                {
                                    if (!currentValue.IsNil) trusteeOrPartner.IEE25 = currentValue.Value;
                                }
                        
                                if (sbrTrusteeOrPartnerTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Identifiers.TaxFileNumberInexistent.Indicator", out currentValue))
                                {
                                    if (!currentValue.IsNil) trusteeOrPartner.IEE26 = ToBoolean(currentValue.Value);
                                }
                        
                                if (sbrTrusteeOrPartnerTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Residency.NonResidentFullPeriod.Indicator", out currentValue))
                                {
                                    if (!currentValue.IsNil) trusteeOrPartner.IEE42 = ToBoolean(currentValue.Value);
                                }
                        
                                if (sbrTrusteeOrPartnerTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Residency.TaxPurposesPersonStatus.Indicator", out currentValue))
                                {
                                    if (!currentValue.IsNil) trusteeOrPartner.IEE41 = ToBoolean(currentValue.Value);
                                }
                    
                                IEnumerable<SBRElement> sbrTrusteeOrPartner_OrganisationNameDetailsCollection;
                                if (sbrTrusteeOrPartnerTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}OrganisationNameDetails", out currentValue))
                                    sbrTrusteeOrPartner_OrganisationNameDetailsCollection = currentValue.Occurrences;
                                else
                                    sbrTrusteeOrPartner_OrganisationNameDetailsCollection = null;
                        
                                #region sbrTrusteeOrPartner_OrganisationNameDetailsCollection
                        
                                if (sbrTrusteeOrPartner_OrganisationNameDetailsCollection != null)
                                {
                                    trusteeOrPartner.TrusteeOrPartner_OrganisationNameDetailsCollectionExists = true;
                                    trusteeOrPartner.TrusteeOrPartner_OrganisationNameDetailsCollectionCount = sbrTrusteeOrPartner_OrganisationNameDetailsCollection.Count();
                            
                                    IEnumerable<SBRElement> sbrTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPCollection;
                                    sbrTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPCollection = sbrTrusteeOrPartner_OrganisationNameDetailsCollection.Where(
                                        ce => ce.ChildElements != null && 
                                        ce.ChildElements.Any(te => te.Name == "OrganisationNameDetails.OrganisationalNameType.Code" && te.Value == "MN") &&
                                        ce.ChildElements.Any(te => te.Name == "OrganisationNameDetails.Currency.Code" && te.Value == "C" &&
                                        sbrRPContexts.Contains(te.Context)));
                                    if (sbrTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPCollection != null && sbrTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPCollection.Count() > 0)
                                    {
                                        SBRElement sbrTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RP;
                                        sbrTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RP = sbrTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPCollection.ElementAt(0);
                            
                                        IDictionary<string, SBRElement> sbrTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap;
                                        sbrTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap = sbrTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                            
                                        trusteeOrPartner.TrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPExists = true;
                                        trusteeOrPartner.TrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPCount = sbrTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPCollection.Count();
                                        trusteeOrPartner.TrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex = sbrTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RP.OccurrenceIndex + 1;
                                        trusteeOrPartner.TrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPLastOccurrenceIndex = sbrTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPCollection.Last().OccurrenceIndex + 1;
                                
                                        if (sbrTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}OrganisationNameDetails.OrganisationalNameType.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trusteeOrPartner.IEE167 = currentValue.Value;
                                        }
                                
                                        if (sbrTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}OrganisationNameDetails.Currency.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trusteeOrPartner.IEE166 = currentValue.Value;
                                        }
                                
                                        if (sbrTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}OrganisationNameDetails.OrganisationalName.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trusteeOrPartner.IEE32 = currentValue.Value;
                                        }
                                    }  
                                } 
                                #endregion End of Tuple sbrTrusteeOrPartner_OrganisationNameDetailsCollection
                                if (validationMode) ValidateRPTrusteeOrPartner_OrganisationNameDetailsCollection(errors, sbrTrusteeOrPartner_OrganisationNameDetailsCollection, sbrRPContexts, report);
                        
                    
                                IEnumerable<SBRElement> sbrTrusteeOrPartner_PersonNameDetailsCollection;
                                if (sbrTrusteeOrPartnerTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails", out currentValue))
                                    sbrTrusteeOrPartner_PersonNameDetailsCollection = currentValue.Occurrences;
                                else
                                    sbrTrusteeOrPartner_PersonNameDetailsCollection = null;
                        
                                #region sbrTrusteeOrPartner_PersonNameDetailsCollection
                        
                                if (sbrTrusteeOrPartner_PersonNameDetailsCollection != null)
                                {
                                    trusteeOrPartner.TrusteeOrPartner_PersonNameDetailsCollectionExists = true;
                                    trusteeOrPartner.TrusteeOrPartner_PersonNameDetailsCollectionCount = sbrTrusteeOrPartner_PersonNameDetailsCollection.Count();
                            
                                    IEnumerable<SBRElement> sbrTrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPCollection;
                                    sbrTrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPCollection = sbrTrusteeOrPartner_PersonNameDetailsCollection.Where(
                                        ce => ce.ChildElements != null && 
                                        ce.ChildElements.Any(te => te.Name == "PersonNameDetails.PersonNameType.Code" && te.Value == "LGL") &&
                                        ce.ChildElements.Any(te => te.Name == "PersonNameDetails.Usage.Code" && te.Value == "Identifier") &&
                                        ce.ChildElements.Any(te => te.Name == "PersonNameDetails.Currency.Code" && te.Value == "C" &&
                                        sbrRPContexts.Contains(te.Context)));
                                    if (sbrTrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPCollection != null && sbrTrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPCollection.Count() > 0)
                                    {
                                        SBRElement sbrTrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RP;
                                        sbrTrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RP = sbrTrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPCollection.ElementAt(0);
                            
                                        IDictionary<string, SBRElement> sbrTrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPTupleElementMap;
                                        sbrTrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPTupleElementMap = sbrTrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                            
                                        trusteeOrPartner.TrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPExists = true;
                                        trusteeOrPartner.TrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPCount = sbrTrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPCollection.Count();
                                        trusteeOrPartner.TrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex = sbrTrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RP.OccurrenceIndex + 1;
                                        trusteeOrPartner.TrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPLastOccurrenceIndex = sbrTrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPCollection.Last().OccurrenceIndex + 1;
                                
                                        if (sbrTrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.PersonNameType.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trusteeOrPartner.IEE158 = currentValue.Value;
                                        }
                                
                                        if (sbrTrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.Usage.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trusteeOrPartner.IEE168 = currentValue.Value;
                                        }
                                
                                        if (sbrTrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.Currency.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trusteeOrPartner.IEE176 = currentValue.Value;
                                        }
                                
                                        if (sbrTrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.Title.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trusteeOrPartner.IEE27 = currentValue.Value;
                                        }
                                
                                        if (sbrTrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.NameSuffix.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trusteeOrPartner.IEE29 = currentValue.Value;
                                        }
                                
                                        if (sbrTrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.FamilyName.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trusteeOrPartner.IEE28 = currentValue.Value;
                                        }
                                
                                        if (sbrTrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.GivenName.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trusteeOrPartner.IEE30 = currentValue.Value;
                                        }
                                
                                        if (sbrTrusteeOrPartner_PersonNameDetailsCollection_LGL_Identifier_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.OtherGivenName.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trusteeOrPartner.IEE31 = currentValue.Value;
                                        }
                                    }  
                                } 
                                #endregion End of Tuple sbrTrusteeOrPartner_PersonNameDetailsCollection
                                if (validationMode) ValidateRPTrusteeOrPartner_PersonNameDetailsCollection(errors, sbrTrusteeOrPartner_PersonNameDetailsCollection, sbrRPContexts, report);
                        
                    
                                IEnumerable<SBRElement> sbrTrusteeOrPartner_AddressDetailsCollection;
                                if (sbrTrusteeOrPartnerTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails", out currentValue))
                                    sbrTrusteeOrPartner_AddressDetailsCollection = currentValue.Occurrences;
                                else
                                    sbrTrusteeOrPartner_AddressDetailsCollection = null;
                        
                                #region sbrTrusteeOrPartner_AddressDetailsCollection
                        
                                if (sbrTrusteeOrPartner_AddressDetailsCollection != null)
                                {
                                    trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollectionExists = true;
                                    trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollectionCount = sbrTrusteeOrPartner_AddressDetailsCollection.Count();
                            
                                    IEnumerable<SBRElement> sbrTrusteeOrPartner_AddressDetailsCollection_POS_C_RPCollection;
                                    sbrTrusteeOrPartner_AddressDetailsCollection_POS_C_RPCollection = sbrTrusteeOrPartner_AddressDetailsCollection.Where(
                                        ce => ce.ChildElements != null && 
                                        ce.ChildElements.Any(te => te.Name == "AddressDetails.Usage.Code" && te.Value == "POS") &&
                                        ce.ChildElements.Any(te => te.Name == "AddressDetails.Currency.Code" && te.Value == "C" &&
                                        sbrRPContexts.Contains(te.Context)));
                                    if (sbrTrusteeOrPartner_AddressDetailsCollection_POS_C_RPCollection != null && sbrTrusteeOrPartner_AddressDetailsCollection_POS_C_RPCollection.Count() > 0)
                                    {
                                        SBRElement sbrTrusteeOrPartner_AddressDetailsCollection_POS_C_RP;
                                        sbrTrusteeOrPartner_AddressDetailsCollection_POS_C_RP = sbrTrusteeOrPartner_AddressDetailsCollection_POS_C_RPCollection.ElementAt(0);
                            
                                        IDictionary<string, SBRElement> sbrTrusteeOrPartner_AddressDetailsCollection_POS_C_RPTupleElementMap;
                                        sbrTrusteeOrPartner_AddressDetailsCollection_POS_C_RPTupleElementMap = sbrTrusteeOrPartner_AddressDetailsCollection_POS_C_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                            
                                        trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollection_POS_C_RPExists = true;
                                        trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollection_POS_C_RPCount = sbrTrusteeOrPartner_AddressDetailsCollection_POS_C_RPCollection.Count();
                                        trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollection_POS_C_RPOccurrenceIndex = sbrTrusteeOrPartner_AddressDetailsCollection_POS_C_RP.OccurrenceIndex + 1;
                                        trusteeOrPartner.TrusteeOrPartner_AddressDetailsCollection_POS_C_RPLastOccurrenceIndex = sbrTrusteeOrPartner_AddressDetailsCollection_POS_C_RPCollection.Last().OccurrenceIndex + 1;
                                
                                        if (sbrTrusteeOrPartner_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.OverseasAddress.Indicator", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trusteeOrPartner.IEE129 = ToBoolean(currentValue.Value);
                                        }
                                
                                        if (sbrTrusteeOrPartner_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Usage.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trusteeOrPartner.IEE106 = currentValue.Value;
                                        }
                                
                                        if (sbrTrusteeOrPartner_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Currency.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trusteeOrPartner.IEE130 = currentValue.Value;
                                        }
                                
                                        if (sbrTrusteeOrPartner_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line1.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trusteeOrPartner.IEE35 = currentValue.Value;
                                        }
                                
                                        if (sbrTrusteeOrPartner_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line2.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trusteeOrPartner.IEE36 = currentValue.Value;
                                        }
                                
                                        if (sbrTrusteeOrPartner_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line3.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trusteeOrPartner.IEE131 = currentValue.Value;
                                        }
                                
                                        if (sbrTrusteeOrPartner_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line4.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trusteeOrPartner.IEE132 = currentValue.Value;
                                        }
                                
                                        if (sbrTrusteeOrPartner_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.LocalityName.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trusteeOrPartner.IEE37 = currentValue.Value;
                                        }
                                
                                        if (sbrTrusteeOrPartner_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Postcode.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trusteeOrPartner.IEE39 = currentValue.Value;
                                        }
                                
                                        if (sbrTrusteeOrPartner_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.StateOrTerritory.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trusteeOrPartner.IEE38 = currentValue.Value;
                                        }
                                
                                        if (sbrTrusteeOrPartner_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.CountryName.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trusteeOrPartner.IEE40 = currentValue.Value;
                                        }
                                
                                        if (sbrTrusteeOrPartner_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Country.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) trusteeOrPartner.IEE113 = currentValue.Value;
                                        }
                                    }  
                                } 
                                #endregion End of Tuple sbrTrusteeOrPartner_AddressDetailsCollection
                                if (validationMode) ValidateRPTrusteeOrPartner_AddressDetailsCollection(errors, sbrTrusteeOrPartner_AddressDetailsCollection, sbrRPContexts, report);
                        
                    
                                IEnumerable<SBRElement> sbrTrusteeOrPartner_PeriodDetailsCollection;
                                if (sbrTrusteeOrPartnerTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PeriodDetails", out currentValue))
                                    sbrTrusteeOrPartner_PeriodDetailsCollection = currentValue.Occurrences;
                                else
                                    sbrTrusteeOrPartner_PeriodDetailsCollection = null;
                        
                                #region sbrTrusteeOrPartner_PeriodDetailsCollection
                        
                                if (sbrTrusteeOrPartner_PeriodDetailsCollection != null)
                                {
                                    trusteeOrPartner.TrusteeOrPartner_PeriodDetailsCollectionExists = true;
                                    trusteeOrPartner.TrusteeOrPartner_PeriodDetailsCollectionCount = sbrTrusteeOrPartner_PeriodDetailsCollection.Count();
                                    if (sbrTrusteeOrPartner_PeriodDetailsCollection.Count() > 0)
                                    {
                                
                                        IEnumerable<SBRElement> sbrTrusteeOrPartner_PeriodDetailsCollection_NonResidentPartPeriod_RPCollection;
                                        sbrTrusteeOrPartner_PeriodDetailsCollection_NonResidentPartPeriod_RPCollection = sbrTrusteeOrPartner_PeriodDetailsCollection.Where(
                                            ce => ce.ChildElements != null && 
                                            ce.ChildElements.Any(te => te.Name == "Period.Type.Code" && te.Value == "NonResidentPartPeriod" &&
                                            sbrRPContexts.Contains(te.Context)));
                            
                                        trusteeOrPartner.TrusteeOrPartner_PeriodDetailsNonResidentPartPeriodRPCollection = new List<IEE2018.TrusteeOrPartner.TrusteeOrPartner_PeriodDetailsNonResidentPartPeriodRP>();
                            
                                        for (int explicitTupleIndex = 0; explicitTupleIndex < sbrTrusteeOrPartner_PeriodDetailsCollection_NonResidentPartPeriod_RPCollection.Count(); explicitTupleIndex++)
                                        {
                                            SBRElement sbrTrusteeOrPartner_PeriodDetails;
                                            sbrTrusteeOrPartner_PeriodDetails = sbrTrusteeOrPartner_PeriodDetailsCollection_NonResidentPartPeriod_RPCollection.ElementAt(explicitTupleIndex);
                            
                                            IDictionary<string, SBRElement> sbrTrusteeOrPartner_PeriodDetailsCollection_NonResidentPartPeriod_RPTupleElementMap;
                                            sbrTrusteeOrPartner_PeriodDetailsCollection_NonResidentPartPeriod_RPTupleElementMap = sbrTrusteeOrPartner_PeriodDetails.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                            
                                            IEE2018.TrusteeOrPartner.TrusteeOrPartner_PeriodDetailsNonResidentPartPeriodRP periodDetailsNonResidentPartPeriod = new IEE2018.TrusteeOrPartner.TrusteeOrPartner_PeriodDetailsNonResidentPartPeriodRP();
                                            trusteeOrPartner.TrusteeOrPartner_PeriodDetailsNonResidentPartPeriodRPCollection.Add(periodDetailsNonResidentPartPeriod);
                                    
                                            if (sbrTrusteeOrPartner_PeriodDetailsCollection_NonResidentPartPeriod_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Period.Type.Code", out currentValue))
                                            {
                                                if (!currentValue.IsNil) periodDetailsNonResidentPartPeriod.IEE102 = currentValue.Value;
                                            }
                                    
                                            if (sbrTrusteeOrPartner_PeriodDetailsCollection_NonResidentPartPeriod_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Period.Start.Date", out currentValue))
                                            {
                                                if (!currentValue.IsNil) periodDetailsNonResidentPartPeriod.IEE43 = DateTime.Parse(currentValue.Value);
                                            }
                                    
                                            if (sbrTrusteeOrPartner_PeriodDetailsCollection_NonResidentPartPeriod_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Period.End.Date", out currentValue))
                                            {
                                                if (!currentValue.IsNil) periodDetailsNonResidentPartPeriod.IEE44 = DateTime.Parse(currentValue.Value);
                                            }
                                        }
                                    }
                                } 
                                #endregion End of Tuple sbrTrusteeOrPartner_PeriodDetailsCollection
                    } 
                    #endregion End of Tuple sbrTrusteeOrPartnerCollection
                        } 
                    } 
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Elections.InterposedEntityElectionStatus.Year", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IEE45 = int.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Elections.DateSpecified.Date", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IEE46 = DateTime.Parse(currentValues[0].Value);
                    }
            
                    #region sbrPeriodDetailsCollection
            
                    if (sbrPeriodDetailsCollection != null)
                    {
                
                        IEnumerable<SBRElement> sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RPCollection;
                        sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RPCollection = sbrPeriodDetailsCollection.Where(
                            ce => ce.ChildElements != null && 
                            ce.ChildElements.Any(te => te.Name == "Period.Type.Code" && te.Value == "SubstitutedAccountingPeriod" &&
                            sbrRPContexts.Contains(te.Context)));
                        if (sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RPCollection != null && sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RPCollection.Count() > 0)
                        {
                            SBRElement sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RP;
                            sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RP = sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RPCollection.ElementAt(0);
                
                            IDictionary<string, SBRElement> sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RPTupleElementMap;
                            sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RPTupleElementMap = sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                            report.PeriodDetailsCollection_SubstitutedAccountingPeriod_RPExists = true;
                            report.PeriodDetailsCollection_SubstitutedAccountingPeriod_RPCount = sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RPCollection.Count();
                            report.PeriodDetailsCollection_SubstitutedAccountingPeriod_RPOccurrenceIndex = sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RP.OccurrenceIndex + 1;
                            report.PeriodDetailsCollection_SubstitutedAccountingPeriod_RPLastOccurrenceIndex = sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RPCollection.Last().OccurrenceIndex + 1;
                    
                            if (sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Period.Type.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE99 = currentValue.Value;
                            }
                    
                            if (sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Period.Start.Date", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE47 = DateTime.Parse(currentValue.Value);
                            }
                    
                            if (sbrPeriodDetailsCollection_SubstitutedAccountingPeriod_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Period.End.Date", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE48 = DateTime.Parse(currentValue.Value);
                            }
                        }  
                    } 
                    #endregion End of Tuple sbrPeriodDetailsCollection
                    if (validationMode) ValidateRPPeriodDetailsCollection2(errors, sbrPeriodDetailsCollection, sbrRPContexts, report);
            
        
                    IEnumerable<SBRElement> sbrFamilyTrustCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}FamilyTrust", out currentValue))
                        sbrFamilyTrustCollection = currentValue.Occurrences;
                    else
                        sbrFamilyTrustCollection = null;
            
                    #region sbrFamilyTrustCollection
            
                    if (sbrFamilyTrustCollection != null)
                    {
                        report.FamilyTrustCollectionExists = true;
                        report.FamilyTrustCollectionCount = sbrFamilyTrustCollection.Count();
                        SBRElement sbrFamilyTrust;
                        sbrFamilyTrust = sbrFamilyTrustCollection.ElementAt(0);
                
                        IDictionary<string, SBRElement> sbrFamilyTrustTupleElementMap;
                        sbrFamilyTrustTupleElementMap = sbrFamilyTrust.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                        if (sbrFamilyTrustTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Elections.Commencement.Date", out currentValue))
                        {
                            if (!currentValue.IsNil) report.IEE59 = DateTime.Parse(currentValue.Value);
                        }
                
                        if (sbrFamilyTrustTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Elections.FamilyTrustElectionStatus.Year", out currentValue))
                        {
                            if (!currentValue.IsNil) report.IEE58 = int.Parse(currentValue.Value);
                        }
                
                        if (sbrFamilyTrustTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Identifiers.TaxFileNumber.Identifier", out currentValue))
                        {
                            if (!currentValue.IsNil) report.IEE50 = currentValue.Value;
                        }
                
                        if (sbrFamilyTrustTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Identifiers.TaxFileNumberInexistent.Indicator", out currentValue))
                        {
                            if (!currentValue.IsNil) report.IEE51 = ToBoolean(currentValue.Value);
                        }
            
                        IEnumerable<SBRElement> sbrFamilyTrust_OrganisationNameDetailsCollection;
                        if (sbrFamilyTrustTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}OrganisationNameDetails", out currentValue))
                            sbrFamilyTrust_OrganisationNameDetailsCollection = currentValue.Occurrences;
                        else
                            sbrFamilyTrust_OrganisationNameDetailsCollection = null;
                
                        #region sbrFamilyTrust_OrganisationNameDetailsCollection
                
                        if (sbrFamilyTrust_OrganisationNameDetailsCollection != null)
                        {
                            report.FamilyTrust_OrganisationNameDetailsCollectionExists = true;
                            report.FamilyTrust_OrganisationNameDetailsCollectionCount = sbrFamilyTrust_OrganisationNameDetailsCollection.Count();
                    
                            IEnumerable<SBRElement> sbrFamilyTrust_OrganisationNameDetailsCollection_MN_C_RPCollection;
                            sbrFamilyTrust_OrganisationNameDetailsCollection_MN_C_RPCollection = sbrFamilyTrust_OrganisationNameDetailsCollection.Where(
                                ce => ce.ChildElements != null && 
                                ce.ChildElements.Any(te => te.Name == "OrganisationNameDetails.OrganisationalNameType.Code" && te.Value == "MN") &&
                                ce.ChildElements.Any(te => te.Name == "OrganisationNameDetails.Currency.Code" && te.Value == "C" &&
                                sbrRPContexts.Contains(te.Context)));
                            if (sbrFamilyTrust_OrganisationNameDetailsCollection_MN_C_RPCollection != null && sbrFamilyTrust_OrganisationNameDetailsCollection_MN_C_RPCollection.Count() > 0)
                            {
                                SBRElement sbrFamilyTrust_OrganisationNameDetailsCollection_MN_C_RP;
                                sbrFamilyTrust_OrganisationNameDetailsCollection_MN_C_RP = sbrFamilyTrust_OrganisationNameDetailsCollection_MN_C_RPCollection.ElementAt(0);
                    
                                IDictionary<string, SBRElement> sbrFamilyTrust_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap;
                                sbrFamilyTrust_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap = sbrFamilyTrust_OrganisationNameDetailsCollection_MN_C_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                    
                                report.FamilyTrust_OrganisationNameDetailsCollection_MN_C_RPExists = true;
                                report.FamilyTrust_OrganisationNameDetailsCollection_MN_C_RPCount = sbrFamilyTrust_OrganisationNameDetailsCollection_MN_C_RPCollection.Count();
                                report.FamilyTrust_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex = sbrFamilyTrust_OrganisationNameDetailsCollection_MN_C_RP.OccurrenceIndex + 1;
                                report.FamilyTrust_OrganisationNameDetailsCollection_MN_C_RPLastOccurrenceIndex = sbrFamilyTrust_OrganisationNameDetailsCollection_MN_C_RPCollection.Last().OccurrenceIndex + 1;
                        
                                if (sbrFamilyTrust_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}OrganisationNameDetails.OrganisationalNameType.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE119 = currentValue.Value;
                                }
                        
                                if (sbrFamilyTrust_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}OrganisationNameDetails.Currency.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE170 = currentValue.Value;
                                }
                        
                                if (sbrFamilyTrust_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}OrganisationNameDetails.OrganisationalName.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE52 = currentValue.Value;
                                }
                            }  
                        } 
                        #endregion End of Tuple sbrFamilyTrust_OrganisationNameDetailsCollection
                        if (validationMode) ValidateRPFamilyTrust_OrganisationNameDetailsCollection(errors, sbrFamilyTrust_OrganisationNameDetailsCollection, sbrRPContexts, report);
                
            
                        IEnumerable<SBRElement> sbrFamilyTrust_AddressDetailsCollection;
                        if (sbrFamilyTrustTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails", out currentValue))
                            sbrFamilyTrust_AddressDetailsCollection = currentValue.Occurrences;
                        else
                            sbrFamilyTrust_AddressDetailsCollection = null;
                
                        #region sbrFamilyTrust_AddressDetailsCollection
                
                        if (sbrFamilyTrust_AddressDetailsCollection != null)
                        {
                            report.FamilyTrust_AddressDetailsCollectionExists = true;
                            report.FamilyTrust_AddressDetailsCollectionCount = sbrFamilyTrust_AddressDetailsCollection.Count();
                    
                            IEnumerable<SBRElement> sbrFamilyTrust_AddressDetailsCollection_POS_C_RPCollection;
                            sbrFamilyTrust_AddressDetailsCollection_POS_C_RPCollection = sbrFamilyTrust_AddressDetailsCollection.Where(
                                ce => ce.ChildElements != null && 
                                ce.ChildElements.Any(te => te.Name == "AddressDetails.Usage.Code" && te.Value == "POS") &&
                                ce.ChildElements.Any(te => te.Name == "AddressDetails.Currency.Code" && te.Value == "C" &&
                                sbrRPContexts.Contains(te.Context)));
                            if (sbrFamilyTrust_AddressDetailsCollection_POS_C_RPCollection != null && sbrFamilyTrust_AddressDetailsCollection_POS_C_RPCollection.Count() > 0)
                            {
                                SBRElement sbrFamilyTrust_AddressDetailsCollection_POS_C_RP;
                                sbrFamilyTrust_AddressDetailsCollection_POS_C_RP = sbrFamilyTrust_AddressDetailsCollection_POS_C_RPCollection.ElementAt(0);
                    
                                IDictionary<string, SBRElement> sbrFamilyTrust_AddressDetailsCollection_POS_C_RPTupleElementMap;
                                sbrFamilyTrust_AddressDetailsCollection_POS_C_RPTupleElementMap = sbrFamilyTrust_AddressDetailsCollection_POS_C_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                    
                                report.FamilyTrust_AddressDetailsCollection_POS_C_RPExists = true;
                                report.FamilyTrust_AddressDetailsCollection_POS_C_RPCount = sbrFamilyTrust_AddressDetailsCollection_POS_C_RPCollection.Count();
                                report.FamilyTrust_AddressDetailsCollection_POS_C_RPOccurrenceIndex = sbrFamilyTrust_AddressDetailsCollection_POS_C_RP.OccurrenceIndex + 1;
                                report.FamilyTrust_AddressDetailsCollection_POS_C_RPLastOccurrenceIndex = sbrFamilyTrust_AddressDetailsCollection_POS_C_RPCollection.Last().OccurrenceIndex + 1;
                        
                                if (sbrFamilyTrust_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.OverseasAddress.Indicator", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE133 = ToBoolean(currentValue.Value);
                                }
                        
                                if (sbrFamilyTrust_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Usage.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE107 = currentValue.Value;
                                }
                        
                                if (sbrFamilyTrust_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Currency.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE153 = currentValue.Value;
                                }
                        
                                if (sbrFamilyTrust_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line1.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE53 = currentValue.Value;
                                }
                        
                                if (sbrFamilyTrust_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line2.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE134 = currentValue.Value;
                                }
                        
                                if (sbrFamilyTrust_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line3.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE135 = currentValue.Value;
                                }
                        
                                if (sbrFamilyTrust_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line4.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE136 = currentValue.Value;
                                }
                        
                                if (sbrFamilyTrust_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.LocalityName.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE54 = currentValue.Value;
                                }
                        
                                if (sbrFamilyTrust_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Postcode.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE56 = currentValue.Value;
                                }
                        
                                if (sbrFamilyTrust_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.StateOrTerritory.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE55 = currentValue.Value;
                                }
                        
                                if (sbrFamilyTrust_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.CountryName.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE57 = currentValue.Value;
                                }
                        
                                if (sbrFamilyTrust_AddressDetailsCollection_POS_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Country.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE155 = currentValue.Value;
                                }
                            }  
                        } 
                        #endregion End of Tuple sbrFamilyTrust_AddressDetailsCollection
                        if (validationMode) ValidateRPFamilyTrust_AddressDetailsCollection(errors, sbrFamilyTrust_AddressDetailsCollection, sbrRPContexts, report);
                
            
                        IEnumerable<SBRElement> sbrFamilyTrust_PeriodDetailsCollection;
                        if (sbrFamilyTrustTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PeriodDetails", out currentValue))
                            sbrFamilyTrust_PeriodDetailsCollection = currentValue.Occurrences;
                        else
                            sbrFamilyTrust_PeriodDetailsCollection = null;
                
                        #region sbrFamilyTrust_PeriodDetailsCollection
                
                        if (sbrFamilyTrust_PeriodDetailsCollection != null)
                        {
                            report.FamilyTrust_PeriodDetailsCollectionExists = true;
                            report.FamilyTrust_PeriodDetailsCollectionCount = sbrFamilyTrust_PeriodDetailsCollection.Count();
                    
                            IEnumerable<SBRElement> sbrFamilyTrust_PeriodDetailsCollection_SubstitutedAccountingPeriod_RPCollection;
                            sbrFamilyTrust_PeriodDetailsCollection_SubstitutedAccountingPeriod_RPCollection = sbrFamilyTrust_PeriodDetailsCollection.Where(
                                ce => ce.ChildElements != null && 
                                ce.ChildElements.Any(te => te.Name == "Period.Type.Code" && te.Value == "SubstitutedAccountingPeriod" &&
                                sbrRPContexts.Contains(te.Context)));
                            if (sbrFamilyTrust_PeriodDetailsCollection_SubstitutedAccountingPeriod_RPCollection != null && sbrFamilyTrust_PeriodDetailsCollection_SubstitutedAccountingPeriod_RPCollection.Count() > 0)
                            {
                                SBRElement sbrFamilyTrust_PeriodDetailsCollection_SubstitutedAccountingPeriod_RP;
                                sbrFamilyTrust_PeriodDetailsCollection_SubstitutedAccountingPeriod_RP = sbrFamilyTrust_PeriodDetailsCollection_SubstitutedAccountingPeriod_RPCollection.ElementAt(0);
                    
                                IDictionary<string, SBRElement> sbrFamilyTrust_PeriodDetailsCollection_SubstitutedAccountingPeriod_RPTupleElementMap;
                                sbrFamilyTrust_PeriodDetailsCollection_SubstitutedAccountingPeriod_RPTupleElementMap = sbrFamilyTrust_PeriodDetailsCollection_SubstitutedAccountingPeriod_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                    
                                report.FamilyTrust_PeriodDetailsCollection_SubstitutedAccountingPeriod_RPExists = true;
                                report.FamilyTrust_PeriodDetailsCollection_SubstitutedAccountingPeriod_RPCount = sbrFamilyTrust_PeriodDetailsCollection_SubstitutedAccountingPeriod_RPCollection.Count();
                                report.FamilyTrust_PeriodDetailsCollection_SubstitutedAccountingPeriod_RPOccurrenceIndex = sbrFamilyTrust_PeriodDetailsCollection_SubstitutedAccountingPeriod_RP.OccurrenceIndex + 1;
                                report.FamilyTrust_PeriodDetailsCollection_SubstitutedAccountingPeriod_RPLastOccurrenceIndex = sbrFamilyTrust_PeriodDetailsCollection_SubstitutedAccountingPeriod_RPCollection.Last().OccurrenceIndex + 1;
                        
                                if (sbrFamilyTrust_PeriodDetailsCollection_SubstitutedAccountingPeriod_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Period.Type.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE103 = currentValue.Value;
                                }
                        
                                if (sbrFamilyTrust_PeriodDetailsCollection_SubstitutedAccountingPeriod_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Period.Start.Date", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE60 = DateTime.Parse(currentValue.Value);
                                }
                        
                                if (sbrFamilyTrust_PeriodDetailsCollection_SubstitutedAccountingPeriod_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Period.End.Date", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE61 = DateTime.Parse(currentValue.Value);
                                }
                            }  
                        } 
                        #endregion End of Tuple sbrFamilyTrust_PeriodDetailsCollection
                        if (validationMode) ValidateRPFamilyTrust_PeriodDetailsCollection(errors, sbrFamilyTrust_PeriodDetailsCollection, sbrRPContexts, report);
                
                    } 
                    #endregion End of Tuple sbrFamilyTrustCollection
        
                    IEnumerable<SBRElement> sbrSpecifiedIndividualCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}SpecifiedIndividual", out currentValue))
                        sbrSpecifiedIndividualCollection = currentValue.Occurrences;
                    else
                        sbrSpecifiedIndividualCollection = null;
            
                    #region sbrSpecifiedIndividualCollection
            
                    if (sbrSpecifiedIndividualCollection != null)
                    {
                        report.SpecifiedIndividualCollectionExists = true;
                        report.SpecifiedIndividualCollectionCount = sbrSpecifiedIndividualCollection.Count();
                        SBRElement sbrSpecifiedIndividual;
                        sbrSpecifiedIndividual = sbrSpecifiedIndividualCollection.ElementAt(0);
                
                        IDictionary<string, SBRElement> sbrSpecifiedIndividualTupleElementMap;
                        sbrSpecifiedIndividualTupleElementMap = sbrSpecifiedIndividual.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                        if (sbrSpecifiedIndividualTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Identifiers.TaxFileNumber.Identifier", out currentValue))
                        {
                            if (!currentValue.IsNil) report.IEE62 = currentValue.Value;
                        }
                
                        if (sbrSpecifiedIndividualTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Identifiers.TaxFileNumberInexistent.Indicator", out currentValue))
                        {
                            if (!currentValue.IsNil) report.IEE63 = ToBoolean(currentValue.Value);
                        }
                
                        if (sbrSpecifiedIndividualTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonDemographicDetails.Birth.Date", out currentValue))
                        {
                            if (!currentValue.IsNil) report.IEE64 = DateTime.Parse(currentValue.Value);
                        }
            
                        IEnumerable<SBRElement> sbrSpecifiedIndividual_PersonNameDetailsCollection;
                        if (sbrSpecifiedIndividualTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails", out currentValue))
                            sbrSpecifiedIndividual_PersonNameDetailsCollection = currentValue.Occurrences;
                        else
                            sbrSpecifiedIndividual_PersonNameDetailsCollection = null;
                
                        #region sbrSpecifiedIndividual_PersonNameDetailsCollection
                
                        if (sbrSpecifiedIndividual_PersonNameDetailsCollection != null)
                        {
                            report.SpecifiedIndividual_PersonNameDetailsCollectionExists = true;
                            report.SpecifiedIndividual_PersonNameDetailsCollectionCount = sbrSpecifiedIndividual_PersonNameDetailsCollection.Count();
                    
                            IEnumerable<SBRElement> sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPCollection;
                            sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPCollection = sbrSpecifiedIndividual_PersonNameDetailsCollection.Where(
                                ce => ce.ChildElements != null && 
                                ce.ChildElements.Any(te => te.Name == "PersonNameDetails.PersonNameType.Code" && te.Value == "LGL") &&
                                ce.ChildElements.Any(te => te.Name == "PersonNameDetails.Usage.Code" && te.Value == "Identifier") &&
                                ce.ChildElements.Any(te => te.Name == "PersonNameDetails.Currency.Code" && te.Value == "C" &&
                                sbrRPContexts.Contains(te.Context)));
                            if (sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPCollection != null && sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPCollection.Count() > 0)
                            {
                                SBRElement sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RP;
                                sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RP = sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPCollection.ElementAt(0);
                    
                                IDictionary<string, SBRElement> sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPTupleElementMap;
                                sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPTupleElementMap = sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                    
                                report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPExists = true;
                                report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPCount = sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPCollection.Count();
                                report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPOccurrenceIndex = sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RP.OccurrenceIndex + 1;
                                report.SpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPLastOccurrenceIndex = sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPCollection.Last().OccurrenceIndex + 1;
                        
                                if (sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.PersonNameType.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE171 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.Usage.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE172 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.Currency.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE177 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.Title.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE65 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.NameSuffix.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE67 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.FamilyName.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE66 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.GivenName.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE68 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_PersonNameDetailsCollection_LGL_Identifier_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.OtherGivenName.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE69 = currentValue.Value;
                                }
                            }  
                        } 
                        #endregion End of Tuple sbrSpecifiedIndividual_PersonNameDetailsCollection
                        if (validationMode) ValidateRPSpecifiedIndividual_PersonNameDetailsCollection(errors, sbrSpecifiedIndividual_PersonNameDetailsCollection, sbrRPContexts, report);
                
            
                        IEnumerable<SBRElement> sbrSpecifiedIndividual_AddressDetailsCollection;
                        if (sbrSpecifiedIndividualTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails", out currentValue))
                            sbrSpecifiedIndividual_AddressDetailsCollection = currentValue.Occurrences;
                        else
                            sbrSpecifiedIndividual_AddressDetailsCollection = null;
                
                        #region sbrSpecifiedIndividual_AddressDetailsCollection
                
                        if (sbrSpecifiedIndividual_AddressDetailsCollection != null)
                        {
                            report.SpecifiedIndividual_AddressDetailsCollectionExists = true;
                            report.SpecifiedIndividual_AddressDetailsCollectionCount = sbrSpecifiedIndividual_AddressDetailsCollection.Count();
                    
                            IEnumerable<SBRElement> sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPCollection;
                            sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPCollection = sbrSpecifiedIndividual_AddressDetailsCollection.Where(
                                ce => ce.ChildElements != null && 
                                ce.ChildElements.Any(te => te.Name == "AddressDetails.Usage.Code" && te.Value == "RES") &&
                                ce.ChildElements.Any(te => te.Name == "AddressDetails.Currency.Code" && te.Value == "C" &&
                                sbrRPContexts.Contains(te.Context)));
                            if (sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPCollection != null && sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPCollection.Count() > 0)
                            {
                                SBRElement sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RP;
                                sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RP = sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPCollection.ElementAt(0);
                    
                                IDictionary<string, SBRElement> sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPTupleElementMap;
                                sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPTupleElementMap = sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                    
                                report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPExists = true;
                                report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPCount = sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPCollection.Count();
                                report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPOccurrenceIndex = sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RP.OccurrenceIndex + 1;
                                report.SpecifiedIndividual_AddressDetailsCollection_RES_C_RPLastOccurrenceIndex = sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPCollection.Last().OccurrenceIndex + 1;
                        
                                if (sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.OverseasAddress.Indicator", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE137 = ToBoolean(currentValue.Value);
                                }
                        
                                if (sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Usage.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE108 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Currency.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE152 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line1.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE70 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line2.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE138 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line3.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE139 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line4.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE140 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.LocalityName.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE71 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Postcode.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE73 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.StateOrTerritory.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE72 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.CountryName.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE74 = currentValue.Value;
                                }
                        
                                if (sbrSpecifiedIndividual_AddressDetailsCollection_RES_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Country.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) report.IEE114 = currentValue.Value;
                                }
                            }  
                        } 
                        #endregion End of Tuple sbrSpecifiedIndividual_AddressDetailsCollection
                        if (validationMode) ValidateRPSpecifiedIndividual_AddressDetailsCollection(errors, sbrSpecifiedIndividual_AddressDetailsCollection, sbrRPContexts, report);
                
                    } 
                    #endregion End of Tuple sbrSpecifiedIndividualCollection
        
                    IEnumerable<SBRElement> sbrDeclarationOfTrusteeOrPartnerCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}DeclarationOfTrusteeOrPartner", out currentValue))
                        sbrDeclarationOfTrusteeOrPartnerCollection = currentValue.Occurrences;
                    else
                        sbrDeclarationOfTrusteeOrPartnerCollection = null;
            
                    #region sbrDeclarationOfTrusteeOrPartnerCollection
            
                    if (sbrDeclarationOfTrusteeOrPartnerCollection != null)
                    {
                        report.DeclarationOfTrusteeOrPartnerCollectionExists = true;
                        report.DeclarationOfTrusteeOrPartnerCollectionCount = sbrDeclarationOfTrusteeOrPartnerCollection.Count();
                        if (sbrDeclarationOfTrusteeOrPartnerCollection != null && sbrDeclarationOfTrusteeOrPartnerCollection.Count() > 0)
                        {
                            report.DeclarationOfTrusteeOrPartnerCollection = new List<IEE2018.DeclarationOfTrusteeOrPartner>();
                            for (int tupleIndex = 0; tupleIndex < sbrDeclarationOfTrusteeOrPartnerCollection.Count(); tupleIndex++)
                            {
                                SBRElement sbrDeclarationOfTrusteeOrPartner;
                                sbrDeclarationOfTrusteeOrPartner = sbrDeclarationOfTrusteeOrPartnerCollection.ElementAt(tupleIndex);
                
                                IDictionary<string, SBRElement> sbrDeclarationOfTrusteeOrPartnerTupleElementMap;
                                sbrDeclarationOfTrusteeOrPartnerTupleElementMap = sbrDeclarationOfTrusteeOrPartner.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                                IEE2018.DeclarationOfTrusteeOrPartner declarationOfTrusteeOrPartner = new IEE2018.DeclarationOfTrusteeOrPartner();
                                report.DeclarationOfTrusteeOrPartnerCollection.Add(declarationOfTrusteeOrPartner);
                                declarationOfTrusteeOrPartner.OccurrenceIndex = tupleIndex + 1;
                    
                                IEnumerable<SBRElement> sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection;
                                if (sbrDeclarationOfTrusteeOrPartnerTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails", out currentValue))
                                    sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection = currentValue.Occurrences;
                                else
                                    sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection = null;
                        
                                #region sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection
                        
                                if (sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection != null)
                                {
                                    declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollectionExists = true;
                                    declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollectionCount = sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection.Count();
                            
                                    IEnumerable<SBRElement> sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPCollection;
                                    sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPCollection = sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection.Where(
                                        ce => ce.ChildElements != null && 
                                        ce.ChildElements.Any(te => te.Name == "PersonNameDetails.PersonNameType.Code" && te.Value == "LGL") &&
                                        ce.ChildElements.Any(te => te.Name == "PersonNameDetails.Usage.Code" && te.Value == "DeclarationSignatory") &&
                                        ce.ChildElements.Any(te => te.Name == "PersonNameDetails.Currency.Code" && te.Value == "C" &&
                                        sbrRPContexts.Contains(te.Context)));
                                    if (sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPCollection != null && sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPCollection.Count() > 0)
                                    {
                                        SBRElement sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RP;
                                        sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RP = sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPCollection.ElementAt(0);
                            
                                        IDictionary<string, SBRElement> sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap;
                                        sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap = sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                            
                                        declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPExists = true;
                                        declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPCount = sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPCollection.Count();
                                        declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex = sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RP.OccurrenceIndex + 1;
                                        declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPLastOccurrenceIndex = sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPCollection.Last().OccurrenceIndex + 1;
                                
                                        if (sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.PersonNameType.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) declarationOfTrusteeOrPartner.IEE198 = currentValue.Value;
                                        }
                                
                                        if (sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.Usage.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) declarationOfTrusteeOrPartner.IEE199 = currentValue.Value;
                                        }
                                
                                        if (sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.Currency.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) declarationOfTrusteeOrPartner.IEE200 = currentValue.Value;
                                        }
                                
                                        if (sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.Title.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) declarationOfTrusteeOrPartner.IEE201 = currentValue.Value;
                                        }
                                
                                        if (sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.NameSuffix.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) declarationOfTrusteeOrPartner.IEE202 = currentValue.Value;
                                        }
                                
                                        if (sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.FamilyName.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) declarationOfTrusteeOrPartner.IEE203 = currentValue.Value;
                                        }
                                
                                        if (sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.GivenName.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) declarationOfTrusteeOrPartner.IEE204 = currentValue.Value;
                                        }
                                
                                        if (sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.OtherGivenName.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) declarationOfTrusteeOrPartner.IEE205 = currentValue.Value;
                                        }
                                    }  
                                } 
                                #endregion End of Tuple sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection
                                if (validationMode) ValidateRPDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection(errors, sbrDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection, sbrRPContexts, report);
                        
                    
                                IEnumerable<SBRElement> sbrDeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection;
                                if (sbrDeclarationOfTrusteeOrPartnerTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}OrganisationNameDetails", out currentValue))
                                    sbrDeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection = currentValue.Occurrences;
                                else
                                    sbrDeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection = null;
                        
                                #region sbrDeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection
                        
                                if (sbrDeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection != null)
                                {
                                    declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollectionExists = true;
                                    declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollectionCount = sbrDeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection.Count();
                            
                                    IEnumerable<SBRElement> sbrDeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPCollection;
                                    sbrDeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPCollection = sbrDeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection.Where(
                                        ce => ce.ChildElements != null && 
                                        ce.ChildElements.Any(te => te.Name == "OrganisationNameDetails.OrganisationalNameType.Code" && te.Value == "MN") &&
                                        ce.ChildElements.Any(te => te.Name == "OrganisationNameDetails.Currency.Code" && te.Value == "C" &&
                                        sbrRPContexts.Contains(te.Context)));
                                    if (sbrDeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPCollection != null && sbrDeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPCollection.Count() > 0)
                                    {
                                        SBRElement sbrDeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RP;
                                        sbrDeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RP = sbrDeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPCollection.ElementAt(0);
                            
                                        IDictionary<string, SBRElement> sbrDeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap;
                                        sbrDeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap = sbrDeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                            
                                        declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPExists = true;
                                        declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPCount = sbrDeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPCollection.Count();
                                        declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex = sbrDeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RP.OccurrenceIndex + 1;
                                        declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPLastOccurrenceIndex = sbrDeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPCollection.Last().OccurrenceIndex + 1;
                                
                                        if (sbrDeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}OrganisationNameDetails.OrganisationalNameType.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) declarationOfTrusteeOrPartner.IEE173 = currentValue.Value;
                                        }
                                
                                        if (sbrDeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}OrganisationNameDetails.Currency.Code", out currentValue))
                                        {
                                            if (!currentValue.IsNil) declarationOfTrusteeOrPartner.IEE178 = currentValue.Value;
                                        }
                                
                                        if (sbrDeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection_MN_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}OrganisationNameDetails.OrganisationalName.Text", out currentValue))
                                        {
                                            if (!currentValue.IsNil) declarationOfTrusteeOrPartner.IEE88 = currentValue.Value;
                                        }
                                    }  
                                } 
                                #endregion End of Tuple sbrDeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection
                                if (validationMode) ValidateRPDeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection(errors, sbrDeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection, sbrRPContexts, report);
                        
                    
                                IEnumerable<SBRElement> sbrDeclarationOfTrusteeOrPartner_DeclarationCollection;
                                if (sbrDeclarationOfTrusteeOrPartnerTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Declaration", out currentValue))
                                    sbrDeclarationOfTrusteeOrPartner_DeclarationCollection = currentValue.Occurrences;
                                else
                                    sbrDeclarationOfTrusteeOrPartner_DeclarationCollection = null;
                        
                                #region sbrDeclarationOfTrusteeOrPartner_DeclarationCollection
                        
                                if (sbrDeclarationOfTrusteeOrPartner_DeclarationCollection != null)
                                {
                                    sbrDeclarationOfTrusteeOrPartner_DeclarationCollection =
                                        sbrDeclarationOfTrusteeOrPartner_DeclarationCollection.Where(
                                            t =>
                                                t.ChildElements != null && t.ChildElements.Count > 0 &&
                                                sbrRPContexts.Contains(t.ChildElements[0].Context));
                                }
                        
                        
                                if (sbrDeclarationOfTrusteeOrPartner_DeclarationCollection != null && sbrDeclarationOfTrusteeOrPartner_DeclarationCollection.Count() > 0)
                                {
                                    declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_DeclarationCollectionExists = true;
                                    declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_DeclarationCollectionCount = sbrDeclarationOfTrusteeOrPartner_DeclarationCollection.Count();
                                    declarationOfTrusteeOrPartner.RpDeclarationOfTrusteeOrPartner_DeclarationCollectionExists = true;
                                    declarationOfTrusteeOrPartner.RpDeclarationOfTrusteeOrPartner_DeclarationCollectionCount = sbrDeclarationOfTrusteeOrPartner_DeclarationCollection.Count();
                                    SBRElement sbrDeclarationOfTrusteeOrPartner_Declaration;
                                    sbrDeclarationOfTrusteeOrPartner_Declaration = sbrDeclarationOfTrusteeOrPartner_DeclarationCollection.ElementAt(0);
                            
                                    IDictionary<string, SBRElement> sbrDeclarationOfTrusteeOrPartner_DeclarationTupleElementMap;
                                    sbrDeclarationOfTrusteeOrPartner_DeclarationTupleElementMap = sbrDeclarationOfTrusteeOrPartner_Declaration.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                            
                                    if (sbrDeclarationOfTrusteeOrPartner_DeclarationTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Declaration.StatementType.Code", out currentValue))
                                    {
                                        if (!currentValue.IsNil) declarationOfTrusteeOrPartner.IEE193 = currentValue.Value;
                                    }
                            
                                    if (sbrDeclarationOfTrusteeOrPartner_DeclarationTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Declaration.StatementAccepted.Indicator", out currentValue))
                                    {
                                        if (!currentValue.IsNil) declarationOfTrusteeOrPartner.IEE146 = ToBoolean(currentValue.Value);
                                    }
                            
                                    if (sbrDeclarationOfTrusteeOrPartner_DeclarationTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Declaration.Statement.Text", out currentValue))
                                    {
                                        if (!currentValue.IsNil) declarationOfTrusteeOrPartner.IEE194 = currentValue.Value;
                                    }
                            
                                    if (sbrDeclarationOfTrusteeOrPartner_DeclarationTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Declaration.Signature.Date", out currentValue))
                                    {
                                        if (!currentValue.IsNil) declarationOfTrusteeOrPartner.IEE90 = DateTime.Parse(currentValue.Value);
                                    }
                            
                                    if (sbrDeclarationOfTrusteeOrPartner_DeclarationTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Declaration.SignatoryIdentifier.Text", out currentValue))
                                    {
                                        if (!currentValue.IsNil) declarationOfTrusteeOrPartner.IEE195 = currentValue.Value;
                                    }
                        
                                    IEnumerable<SBRElement> sbrDeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection;
                                    if (sbrDeclarationOfTrusteeOrPartner_DeclarationTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonUnstructuredName", out currentValue))
                                        sbrDeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection = currentValue.Occurrences;
                                    else
                                        sbrDeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection = null;
                            
                                    #region sbrDeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection
                            
                                    if (sbrDeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection != null)
                                    {
                                        declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollectionExists = true;
                                        declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollectionCount = sbrDeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection.Count();
                                
                                        IEnumerable<SBRElement> sbrDeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCollection;
                                        sbrDeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCollection = sbrDeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection.Where(
                                            ce => ce.ChildElements != null && 
                                            ce.ChildElements.Any(te => te.Name == "PersonUnstructuredName.Usage.Code" && te.Value == "DeclarationSignatory" &&
                                            sbrRPContexts.Contains(te.Context)));
                                        if (sbrDeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCollection != null && sbrDeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCollection.Count() > 0)
                                        {
                                            SBRElement sbrDeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RP;
                                            sbrDeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RP = sbrDeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCollection.ElementAt(0);
                                
                                            IDictionary<string, SBRElement> sbrDeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPTupleElementMap;
                                            sbrDeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPTupleElementMap = sbrDeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                                
                                            declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPExists = true;
                                            declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCount = sbrDeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCollection.Count();
                                            declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPOccurrenceIndex = sbrDeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RP.OccurrenceIndex + 1;
                                            declarationOfTrusteeOrPartner.DeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPLastOccurrenceIndex = sbrDeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCollection.Last().OccurrenceIndex + 1;
                                    
                                            if (sbrDeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonUnstructuredName.Usage.Code", out currentValue))
                                            {
                                                if (!currentValue.IsNil) declarationOfTrusteeOrPartner.IEE196 = currentValue.Value;
                                            }
                                    
                                            if (sbrDeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonUnstructuredName.FullName.Text", out currentValue))
                                            {
                                                if (!currentValue.IsNil) declarationOfTrusteeOrPartner.IEE197 = currentValue.Value;
                                            }
                                        }  
                                    } 
                                    #endregion End of Tuple sbrDeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection
                                    if (validationMode) ValidateRPDeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection(errors, sbrDeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection, sbrRPContexts, report);
                            
                                } 
                                #endregion End of Tuple sbrDeclarationOfTrusteeOrPartner_DeclarationCollection
                    } 
                    #endregion End of Tuple sbrDeclarationOfTrusteeOrPartnerCollection
                        } 
                    } 
        
                    IEnumerable<SBRElement> sbrPersonNameDetailsCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails", out currentValue))
                        sbrPersonNameDetailsCollection = currentValue.Occurrences;
                    else
                        sbrPersonNameDetailsCollection = null;
            
                    #region sbrPersonNameDetailsCollection
            
                    if (sbrPersonNameDetailsCollection != null)
                    {
                        report.PersonNameDetailsCollectionExists = true;
                        report.PersonNameDetailsCollectionCount = sbrPersonNameDetailsCollection.Count();
                
                        IEnumerable<SBRElement> sbrPersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPCollection;
                        sbrPersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPCollection = sbrPersonNameDetailsCollection.Where(
                            ce => ce.ChildElements != null && 
                            ce.ChildElements.Any(te => te.Name == "PersonNameDetails.PersonNameType.Code" && te.Value == "LGL") &&
                            ce.ChildElements.Any(te => te.Name == "PersonNameDetails.Usage.Code" && te.Value == "DeclarationSignatory") &&
                            ce.ChildElements.Any(te => te.Name == "PersonNameDetails.Currency.Code" && te.Value == "C" &&
                            sbrRPContexts.Contains(te.Context)));
                        if (sbrPersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPCollection != null && sbrPersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPCollection.Count() > 0)
                        {
                            SBRElement sbrPersonNameDetailsCollection_LGL_DeclarationSignatory_C_RP;
                            sbrPersonNameDetailsCollection_LGL_DeclarationSignatory_C_RP = sbrPersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPCollection.ElementAt(0);
                
                            IDictionary<string, SBRElement> sbrPersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap;
                            sbrPersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap = sbrPersonNameDetailsCollection_LGL_DeclarationSignatory_C_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                            report.PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPExists = true;
                            report.PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPCount = sbrPersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPCollection.Count();
                            report.PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPOccurrenceIndex = sbrPersonNameDetailsCollection_LGL_DeclarationSignatory_C_RP.OccurrenceIndex + 1;
                            report.PersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPLastOccurrenceIndex = sbrPersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPCollection.Last().OccurrenceIndex + 1;
                    
                            if (sbrPersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.PersonNameType.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE174 = currentValue.Value;
                            }
                    
                            if (sbrPersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.Usage.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE157 = currentValue.Value;
                            }
                    
                            if (sbrPersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.Currency.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE175 = currentValue.Value;
                            }
                    
                            if (sbrPersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.Title.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE83 = currentValue.Value;
                            }
                    
                            if (sbrPersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.NameSuffix.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE85 = currentValue.Value;
                            }
                    
                            if (sbrPersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.FamilyName.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE84 = currentValue.Value;
                            }
                    
                            if (sbrPersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.GivenName.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE86 = currentValue.Value;
                            }
                    
                            if (sbrPersonNameDetailsCollection_LGL_DeclarationSignatory_C_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.OtherGivenName.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE87 = currentValue.Value;
                            }
                        }  
                    } 
                    #endregion End of Tuple sbrPersonNameDetailsCollection
                    if (validationMode) ValidateRPPersonNameDetailsCollection(errors, sbrPersonNameDetailsCollection, sbrRPContexts, report);
            
        
                    IEnumerable<SBRElement> sbrDeclarationCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Declaration", out currentValue))
                        sbrDeclarationCollection = currentValue.Occurrences;
                    else
                        sbrDeclarationCollection = null;
            
                    #region sbrDeclarationCollection
            
                    if (sbrDeclarationCollection != null)
                    {
                
                        IEnumerable<SBRElement> sbrDeclarationCollection_TrueAndCorrect_RPCollection;
                        sbrDeclarationCollection_TrueAndCorrect_RPCollection = sbrDeclarationCollection.Where(
                            ce => ce.ChildElements != null && 
                            ce.ChildElements.Any(te => te.Name == "Declaration.StatementType.Code" && te.Value == "TrueAndCorrect" &&
                            sbrRPContexts.Contains(te.Context)));
                        if (sbrDeclarationCollection_TrueAndCorrect_RPCollection != null && sbrDeclarationCollection_TrueAndCorrect_RPCollection.Count() > 0)
                        {
                            SBRElement sbrDeclarationCollection_TrueAndCorrect_RP;
                            sbrDeclarationCollection_TrueAndCorrect_RP = sbrDeclarationCollection_TrueAndCorrect_RPCollection.ElementAt(0);
                
                            IDictionary<string, SBRElement> sbrDeclarationCollection_TrueAndCorrect_RPTupleElementMap;
                            sbrDeclarationCollection_TrueAndCorrect_RPTupleElementMap = sbrDeclarationCollection_TrueAndCorrect_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                            report.DeclarationCollection_TrueAndCorrect_RPExists = true;
                            report.DeclarationCollection_TrueAndCorrect_RPCount = sbrDeclarationCollection_TrueAndCorrect_RPCollection.Count();
                            report.DeclarationCollection_TrueAndCorrect_RPOccurrenceIndex = sbrDeclarationCollection_TrueAndCorrect_RP.OccurrenceIndex + 1;
                            report.DeclarationCollection_TrueAndCorrect_RPLastOccurrenceIndex = sbrDeclarationCollection_TrueAndCorrect_RPCollection.Last().OccurrenceIndex + 1;
                    
                            if (sbrDeclarationCollection_TrueAndCorrect_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Declaration.StatementType.Code", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE147 = currentValue.Value;
                            }
                    
                            if (sbrDeclarationCollection_TrueAndCorrect_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Declaration.StatementAccepted.Indicator", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE192 = ToBoolean(currentValue.Value);
                            }
                    
                            if (sbrDeclarationCollection_TrueAndCorrect_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Declaration.Statement.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE149 = currentValue.Value;
                            }
                    
                            if (sbrDeclarationCollection_TrueAndCorrect_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Declaration.Signature.Date", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE191 = DateTime.Parse(currentValue.Value);
                            }
                    
                            if (sbrDeclarationCollection_TrueAndCorrect_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}Declaration.SignatoryIdentifier.Text", out currentValue))
                            {
                                if (!currentValue.IsNil) report.IEE148 = currentValue.Value;
                            }
                
                            IEnumerable<SBRElement> sbrDeclaration_PersonUnstructuredNameCollection;
                            if (sbrDeclarationCollection_TrueAndCorrect_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonUnstructuredName", out currentValue))
                                sbrDeclaration_PersonUnstructuredNameCollection = currentValue.Occurrences;
                            else
                                sbrDeclaration_PersonUnstructuredNameCollection = null;
                    
                            #region sbrDeclaration_PersonUnstructuredNameCollection
                    
                            if (sbrDeclaration_PersonUnstructuredNameCollection != null)
                            {
                        
                                IEnumerable<SBRElement> sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCollection;
                                sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCollection = sbrDeclaration_PersonUnstructuredNameCollection.Where(
                                    ce => ce.ChildElements != null && 
                                    ce.ChildElements.Any(te => te.Name == "PersonUnstructuredName.Usage.Code" && te.Value == "DeclarationSignatory" &&
                                    sbrRPContexts.Contains(te.Context)));
                                if (sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCollection != null && sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCollection.Count() > 0)
                                {
                                    SBRElement sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RP;
                                    sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RP = sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCollection.ElementAt(0);
                        
                                    IDictionary<string, SBRElement> sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPTupleElementMap;
                                    sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPTupleElementMap = sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RP.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                        
                                    report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPExists = true;
                                    report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCount = sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCollection.Count();
                                    report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPOccurrenceIndex = sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RP.OccurrenceIndex + 1;
                                    report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPLastOccurrenceIndex = sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPCollection.Last().OccurrenceIndex + 1;
                            
                                    if (sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonUnstructuredName.Usage.Code", out currentValue))
                                    {
                                        if (!currentValue.IsNil) report.IEE156 = currentValue.Value;
                                    }
                            
                                    if (sbrDeclaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonUnstructuredName.FullName.Text", out currentValue))
                                    {
                                        if (!currentValue.IsNil) report.IEE91 = currentValue.Value;
                                    }
                                }  
                            } 
                            #endregion End of Tuple sbrDeclaration_PersonUnstructuredNameCollection
                            if (validationMode) ValidateRPDeclaration_PersonUnstructuredNameCollection(errors, sbrDeclaration_PersonUnstructuredNameCollection, sbrRPContexts, report);
                    
                        }  
                    } 
                    #endregion End of Tuple sbrDeclarationCollection
                    if (validationMode) ValidateRPDeclarationCollection(errors, sbrDeclarationCollection, sbrRPContexts, report);
            
                } 
            } 
            #endregion End of Context sbrRPContext
            return report;

        } // Of Consume Method
        #region ValidateINTDeclaration_PersonUnstructuredNameCollection
        protected virtual void ValidateINTDeclaration_PersonUnstructuredNameCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, IEE2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the INT Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                }
            }
        }
        #endregion

        #region ValidateINTDeclarationCollection
        protected virtual void ValidateINTDeclarationCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, IEE2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the INT Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                }
            }
        }
        #endregion

        #region ValidateINTPersonUnstructuredNameCollection
        protected virtual void ValidateINTPersonUnstructuredNameCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, IEE2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the INT Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;

                    SBRElement IEE185Value = null;
                    string IEE185 = null;
                    SBRElement IEE186Value = null;
                    string IEE186 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonUnstructuredName.Usage.Code", out IEE185Value))
                    {
                        if (!IEE185Value.IsNil) IEE185 = IEE185Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonUnstructuredName.FullName.Text", out IEE186Value))
                    {
                        if (!IEE186Value.IsNil) IEE186 = IEE186Value.Value;
                    }
        
                    #region VR.ATO.IEE.410263

        /*  VR.ATO.IEE.410263
            Intermediary contact name must have a Usage code of 'Contact"

        Legacy Rule Format:
            WHERE IN TUPLE (xbrli\prsnstrcnm1.xx.xx:PersonUnstructuredName)
            IF INT:pyde.xx.xx:PersonUnstructuredName.Usage.Code <> NULLORBLANK AND
             INT:pyde.xx.xx:PersonUnstructuredName.Usage.Code <> "Contact"
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            ^IEE185 <> BLANK AND ^IEE185 <> 'Contact'

        Data Elements:
    
            INT:^IEE185 = pyde.02.37:PersonUnstructuredName.Usage.Code WHERE [PersonUnstructuredName.Usage.Code='Contact']
            */
                    assertion = (string.IsNullOrWhiteSpace(IEE185) != true && IEE185 != @"Contact");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410263", Severity = ProcessMessageSeverity.Error,
                            Description = @"Contact name must have a Usage code of 'Contact'",
                            LongDescription = @"The unstructured person name for the contact must include a usage Code of 'Contact'. Please contact your software vendor in order to rectify this error",
                            Location = IEE185Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:PersonUnstructuredName.Usage.Code" : DeriveSbrElementLocation(IEE185Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IEE.410263"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE185", Value = IEE185 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateINTElectronicContactTelephoneCollection
        protected virtual void ValidateINTElectronicContactTelephoneCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, IEE2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the INT Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                }
            }
        }
        #endregion

        #region ValidateRPOrganisationNameDetailsCollection
        protected virtual void ValidateRPOrganisationNameDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, IEE2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;

                    SBRElement IEE117Value = null;
                    string IEE117 = null;
                    SBRElement IEE164Value = null;
                    string IEE164 = null;
                    SBRElement IEE4Value = null;
                    string IEE4 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}OrganisationNameDetails.OrganisationalNameType.Code", out IEE117Value))
                    {
                        if (!IEE117Value.IsNil) IEE117 = IEE117Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}OrganisationNameDetails.Currency.Code", out IEE164Value))
                    {
                        if (!IEE164Value.IsNil) IEE164 = IEE164Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}OrganisationNameDetails.OrganisationalName.Text", out IEE4Value))
                    {
                        if (!IEE4Value.IsNil) IEE4 = IEE4Value.Value;
                    }
        
                    #region VR.ATO.GEN.410049

        /*  VR.ATO.GEN.410049

        Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalNameType.Code <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalNameType.Code <> "MN")
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^IEE117 <> BLANK) AND (^IEE117 <> 'MN')

        Data Elements:
    
            RP:^IEE117 = pyde.02.18:OrganisationNameDetails.OrganisationalNameType.Code WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
                    assertion = (string.IsNullOrWhiteSpace(IEE117) != true && IEE117 != @"MN");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410049", Severity = ProcessMessageSeverity.Error,
                            Description = @"Organisation Name Type Code must be 'MN'",
                            Location = IEE117Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:OrganisationNameDetails.OrganisationalNameType.Code" : DeriveSbrElementLocation(IEE117Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410049"} },
                        };
                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPAddressDetailsCollection
        protected virtual void ValidateRPAddressDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, IEE2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;

                    SBRElement IEE104Value = null;
                    string IEE104 = null;

                    SBRElement IEE150Value = null;
                    string IEE150 = null;
                    SBRElement IEE121Value = null;
                    bool? IEE121 = null;
                    SBRElement IEE5Value = null;
                    string IEE5 = null;
                    SBRElement IEE122Value = null;
                    string IEE122 = null;
                    SBRElement IEE123Value = null;
                    string IEE123 = null;
                    SBRElement IEE124Value = null;
                    string IEE124 = null;
                    SBRElement IEE6Value = null;
                    string IEE6 = null;
                    SBRElement IEE8Value = null;
                    string IEE8 = null;
                    SBRElement IEE7Value = null;
                    string IEE7 = null;
                    SBRElement IEE9Value = null;
                    string IEE9 = null;
                    SBRElement IEE111Value = null;
                    string IEE111 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Usage.Code", out IEE104Value))
                    {
                        if (!IEE104Value.IsNil) IEE104 = IEE104Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Currency.Code", out IEE150Value))
                    {
                        if (!IEE150Value.IsNil) IEE150 = IEE150Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.OverseasAddress.Indicator", out IEE121Value))
                    {
                        if (!IEE121Value.IsNil) IEE121 = ToBoolean(IEE121Value.Value);
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line1.Text", out IEE5Value))
                    {
                        if (!IEE5Value.IsNil) IEE5 = IEE5Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line2.Text", out IEE122Value))
                    {
                        if (!IEE122Value.IsNil) IEE122 = IEE122Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line3.Text", out IEE123Value))
                    {
                        if (!IEE123Value.IsNil) IEE123 = IEE123Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line4.Text", out IEE124Value))
                    {
                        if (!IEE124Value.IsNil) IEE124 = IEE124Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.LocalityName.Text", out IEE6Value))
                    {
                        if (!IEE6Value.IsNil) IEE6 = IEE6Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Postcode.Text", out IEE8Value))
                    {
                        if (!IEE8Value.IsNil) IEE8 = IEE8Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.StateOrTerritory.Code", out IEE7Value))
                    {
                        if (!IEE7Value.IsNil) IEE7 = IEE7Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.CountryName.Text", out IEE9Value))
                    {
                        if (!IEE9Value.IsNil) IEE9 = IEE9Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Country.Code", out IEE111Value))
                    {
                        if (!IEE111Value.IsNil) IEE111 = IEE111Value.Value;
                    }
        
                    #region VR.ATO.GEN.410174

        /*  VR.ATO.GEN.410174
            Address usage code must be entered

        Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Usage.Code = NULLORBLANK) 
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            ^IEE104 = BLANK

        Data Elements:
    
            RP:^IEE104 = pyde.02.01:AddressDetails.Usage.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
                    assertion = (string.IsNullOrWhiteSpace(IEE104) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410174", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Usage Code must be present",
                            Location = IEE104Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:AddressDetails.Usage.Code" : DeriveSbrElementLocation(IEE104Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410174"} },
                        };
                        errors.Add(processMessage);
                    }
                    #endregion
        
                    #region VR.ATO.GEN.428202

        /*  VR.ATO.GEN.428202
            Currency Code must be 'C' or 'P'

        Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Currency.Code <> NULLORBLANK)
            AND (pyde.xx.xx:AddressDetails.Currency.Code <> SET("C","P"))
             RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^IEE150 <> BLANK) AND (NotInSet(^IEE150, '"C","P"'))

        Data Elements:
    
            RP:^IEE150 = pyde.02.00:AddressDetails.Currency.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
                    assertion = (string.IsNullOrWhiteSpace(IEE150) != true && !(IsMatch(IEE150, @"^(C|P)$",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.428202", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address currency code must be ""C"" or ""P""",
                            Location = IEE150Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:AddressDetails.Currency.Code" : DeriveSbrElementLocation(IEE150Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428202"} },
                        };
                        errors.Add(processMessage);
                    }
                    #endregion
        
                    #region VR.ATO.IEE.410106

        /*  VR.ATO.IEE.410106
            Address Usage code must be 'POS'

        Legacy Rule Format:
            NOT IN TUPLE(iee.0003.lodge.request.xx.xx:SpecifiedIndividual) 
            IF (pyde.xx.xx:AddressDetails.Usage.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Usage.Code <>"POS")
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^IEE104 <> BLANK) AND (^IEE104 <> 'POS')

        Data Elements:
    
            RP:^IEE104 = pyde.02.01:AddressDetails.Usage.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
                    assertion = (string.IsNullOrWhiteSpace(IEE104) != true && IEE104 != @"POS");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410106", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Usage Code must be ""POS""",
                            Location = DeriveSbrElementLocation(tupleElement) + "/tns:AddressDetails.Usage.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IEE.410106"} },
                        };
                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPOrganisationNameDetailsCollection1
        protected virtual void ValidateRPOrganisationNameDetailsCollection1(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, IEE2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                }
            }
        }
        #endregion

        #region ValidateRPAddressDetailsCollection1
        protected virtual void ValidateRPAddressDetailsCollection1(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, IEE2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                }
            }
        }
        #endregion

        #region ValidateRPTrusteeOrPartner_OrganisationNameDetailsCollection
        protected virtual void ValidateRPTrusteeOrPartner_OrganisationNameDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, IEE2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;

                    SBRElement IEE167Value = null;
                    string IEE167 = null;

                    SBRElement IEE166Value = null;
                    string IEE166 = null;
                    SBRElement IEE32Value = null;
                    string IEE32 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}OrganisationNameDetails.OrganisationalNameType.Code", out IEE167Value))
                    {
                        if (!IEE167Value.IsNil) IEE167 = IEE167Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}OrganisationNameDetails.Currency.Code", out IEE166Value))
                    {
                        if (!IEE166Value.IsNil) IEE166 = IEE166Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}OrganisationNameDetails.OrganisationalName.Text", out IEE32Value))
                    {
                        if (!IEE32Value.IsNil) IEE32 = IEE32Value.Value;
                    }
        
                    #region VR.ATO.GEN.410049

        /*  VR.ATO.GEN.410049

        Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalNameType.Code <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalNameType.Code <> "MN")
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^IEE167 <> BLANK) AND (^IEE167 <> 'MN')

        Data Elements:
    
            RP:^IEE167 = pyde.02.18:OrganisationNameDetails.OrganisationalNameType.Code WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
                    assertion = (string.IsNullOrWhiteSpace(IEE167) != true && IEE167 != @"MN");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410049", Severity = ProcessMessageSeverity.Error,
                            Description = @"Organisation Name Type Code must be 'MN'",
                            Location = IEE167Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:OrganisationNameDetails.OrganisationalNameType.Code" : DeriveSbrElementLocation(IEE167Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410049"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE167", Value = IEE167 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
        
                    #region VR.ATO.IEE.410142

        /*  VR.ATO.IEE.410142
            Trustee or partner Name Currency Code must be 'C'

        Legacy Rule Format:
            IN TUPLE(iee.0003.lodge.request.xx.xx:TrusteeOrPartner)
            IN TUPLE(orgname2.xx.xx:OrganisationNameDetails)
            IF (pyde.xx.xx:OrganisationNameDetails.Currency.Code <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.Currency.Code <> "C")
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^IEE166 <> BLANK) AND (^IEE166 <> 'C')

        Data Elements:
    
            RP:^IEE166 = pyde.02.00:OrganisationNameDetails.Currency.Code WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
                    assertion = (string.IsNullOrWhiteSpace(IEE166) != true && IEE166 != @"C");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410142", Severity = ProcessMessageSeverity.Error,
                            Description = @"Trustee or partner name Currency Code must be 'C'",
                            Location = IEE166Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:OrganisationNameDetails.Currency.Code" : DeriveSbrElementLocation(IEE166Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IEE.410142"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE166", Value = IEE166 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPTrusteeOrPartner_PersonNameDetailsCollection
        protected virtual void ValidateRPTrusteeOrPartner_PersonNameDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, IEE2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;

                    SBRElement IEE158Value = null;
                    string IEE158 = null;

                    SBRElement IEE176Value = null;
                    string IEE176 = null;

                    SBRElement IEE168Value = null;
                    string IEE168 = null;
                    SBRElement IEE27Value = null;
                    string IEE27 = null;
                    SBRElement IEE29Value = null;
                    string IEE29 = null;
                    SBRElement IEE28Value = null;
                    string IEE28 = null;
                    SBRElement IEE30Value = null;
                    string IEE30 = null;
                    SBRElement IEE31Value = null;
                    string IEE31 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.PersonNameType.Code", out IEE158Value))
                    {
                        if (!IEE158Value.IsNil) IEE158 = IEE158Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.Currency.Code", out IEE176Value))
                    {
                        if (!IEE176Value.IsNil) IEE176 = IEE176Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.Usage.Code", out IEE168Value))
                    {
                        if (!IEE168Value.IsNil) IEE168 = IEE168Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.Title.Text", out IEE27Value))
                    {
                        if (!IEE27Value.IsNil) IEE27 = IEE27Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.NameSuffix.Text", out IEE29Value))
                    {
                        if (!IEE29Value.IsNil) IEE29 = IEE29Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.FamilyName.Text", out IEE28Value))
                    {
                        if (!IEE28Value.IsNil) IEE28 = IEE28Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.GivenName.Text", out IEE30Value))
                    {
                        if (!IEE30Value.IsNil) IEE30 = IEE30Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.OtherGivenName.Text", out IEE31Value))
                    {
                        if (!IEE31Value.IsNil) IEE31 = IEE31Value.Value;
                    }
        
                    #region VR.ATO.IEE.410145

        /*  VR.ATO.IEE.410145
            Person NameType Code must be 'LGL'

        Legacy Rule Format:
            IN TUPLE(prsnstrcnm3.xx.xx:PersonNameDetails)
            IF (pyde.xx.xx:PersonNameDetails.PersonNameType.Code <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.PersonNameType.Code <> "LGL")
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^IEE158 <> BLANK) AND (^IEE158 <> 'LGL')

        Data Elements:
    
            RP:^IEE158 = pyde.02.00:PersonNameDetails.PersonNameType.Code WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            */
                    assertion = (string.IsNullOrWhiteSpace(IEE158) != true && IEE158 != @"LGL");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410145", Severity = ProcessMessageSeverity.Error,
                            Description = @"Person Name Type Code must be ""LGL""",
                            Location = IEE158Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:PersonNameDetails.PersonNameType.Code" : DeriveSbrElementLocation(IEE158Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IEE.410145"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE158", Value = IEE158 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
        
                    #region VR.ATO.IEE.410164

        /*  VR.ATO.IEE.410164
            Person Name Currency Code must be 'C'

        Legacy Rule Format:
            IN TUPLE(prsnstrcnm3.xx.xx:PersonNameDetails)
            IF (pyde.xx.xx:PersonNameDetails.Currency.Code <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.Currency.Code <> "C")
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^IEE176 <> BLANK) AND (^IEE176 <> 'C')

        Data Elements:
    
            RP:^IEE176 = pyde.02.00:PersonNameDetails.Currency.Code WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            */
                    assertion = (string.IsNullOrWhiteSpace(IEE176) != true && IEE176 != @"C");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410164", Severity = ProcessMessageSeverity.Error,
                            Description = @"Declaration name currency code must be ""C""",
                            Location = IEE176Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:PersonNameDetails.Currency.Code" : DeriveSbrElementLocation(IEE176Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IEE.410164"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE176", Value = IEE176 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
        
                    #region VR.ATO.IEE.410239

        /*  VR.ATO.IEE.410239
            Person name - Usage Code must be 'Identifier'

        Legacy Rule Format:
            IN TUPLE(iee.0003.lodge.request.xx.xx:TrusteeOrPartner)
            IN TUPLE(prsnstrcnm3.xx.xx:PersonNameDetails)
            IF (pyde.xx.xx:PersonNameDetails.Usage.Code <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.Usage.Code <> "Identifier")
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^IEE168 <> BLANK) AND (^IEE168 <> 'Identifier')

        Data Elements:
    
            RP:^IEE168 = pyde.02.37:PersonNameDetails.Usage.Code WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            */
                    assertion = (string.IsNullOrWhiteSpace(IEE168) != true && IEE168 != @"Identifier");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410239", Severity = ProcessMessageSeverity.Error,
                            Description = @"Person name - Usage Code must be 'Identifier'",
                            Location = IEE168Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:PersonNameDetails.Usage.Code" : DeriveSbrElementLocation(IEE168Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IEE.410239"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE168", Value = IEE168 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPTrusteeOrPartner_AddressDetailsCollection
        protected virtual void ValidateRPTrusteeOrPartner_AddressDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, IEE2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;

                    SBRElement IEE106Value = null;
                    string IEE106 = null;

                    SBRElement IEE130Value = null;
                    string IEE130 = null;
                    SBRElement IEE129Value = null;
                    bool? IEE129 = null;
                    SBRElement IEE35Value = null;
                    string IEE35 = null;
                    SBRElement IEE36Value = null;
                    string IEE36 = null;
                    SBRElement IEE131Value = null;
                    string IEE131 = null;
                    SBRElement IEE132Value = null;
                    string IEE132 = null;
                    SBRElement IEE37Value = null;
                    string IEE37 = null;
                    SBRElement IEE39Value = null;
                    string IEE39 = null;
                    SBRElement IEE38Value = null;
                    string IEE38 = null;
                    SBRElement IEE40Value = null;
                    string IEE40 = null;
                    SBRElement IEE113Value = null;
                    string IEE113 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Usage.Code", out IEE106Value))
                    {
                        if (!IEE106Value.IsNil) IEE106 = IEE106Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Currency.Code", out IEE130Value))
                    {
                        if (!IEE130Value.IsNil) IEE130 = IEE130Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.OverseasAddress.Indicator", out IEE129Value))
                    {
                        if (!IEE129Value.IsNil) IEE129 = ToBoolean(IEE129Value.Value);
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line1.Text", out IEE35Value))
                    {
                        if (!IEE35Value.IsNil) IEE35 = IEE35Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line2.Text", out IEE36Value))
                    {
                        if (!IEE36Value.IsNil) IEE36 = IEE36Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line3.Text", out IEE131Value))
                    {
                        if (!IEE131Value.IsNil) IEE131 = IEE131Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line4.Text", out IEE132Value))
                    {
                        if (!IEE132Value.IsNil) IEE132 = IEE132Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.LocalityName.Text", out IEE37Value))
                    {
                        if (!IEE37Value.IsNil) IEE37 = IEE37Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Postcode.Text", out IEE39Value))
                    {
                        if (!IEE39Value.IsNil) IEE39 = IEE39Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.StateOrTerritory.Code", out IEE38Value))
                    {
                        if (!IEE38Value.IsNil) IEE38 = IEE38Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.CountryName.Text", out IEE40Value))
                    {
                        if (!IEE40Value.IsNil) IEE40 = IEE40Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Country.Code", out IEE113Value))
                    {
                        if (!IEE113Value.IsNil) IEE113 = IEE113Value.Value;
                    }
        
                    #region VR.ATO.GEN.410174

        /*  VR.ATO.GEN.410174
            Address usage code must be entered

        Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Usage.Code = NULLORBLANK) 
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            ^IEE106 = BLANK

        Data Elements:
    
            RP:^IEE106 = pyde.02.01:AddressDetails.Usage.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
                    assertion = (string.IsNullOrWhiteSpace(IEE106) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410174", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Usage Code must be present",
                            Location = IEE106Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:AddressDetails.Usage.Code" : DeriveSbrElementLocation(IEE106Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410174"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE106", Value = IEE106 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
        
                    #region VR.ATO.GEN.428202

        /*  VR.ATO.GEN.428202
            Currency Code must be 'C' or 'P'

        Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Currency.Code <> NULLORBLANK)
            AND (pyde.xx.xx:AddressDetails.Currency.Code <> SET("C","P"))
             RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^IEE130 <> BLANK) AND (NotInSet(^IEE130, '"C","P"'))

        Data Elements:
    
            RP:^IEE130 = pyde.02.00:AddressDetails.Currency.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
                    assertion = (string.IsNullOrWhiteSpace(IEE130) != true && !(IsMatch(IEE130, @"^(C|P)$",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.428202", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address currency code must be ""C"" or ""P""",
                            Location = IEE130Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:AddressDetails.Currency.Code" : DeriveSbrElementLocation(IEE130Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428202"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE130", Value = IEE130 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPPeriodDetailsCollection2
        protected virtual void ValidateRPPeriodDetailsCollection2(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, IEE2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                }
            }
        }
        #endregion

        #region ValidateRPFamilyTrust_OrganisationNameDetailsCollection
        protected virtual void ValidateRPFamilyTrust_OrganisationNameDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, IEE2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;

                    SBRElement IEE119Value = null;
                    string IEE119 = null;

                    SBRElement IEE170Value = null;
                    string IEE170 = null;
                    SBRElement IEE52Value = null;
                    string IEE52 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}OrganisationNameDetails.OrganisationalNameType.Code", out IEE119Value))
                    {
                        if (!IEE119Value.IsNil) IEE119 = IEE119Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}OrganisationNameDetails.Currency.Code", out IEE170Value))
                    {
                        if (!IEE170Value.IsNil) IEE170 = IEE170Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}OrganisationNameDetails.OrganisationalName.Text", out IEE52Value))
                    {
                        if (!IEE52Value.IsNil) IEE52 = IEE52Value.Value;
                    }
        
                    #region VR.ATO.GEN.410049

        /*  VR.ATO.GEN.410049

        Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalNameType.Code <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalNameType.Code <> "MN")
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^IEE119 <> BLANK) AND (^IEE119 <> 'MN')

        Data Elements:
    
            RP:^IEE119 = pyde.02.18:OrganisationNameDetails.OrganisationalNameType.Code WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
                    assertion = (string.IsNullOrWhiteSpace(IEE119) != true && IEE119 != @"MN");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410049", Severity = ProcessMessageSeverity.Error,
                            Description = @"Organisation Name Type Code must be 'MN'",
                            Location = IEE119Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:OrganisationNameDetails.OrganisationalNameType.Code" : DeriveSbrElementLocation(IEE119Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410049"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE119", Value = IEE119 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
        
                    #region VR.ATO.IEE.410143

        /*  VR.ATO.IEE.410143
            Family trust Name Currency Code must be 'C'

        Legacy Rule Format:
            IN TUPLE(iee.0003.lodge.request.xx.xx:FamilyTrust)
            IN TUPLE(orgname2.xx.xx:OrganisationNameDetails)
            IF (pyde.xx.xx:OrganisationNameDetails.Currency.Code <> "C")
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            ^IEE170 <> 'C'

        Data Elements:
    
            RP:^IEE170 = pyde.02.00:OrganisationNameDetails.Currency.Code WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
                    assertion = (IEE170 != @"C");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410143", Severity = ProcessMessageSeverity.Error,
                            Description = @"Family trust name Currency Code must be 'C'",
                            Location = IEE170Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:OrganisationNameDetails.Currency.Code" : DeriveSbrElementLocation(IEE170Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IEE.410143"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE170", Value = IEE170 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPFamilyTrust_AddressDetailsCollection
        protected virtual void ValidateRPFamilyTrust_AddressDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, IEE2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;

                    SBRElement IEE107Value = null;
                    string IEE107 = null;

                    SBRElement IEE153Value = null;
                    string IEE153 = null;
                    SBRElement IEE133Value = null;
                    bool? IEE133 = null;
                    SBRElement IEE53Value = null;
                    string IEE53 = null;
                    SBRElement IEE134Value = null;
                    string IEE134 = null;
                    SBRElement IEE135Value = null;
                    string IEE135 = null;
                    SBRElement IEE136Value = null;
                    string IEE136 = null;
                    SBRElement IEE54Value = null;
                    string IEE54 = null;
                    SBRElement IEE56Value = null;
                    string IEE56 = null;
                    SBRElement IEE55Value = null;
                    string IEE55 = null;
                    SBRElement IEE57Value = null;
                    string IEE57 = null;
                    SBRElement IEE155Value = null;
                    string IEE155 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Usage.Code", out IEE107Value))
                    {
                        if (!IEE107Value.IsNil) IEE107 = IEE107Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Currency.Code", out IEE153Value))
                    {
                        if (!IEE153Value.IsNil) IEE153 = IEE153Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.OverseasAddress.Indicator", out IEE133Value))
                    {
                        if (!IEE133Value.IsNil) IEE133 = ToBoolean(IEE133Value.Value);
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line1.Text", out IEE53Value))
                    {
                        if (!IEE53Value.IsNil) IEE53 = IEE53Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line2.Text", out IEE134Value))
                    {
                        if (!IEE134Value.IsNil) IEE134 = IEE134Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line3.Text", out IEE135Value))
                    {
                        if (!IEE135Value.IsNil) IEE135 = IEE135Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line4.Text", out IEE136Value))
                    {
                        if (!IEE136Value.IsNil) IEE136 = IEE136Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.LocalityName.Text", out IEE54Value))
                    {
                        if (!IEE54Value.IsNil) IEE54 = IEE54Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Postcode.Text", out IEE56Value))
                    {
                        if (!IEE56Value.IsNil) IEE56 = IEE56Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.StateOrTerritory.Code", out IEE55Value))
                    {
                        if (!IEE55Value.IsNil) IEE55 = IEE55Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.CountryName.Text", out IEE57Value))
                    {
                        if (!IEE57Value.IsNil) IEE57 = IEE57Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Country.Code", out IEE155Value))
                    {
                        if (!IEE155Value.IsNil) IEE155 = IEE155Value.Value;
                    }
        
                    #region VR.ATO.GEN.410174

        /*  VR.ATO.GEN.410174
            Address usage code must be entered

        Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Usage.Code = NULLORBLANK) 
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            ^IEE107 = BLANK

        Data Elements:
    
            RP:^IEE107 = pyde.02.01:AddressDetails.Usage.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
                    assertion = (string.IsNullOrWhiteSpace(IEE107) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410174", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Usage Code must be present",
                            Location = IEE107Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:AddressDetails.Usage.Code" : DeriveSbrElementLocation(IEE107Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410174"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE107", Value = IEE107 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
        
                    #region VR.ATO.GEN.428202

        /*  VR.ATO.GEN.428202
            Currency Code must be 'C' or 'P'

        Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Currency.Code <> NULLORBLANK)
            AND (pyde.xx.xx:AddressDetails.Currency.Code <> SET("C","P"))
             RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^IEE153 <> BLANK) AND (NotInSet(^IEE153, '"C","P"'))

        Data Elements:
    
            RP:^IEE153 = pyde.02.00:AddressDetails.Currency.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
                    assertion = (string.IsNullOrWhiteSpace(IEE153) != true && !(IsMatch(IEE153, @"^(C|P)$",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.428202", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address currency code must be ""C"" or ""P""",
                            Location = IEE153Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:AddressDetails.Currency.Code" : DeriveSbrElementLocation(IEE153Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428202"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE153", Value = IEE153 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPFamilyTrust_PeriodDetailsCollection
        protected virtual void ValidateRPFamilyTrust_PeriodDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, IEE2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                }
            }
        }
        #endregion

        #region ValidateRPSpecifiedIndividual_PersonNameDetailsCollection
        protected virtual void ValidateRPSpecifiedIndividual_PersonNameDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, IEE2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;

                    SBRElement IEE171Value = null;
                    string IEE171 = null;

                    SBRElement IEE177Value = null;
                    string IEE177 = null;

                    SBRElement IEE172Value = null;
                    string IEE172 = null;
                    SBRElement IEE65Value = null;
                    string IEE65 = null;
                    SBRElement IEE67Value = null;
                    string IEE67 = null;
                    SBRElement IEE66Value = null;
                    string IEE66 = null;
                    SBRElement IEE68Value = null;
                    string IEE68 = null;
                    SBRElement IEE69Value = null;
                    string IEE69 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.PersonNameType.Code", out IEE171Value))
                    {
                        if (!IEE171Value.IsNil) IEE171 = IEE171Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.Currency.Code", out IEE177Value))
                    {
                        if (!IEE177Value.IsNil) IEE177 = IEE177Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.Usage.Code", out IEE172Value))
                    {
                        if (!IEE172Value.IsNil) IEE172 = IEE172Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.Title.Text", out IEE65Value))
                    {
                        if (!IEE65Value.IsNil) IEE65 = IEE65Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.NameSuffix.Text", out IEE67Value))
                    {
                        if (!IEE67Value.IsNil) IEE67 = IEE67Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.FamilyName.Text", out IEE66Value))
                    {
                        if (!IEE66Value.IsNil) IEE66 = IEE66Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.GivenName.Text", out IEE68Value))
                    {
                        if (!IEE68Value.IsNil) IEE68 = IEE68Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.OtherGivenName.Text", out IEE69Value))
                    {
                        if (!IEE69Value.IsNil) IEE69 = IEE69Value.Value;
                    }
        
                    #region VR.ATO.IEE.410145

        /*  VR.ATO.IEE.410145
            Person NameType Code must be 'LGL'

        Legacy Rule Format:
            IN TUPLE(prsnstrcnm3.xx.xx:PersonNameDetails)
            IF (pyde.xx.xx:PersonNameDetails.PersonNameType.Code <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.PersonNameType.Code <> "LGL")
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^IEE171 <> BLANK) AND (^IEE171 <> 'LGL')

        Data Elements:
    
            RP:^IEE171 = pyde.02.00:PersonNameDetails.PersonNameType.Code WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            */
                    assertion = (string.IsNullOrWhiteSpace(IEE171) != true && IEE171 != @"LGL");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410145", Severity = ProcessMessageSeverity.Error,
                            Description = @"Person Name Type Code must be ""LGL""",
                            Location = IEE171Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:PersonNameDetails.PersonNameType.Code" : DeriveSbrElementLocation(IEE171Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IEE.410145"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE171", Value = IEE171 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
        
                    #region VR.ATO.IEE.410164

        /*  VR.ATO.IEE.410164
            Person Name Currency Code must be 'C'

        Legacy Rule Format:
            IN TUPLE(prsnstrcnm3.xx.xx:PersonNameDetails)
            IF (pyde.xx.xx:PersonNameDetails.Currency.Code <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.Currency.Code <> "C")
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^IEE177 <> BLANK) AND (^IEE177 <> 'C')

        Data Elements:
    
            RP:^IEE177 = pyde.02.00:PersonNameDetails.Currency.Code WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            */
                    assertion = (string.IsNullOrWhiteSpace(IEE177) != true && IEE177 != @"C");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410164", Severity = ProcessMessageSeverity.Error,
                            Description = @"Declaration name currency code must be ""C""",
                            Location = IEE177Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:PersonNameDetails.Currency.Code" : DeriveSbrElementLocation(IEE177Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IEE.410164"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE177", Value = IEE177 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
        
                    #region VR.ATO.IEE.410240

        /*  VR.ATO.IEE.410240
            Person name - Usage Code must be 'Identifier'

        Legacy Rule Format:
            IN TUPLE(iee.0003.lodge.request.xx.xx:SpecifiedIndividual)
            IN TUPLE(prsnstrcnm3.xx.xx:PersonNameDetails)
            IF (pyde.xx.xx:PersonNameDetails.Usage.Code <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.Usage.Code <> "Identifier")
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^IEE172 <> BLANK) AND (^IEE172 <> 'Identifier')

        Data Elements:
    
            RP:^IEE172 = pyde.02.37:PersonNameDetails.Usage.Code WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Identifier'].[PersonNameDetails.Currency.Code='C']
            */
                    assertion = (string.IsNullOrWhiteSpace(IEE172) != true && IEE172 != @"Identifier");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410239", Severity = ProcessMessageSeverity.Error,
                            Description = @"Person name - Usage Code must be 'Identifier'",
                            Location = IEE172Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:PersonNameDetails.Usage.Code" : DeriveSbrElementLocation(IEE172Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IEE.410240"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE172", Value = IEE172 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPSpecifiedIndividual_AddressDetailsCollection
        protected virtual void ValidateRPSpecifiedIndividual_AddressDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, IEE2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;

                    SBRElement IEE108Value = null;
                    string IEE108 = null;

                    SBRElement IEE152Value = null;
                    string IEE152 = null;
                    SBRElement IEE137Value = null;
                    bool? IEE137 = null;
                    SBRElement IEE70Value = null;
                    string IEE70 = null;
                    SBRElement IEE138Value = null;
                    string IEE138 = null;
                    SBRElement IEE139Value = null;
                    string IEE139 = null;
                    SBRElement IEE140Value = null;
                    string IEE140 = null;
                    SBRElement IEE71Value = null;
                    string IEE71 = null;
                    SBRElement IEE73Value = null;
                    string IEE73 = null;
                    SBRElement IEE72Value = null;
                    string IEE72 = null;
                    SBRElement IEE74Value = null;
                    string IEE74 = null;
                    SBRElement IEE114Value = null;
                    string IEE114 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Usage.Code", out IEE108Value))
                    {
                        if (!IEE108Value.IsNil) IEE108 = IEE108Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Currency.Code", out IEE152Value))
                    {
                        if (!IEE152Value.IsNil) IEE152 = IEE152Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.OverseasAddress.Indicator", out IEE137Value))
                    {
                        if (!IEE137Value.IsNil) IEE137 = ToBoolean(IEE137Value.Value);
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line1.Text", out IEE70Value))
                    {
                        if (!IEE70Value.IsNil) IEE70 = IEE70Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line2.Text", out IEE138Value))
                    {
                        if (!IEE138Value.IsNil) IEE138 = IEE138Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line3.Text", out IEE139Value))
                    {
                        if (!IEE139Value.IsNil) IEE139 = IEE139Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Line4.Text", out IEE140Value))
                    {
                        if (!IEE140Value.IsNil) IEE140 = IEE140Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.LocalityName.Text", out IEE71Value))
                    {
                        if (!IEE71Value.IsNil) IEE71 = IEE71Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Postcode.Text", out IEE73Value))
                    {
                        if (!IEE73Value.IsNil) IEE73 = IEE73Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.StateOrTerritory.Code", out IEE72Value))
                    {
                        if (!IEE72Value.IsNil) IEE72 = IEE72Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.CountryName.Text", out IEE74Value))
                    {
                        if (!IEE74Value.IsNil) IEE74 = IEE74Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}AddressDetails.Country.Code", out IEE114Value))
                    {
                        if (!IEE114Value.IsNil) IEE114 = IEE114Value.Value;
                    }
        
                    #region VR.ATO.GEN.410174

        /*  VR.ATO.GEN.410174
            Address usage code must be entered

        Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Usage.Code = NULLORBLANK) 
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            ^IEE108 = BLANK

        Data Elements:
    
            RP:^IEE108 = pyde.02.01:AddressDetails.Usage.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
                    assertion = (string.IsNullOrWhiteSpace(IEE108) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410174", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Usage Code must be present",
                            Location = IEE108Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:AddressDetails.Usage.Code" : DeriveSbrElementLocation(IEE108Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410174"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE108", Value = IEE108 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
        
                    #region VR.ATO.GEN.428202

        /*  VR.ATO.GEN.428202
            Currency Code must be 'C' or 'P'

        Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Currency.Code <> NULLORBLANK)
            AND (pyde.xx.xx:AddressDetails.Currency.Code <> SET("C","P"))
             RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^IEE152 <> BLANK) AND (NotInSet(^IEE152, '"C","P"'))

        Data Elements:
    
            RP:^IEE152 = pyde.02.00:AddressDetails.Currency.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
                    assertion = (string.IsNullOrWhiteSpace(IEE152) != true && !(IsMatch(IEE152, @"^(C|P)$",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.428202", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address currency code must be ""C"" or ""P""",
                            Location = IEE152Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:AddressDetails.Currency.Code" : DeriveSbrElementLocation(IEE152Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428202"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE152", Value = IEE152 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
        
                    #region VR.ATO.IEE.410110

        /*  VR.ATO.IEE.410110
            The Specified Individual Address Usage code must be 'RES'

        Legacy Rule Format:
            IN TUPLE(iee.0003.lodge.request.xx.xx:SpecifiedIndividual) 
            IF (pyde.xx.xx:AddressDetails.Usage.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Usage.Code <> "RES")
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^IEE108 <> BLANK) AND (^IEE108 <> 'RES')

        Data Elements:
    
            RP:^IEE108 = pyde.02.01:AddressDetails.Usage.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            */
                    assertion = (string.IsNullOrWhiteSpace(IEE108) != true && IEE108 != @"RES");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410110", Severity = ProcessMessageSeverity.Error,
                            Description = @"Specified Individual address usage code must be ""RES""",
                            Location = IEE108Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:AddressDetails.Usage.Code" : DeriveSbrElementLocation(IEE108Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IEE.410110"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE108", Value = IEE108 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection
        protected virtual void ValidateRPDeclarationOfTrusteeOrPartner_PersonNameDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, IEE2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                }
            }
        }
        #endregion

        #region ValidateRPDeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection
        protected virtual void ValidateRPDeclarationOfTrusteeOrPartner_OrganisationNameDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, IEE2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                }
            }
        }
        #endregion

        #region ValidateRPDeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection
        protected virtual void ValidateRPDeclarationOfTrusteeOrPartner_Declaration_PersonUnstructuredNameCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, IEE2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                }
            }
        }
        #endregion

        #region ValidateRPPersonNameDetailsCollection
        protected virtual void ValidateRPPersonNameDetailsCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, IEE2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;

                    SBRElement IEE174Value = null;
                    string IEE174 = null;

                    SBRElement IEE175Value = null;
                    string IEE175 = null;

                    SBRElement IEE157Value = null;
                    string IEE157 = null;
                    SBRElement IEE83Value = null;
                    string IEE83 = null;
                    SBRElement IEE85Value = null;
                    string IEE85 = null;
                    SBRElement IEE84Value = null;
                    string IEE84 = null;
                    SBRElement IEE86Value = null;
                    string IEE86 = null;
                    SBRElement IEE87Value = null;
                    string IEE87 = null;

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.PersonNameType.Code", out IEE174Value))
                    {
                        if (!IEE174Value.IsNil) IEE174 = IEE174Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.Currency.Code", out IEE175Value))
                    {
                        if (!IEE175Value.IsNil) IEE175 = IEE175Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.Usage.Code", out IEE157Value))
                    {
                        if (!IEE157Value.IsNil) IEE157 = IEE157Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.Title.Text", out IEE83Value))
                    {
                        if (!IEE83Value.IsNil) IEE83 = IEE83Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.NameSuffix.Text", out IEE85Value))
                    {
                        if (!IEE85Value.IsNil) IEE85 = IEE85Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.FamilyName.Text", out IEE84Value))
                    {
                        if (!IEE84Value.IsNil) IEE84 = IEE84Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.GivenName.Text", out IEE86Value))
                    {
                        if (!IEE86Value.IsNil) IEE86 = IEE86Value.Value;
                    }

                    if (tupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/iee}PersonNameDetails.OtherGivenName.Text", out IEE87Value))
                    {
                        if (!IEE87Value.IsNil) IEE87 = IEE87Value.Value;
                    }
        
                    #region VR.ATO.IEE.410145

        /*  VR.ATO.IEE.410145
            Person NameType Code must be 'LGL'

        Legacy Rule Format:
            IN TUPLE(prsnstrcnm3.xx.xx:PersonNameDetails)
            IF (pyde.xx.xx:PersonNameDetails.PersonNameType.Code <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.PersonNameType.Code <> "LGL")
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^IEE174 <> BLANK) AND (^IEE174 <> 'LGL')

        Data Elements:
    
            RP:^IEE174 = pyde.02.00:PersonNameDetails.PersonNameType.Code WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            */
                    assertion = (string.IsNullOrWhiteSpace(IEE174) != true && IEE174 != @"LGL");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410145", Severity = ProcessMessageSeverity.Error,
                            Description = @"Person Name Type Code must be ""LGL""",
                            Location = IEE174Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:PersonNameDetails.PersonNameType.Code" : DeriveSbrElementLocation(IEE174Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IEE.410145"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE174", Value = IEE174 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
        
                    #region VR.ATO.IEE.410164

        /*  VR.ATO.IEE.410164
            Person Name Currency Code must be 'C'

        Legacy Rule Format:
            IN TUPLE(prsnstrcnm3.xx.xx:PersonNameDetails)
            IF (pyde.xx.xx:PersonNameDetails.Currency.Code <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.Currency.Code <> "C")
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^IEE175 <> BLANK) AND (^IEE175 <> 'C')

        Data Elements:
    
            RP:^IEE175 = pyde.02.00:PersonNameDetails.Currency.Code WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            */
                    assertion = (string.IsNullOrWhiteSpace(IEE175) != true && IEE175 != @"C");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410164", Severity = ProcessMessageSeverity.Error,
                            Description = @"Declaration name currency code must be ""C""",
                            Location = IEE175Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:PersonNameDetails.Currency.Code" : DeriveSbrElementLocation(IEE175Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IEE.410164"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE175", Value = IEE175 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
        
                    #region VR.ATO.IEE.410241

        /*  VR.ATO.IEE.410241
            Person name - Usage Code must be 'DeclarationSignatory'

        Legacy Rule Format:
            WHERE IN TUPLE(prsnstrcnm3.xx.xx:PersonNameDetails)
            IF (pyde.xx.xx:PersonNameDetails.Usage.Code <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.Usage.Code <> "DeclarationSignatory")
               RETURN VALIDATION MESSAGE
            ENDIF
    
        Technical Business Rule Format:
            (^IEE157 <> BLANK) AND (^IEE157 <> 'DeclarationSignatory')

        Data Elements:
    
            RP:^IEE157 = pyde.02.37:PersonNameDetails.Usage.Code WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='DeclarationSignatory'].[PersonNameDetails.Currency.Code='C']
            */
                    assertion = (string.IsNullOrWhiteSpace(IEE157) != true && IEE157 != @"DeclarationSignatory");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IEE.410125", Severity = ProcessMessageSeverity.Error,
                            Description = @"Person name - Usage Code must be 'DeclarationSignatory'",
                            Location = IEE157Value == null ? DeriveSbrElementLocation(tupleElement, true) + "/tns:PersonNameDetails.Usage.Code" : DeriveSbrElementLocation(IEE157Value),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IEE.410241"} },
                        };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IEE157", Value = IEE157 });
        
                        errors.Add(processMessage);
                    }
                    #endregion
                }
            }
        }
        #endregion

        #region ValidateRPDeclaration_PersonUnstructuredNameCollection
        protected virtual void ValidateRPDeclaration_PersonUnstructuredNameCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, IEE2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                }
            }
        }
        #endregion

        #region ValidateRPDeclarationCollection
        protected virtual void ValidateRPDeclarationCollection(List<ProcessMessageDocument> errors, IEnumerable<SBRElement> tupleCollection, IEnumerable<SBRContext> contexts, IEE2018 report)
        {
            bool assertion;
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            HashSet<string> alreadyChecked = new HashSet<string>();

            if (tupleCollection != null)
            {
                foreach (SBRElement tupleElement in tupleCollection)
                {
                    //Only check this for the RP Context exit out otherwise
                     if (tupleElement.ChildElements == null || tupleElement.ChildElements.Count == 0 || contexts == null || !contexts.Contains(tupleElement.ChildElements[0].Context))
                        continue;

                    IDictionary<string, SBRElement> tupleElementMap = tupleElement.PopulatedChildElementsMap;
                }
            }
        }
        #endregion

    } // Of Class
} // Of Namespace

